@@ -3,19 +3,19 @@
 %%
 %%     Utility functions for E3D meshes, such as cleanup and triangulation.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_mesh.erl,v 1.45 2004/11/14 12:36:36 bjorng Exp $
+%%     $Id: e3d_mesh.erl,v 1.48 2005/02/03 07:30:33 bjorng Exp $
 %%
 
 -module(e3d_mesh).
 -export([clean_faces/1,orient_normals/1,transform/1,transform/2,
 	 merge_vertices/1,triangulate/1,quadrangulate/1,
 	 make_quads/1,vertex_normals/1,renumber/1,partition/1,
-	 used_materials/1]).
+	 split_by_material/1,used_materials/1]).
 -export([triangulate_face/2,triangulate_face/3,
 	 triangulate_face_with_holes/3]).
 -export([quadrangulate_face/2,quadrangulate_face_with_holes/3]).
@@ -155,9 +155,11 @@ vertex_normals(#e3d_mesh{fs=Ftab,vs=Vtab0,he=He}=Mesh) ->
     Mesh#e3d_mesh{fs=Faces,ns=Normals}.
 
 %% renumber(Mesh0) -> Mesh
-%%  Removes vertices and UV coordinates that are not referenced
-%%  from any faces and renumbers vertices and UV coordinates to
-%%  remove the gaps.
+%%  Removes vertices and vertex attributes such as UV coordinates and
+%%  vertex normals that are not referenced from any faces and renumbers
+%%  vertices and attributes to remove the gaps.
+%%  XXX Vertex colors are not renumbered yet, meaning that there can
+%%  remain unused vertex colors.
 renumber(#e3d_mesh{tx=Tx,vs=Vtab,ns=Ns}=Mesh) ->
     {UsedVs,UsedUv,UsedNs} = rn_used_vs(Mesh),
     if
@@ -181,6 +183,18 @@ partition(#e3d_mesh{fs=Faces0,he=He0}=Template) ->
 		end, [], sort(FacePart)),
     reverse(Res).
 
+%% split_by_material(Mesh0) -> [Mesh]
+%%  Split a mesh into separate meshes where all faces in each
+%%  mesh have the same material.
+split_by_material(#e3d_mesh{fs=Fs}=Mesh) ->
+    R0 = foldl(fun(#e3d_face{mat=Mat}=Face, A) ->
+		       [{Mat,Face}|A]
+	       end, [], Fs),
+    R = sofs:relation(R0, [{mat,face}]),
+    F = sofs:relation_to_family(R),
+    Ps = sofs:to_external(sofs:range(F)),
+    [renumber(Mesh#e3d_mesh{fs=P}) || P <- Ps].
+
 %% used_materials(Mesh) -> [MaterialName]
 %%  Returns a sorted list of all materials used in the given mesh.
 used_materials(#e3d_mesh{fs=Fs0}) ->
@@ -863,14 +877,12 @@ slit_hard_x_cut(_Old=#e3d_mesh{vs=VsT,vc=VcT,tx=TxT,ns=NsT},
     Pos = e3d_vec:average([element(VsJ+1, VsT),element(VsK+1, VsT)]),
     Color = case {VcJ,VcK} of
 		{[Vcj],[Vck]} ->
-		    [wings_color:mix(
-		       0.5, element(Vcj+1, VcT), element(Vck+1, VcT))];
+		    [mix(0.5, element(Vcj+1, VcT), element(Vck+1, VcT))];
 		_ -> []
 	    end,
     UV = case {TxJ,TxK} of
 	     {[Txj],[Txk]} ->
-		 [wings_color:mix(
-		    0.5, element(Txj+1, TxT), element(Txk+1, TxT))];
+		 [mix(0.5, element(Txj+1, TxT), element(Txk+1, TxT))];
 	     _ -> []
 	 end,
     Norm = case {NsJ,NsK} of
@@ -903,6 +915,22 @@ slit_hard_x_dup(_Old=#e3d_mesh{vs=VsT},
 mk_edge(V1, V2) when V1 > V2 -> {V2,V1};
 mk_edge(V1, V2) -> {V1,V2}.
 
+mix(_W, Same, Same) -> Same;
+mix(Wa, {Ua,Va}, {Ub,Vb}) when is_float(Wa) ->
+    Wb = 1.0 - Wa,
+    {Wa*Ua+Wb*Ub,Wa*Va+Wb*Vb};
+mix(Wa, {Ra,Ga,Ba}, {Rb,Gb,Bb}) when is_float(Wa) ->
+    Wb = 1.0 - Wa,
+    {Wa*Ra+Wb*Rb,Wa*Ga+Wb*Gb,Wa*Ba+Wb*Bb};
+mix(Wa, {Ra,Ga,Ba}, {_,_,_,Ab}=B) ->
+    mix(Wa, {Ra,Ga,Ba,Ab}, B);
+mix(Wa, {_,_,_,Aa}=A, {Rb,Gb,Bb}) ->
+    mix(Wa, A, {Rb,Gb,Bb,Aa});
+mix(Wa, {Ra,Ga,Ba,Aa}, {Rb,Gb,Bb,Ab}) when is_float(Wa) ->
+    Wb = 1.0 - Wa,
+    {Wa*Ra+Wb*Rb,Wa*Ga+Wb*Gb,Wa*Ba+Wb*Bb,Wa*Aa+Wb*Ab};
+mix(_, _, _) -> none.
+
 %%%
 %%% Help function for face_areas/1,2.
 
