@@ -1,14 +1,15 @@
-%% File    : auv_seg_ui.erl
-%% Author  : Dan Gudmundsson <dgud@erix.ericsson.se>
-%% Description : The segmentation interface.
 %%
-%% Created : 24 Jan 2002 by Dan Gudmundsson <dgud@erix.ericsson.se>
-%%-------------------------------------------------------------------
-%%  Copyright (c) 2002-2004 Bjorn Gustavsson, Dan Gudmundsson
+%%  auv_seg_ui.erl --
+%%
+%%     Segmentation UI for AutoUV.
+%%
+%%  Copyright (c) 2002-2004 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-%%     $Id: auv_seg_ui.erl,v 1.25 2004/08/17 10:34:36 dgud Exp $
+%%
+%%     $Id: auv_seg_ui.erl,v 1.32 2005/01/09 09:30:36 bjorng Exp $
+%%
 
 -module(auv_seg_ui).
 -export([start/3]).
@@ -49,10 +50,10 @@ start(#we{id=Id}=We0, OrigWe, St0) ->
     get_seg_event(Ss).
 
 seg_init_message(Ss) ->
-    Msg1 = wings_util:button_format("Select"),
+    Msg1 = wings_msg:button_format("Select"),
     Msg2 = wings_camera:help(),
-    Msg3 = wings_util:button_format([], [], "Show menu"),
-    Msg = wings_util:join_msg([Msg1,Msg2,Msg3]),
+    Msg3 = wings_msg:button_format([], [], "Show menu"),
+    Msg = wings_msg:join([Msg1,Msg2,Msg3]),
     Ss#seg{msg=Msg}.
 
 get_seg_event(#seg{st=St}=Ss) ->
@@ -64,9 +65,8 @@ get_seg_event(#seg{st=St}=Ss) ->
 get_seg_event_noredraw(Ss) ->
     {replace,fun(Ev) -> seg_event(Ev, Ss) end}.
 
-seg_event({crash,_}=Crash, _) ->
-    LogName = wings_util:crash_log(wings_wm:this(), Crash),
-    wings_wm:send(geom, {crash_in_other_window,LogName}),
+seg_event({crash,Crash}, _) ->
+    wings_u:win_crash(Crash),
     delete;
 seg_event(init_opengl, #seg{st=St}=Ss) ->
     wings:init_opengl(St),
@@ -188,7 +188,7 @@ seg_event_6({action,{auv_segmentation,Cmd}}, Ss) ->
 seg_event_6({callback, Fun}, _) when function(Fun) ->
     Fun();
 seg_event_6({message,Message}, _) ->
-    wings_util:message(Message);
+    wings_u:message(Message);
 seg_event_6(#mousemotion{}, _) -> keep;
 seg_event_6(#mousebutton{}, _) -> keep;
 seg_event_6(#keyboard{}, _) -> keep;
@@ -209,10 +209,10 @@ filter_sel_command(#seg{selmodes=Modes}=Ss, #st{selmode=Mode}=St) ->
 seg_command({continue,Method}, Ss) ->
     seg_map_charts(Method, Ss);
 seg_command(cut_edges, #seg{st=St0}=Ss) ->
-    St = wings_edge:hardness(hard, St0),
+    St = wings_edge_cmd:hardness(hard, St0),
     get_seg_event(Ss#seg{st=St});
 seg_command(no_cut_edges, #seg{st=St0}=Ss) ->
-    St = wings_edge:hardness(soft, St0),
+    St = wings_edge_cmd:hardness(soft, St0),
     get_seg_event(Ss#seg{st=St});
 seg_command(select_hard_edges, #seg{st=St0}=Ss) ->
     case wings_sel_cmd:command({by,hard_edges}, St0) of
@@ -276,38 +276,39 @@ seg_create_materials(St0) ->
     {St,[]} = wings_material:add_materials(M, St0),
     St.
 
-seg_hide_other(Id, #st{selmode=face,sel=[{Id,Faces}],shapes=Shs}=St0) ->
+seg_hide_other(Id, #st{selmode=face,sel=[{Id,Faces}],shapes=Shs}=St) ->
     We0 = gb_trees:get(Id, Shs),
     Other = wings_sel:inverse_items(face, Faces, We0),
-    %%We = wings_we:hide_faces(Other, We0),
-    We = We0,
-    St = St0#st{sel=[{Id,Other}],shapes=gb_trees:update(Id, We, Shs)},
-    wings_material:command({assign,atom_to_list(?HOLE)}, St);
+    We = wings_we:hide_faces(Other, We0),
+    wings_sel:clear(St#st{shapes=gb_trees:update(Id, We, Shs)});
 seg_hide_other(_, St) -> St.    
 
 seg_map_charts(Method, #seg{st=#st{shapes=Shs},we=OrigWe}=Ss) ->
     wings_pb:start("preparing mapping"),
-    [We0] = gb_trees:values(Shs),
-    #we{he=Cuts0} = We1 = wings_we:show_faces(We0),
+    [#we{he=Cuts0}=We] = gb_trees:values(Shs),
     wings_pb:update(0.12, "segmenting"),
-    Charts0 = (catch auv_segment:segment_by_material(We1)),
+    Charts0 = auv_segment:segment_by_material(We),
     wings_pb:update(0.35, "normalizing"),
-    {Charts1,Cuts} = auv_segment:normalize_charts(Charts0, Cuts0, We1),
+    {Charts1,Cuts} = auv_segment:normalize_charts(Charts0, Cuts0, We),
     wings_pb:update(1.0, "cutting"),
     Charts = auv_segment:cut_model(Charts1, Cuts, OrigWe),
     wings_pb:done(),
-    N = length(Charts),
-    wings_pb:start("mapping"),
-    seg_map_charts_1(Charts, Method, 1, N, [], Ss).
+    case length(Charts) of
+	0 ->
+	    wings_u:error("No mappable faces.");
+	N ->
+	    wings_pb:start("mapping"),
+	    seg_map_charts_1(Charts, Method, 1, N, [], Ss)
+    end.
 
-seg_map_charts_1([{Fs,We0}|Cs], Type, Id, N, Acc,
+seg_map_charts_1([We0|Cs], Type, Id, N, Acc,
 		 #seg{we=#we{id=OrigId},st=St0}=Ss) ->
     wings_pb:update(Id/N, lists:flatten(io_lib:format("chart ~w/~w", [Id,N]))),
-    We1 = auv_segment:finalize_chart(Fs, We0#we{id=Id}),
+    We1 = We0#we{id=Id},
     case auv_mapping:map_chart(Type, We1, none) of
 	{error,Message} ->
 	    wings_pb:done(),
-	    wings_util:message(Message),
+	    wings_u:message(Message),
 	    Fs = wings_we:visible(We1),
 	    St = St0#st{selmode=face,sel=[{OrigId,gb_sets:from_ordset(Fs)}]},
 	    get_seg_event(seg_init_message(Ss#seg{st=St}));
