@@ -5,18 +5,17 @@
 %%%               Segments Model into set of charts containg faces.
 %%% Created :  3 Oct 2002 by Dan Gudmundsson <dgud@erix.ericsson.se>
 %%%-------------------------------------------------------------------
-%%  Copyright (c) 2001-2004 Dan Gudmundsson, Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-%%     $Id: auv_segment.erl,v 1.59 2004/07/31 18:46:15 dgud Exp $
+%%     $Id: auv_segment.erl,v 1.76 2005/01/29 18:25:13 bjorng Exp $
 
 -module(auv_segment).
 
 -export([create/2,segment_by_material/1,cut_model/3,
 	 normalize_charts/3,map_vertex/2,map_edge/2,
-	 fv_to_uv_map/2,uv_to_charts/3,
-	 finalize_charts/1,finalize_chart/2]).
+	 fv_to_uv_map/1,uv_to_charts/3]).
 
 -ifdef(DEBUG).
 -export([degrees/0, find_features/3, build_seeds/2]). %% Debugging
@@ -632,10 +631,11 @@ find_delete(Face,Level,Dist0,Next0,Fs0,Fg) ->
 
 segment_by_direction(We) ->
     Rel = foldl(fun({_,'_hole_'}, A) -> A;
+		   ({_,?HOLE}, A) -> A;
 		   ({Face,_}, A) ->
 			N = wings_face:normal(Face, We),
 			[{seg_dir(N),Face}|A]
-		end, [], wings_material:get_all(We)),
+		end, [], wings_facemat:all(We)),
     segment_by_cluster(Rel, We).
 
 seg_dir({X,Y,Z}) ->
@@ -652,9 +652,9 @@ seg_dir({X,Y,Z}) ->
 %% By Color 
 segment_by_material(We) ->
     Rel = foldl(fun({_,'_hole_'}, A) -> A;
-		   ({_, ?HOLE}, A) -> A;
+		   ({_,?HOLE}, A) -> A;
 		   ({Face,Name}, A) -> [{Name,Face}|A]
-		end, [], wings_material:get_all(We)),
+		end, [], wings_facemat:all(We)),
     segment_by_cluster(Rel, We).
 
 %% segment_by_cluster([{Key,Face}], We)
@@ -694,47 +694,61 @@ map_edge(E0, Emap) ->
 %%% Cutting along hard edges.
 %%%
 
-cut_model(Charts, Cuts0, We) ->
+cut_model(Charts, Cuts0, #we{es=Etab0}=We0) ->
+    Etab = wings_util:gb_trees_map(fun(_, Rec) ->
+					   Rec#edge{a=none,b=none}
+				   end, Etab0),
+    We = We0#we{mode=material,es=Etab},
     Cuts = gb_sets:to_list(Cuts0),
-    map(fun(Keep) ->
-		cut_one_chart(Keep, Cuts, We#we{mirror=none})
-	end, Charts).
+    cut_model_1(Charts, Cuts, We#we{mirror=none}, length(Charts), []).
+
+cut_model_1([Fs|Cs], Cuts, OrigWe, Id, Acc) ->
+    We = cut_one_chart(Fs, Cuts, OrigWe#we{id=Id}),
+    cut_model_1(Cs, Cuts, OrigWe, Id-1, [We|Acc]);
+cut_model_1([], _, _, _, Acc) -> Acc.
 
 cut_one_chart(Keep0, Cuts, We0) ->
+    {InnerEdges,OuterEdges} = wings_face:inner_outer_edges(Keep0, We0),
     Keep = gb_sets:from_list(Keep0),
-    OuterEdges = wings_face:outer_edges(Keep, We0),
     Map0 = gb_trees:empty(),
     {We1,Map1} = cut_shared_vertices(Keep, OuterEdges, We0, Map0),
-    {We2,Vmap} = cut_edges(Keep0, Cuts, We1, Map1),
-    Me = gb_sets:to_list(wings_we:new_items(edge, We1, We2)),
-    
-    %% Testing
-    Emap = 
-	foldl(fun(ME,Acc) ->
-		      case gb_trees:lookup(ME,We2#we.es) of
-			  none ->
-			      Acc;
-			  {value,#edge{vs=V1,ve=V2}} ->
-			      V1m = map_vertex(V1, Vmap),
-			      V2m = map_vertex(V2, Vmap),
-			      if V1m == V2m ->
-				      Acc;
-				 true ->
-				      Lookup = 
-					  fun(E,_,#edge{vs=Vs,ve=Ve},_) 
-					     when Vs==V1m,Ve==V2m -> E;
-					     (E,_,#edge{vs=Ve,ve=Vs},_) 
-					     when Vs==V1m,Ve==V2m -> E;
-					     (_,_,_,Cont) -> Cont
-					  end,
-				      E = wings_vertex:until(Lookup,fail,V1m,We0),
-				      [{ME,E}|Acc]
-			      end
-		      end
-	      end, [], Me),
-    We = We2#we{name=#ch{vmap=Vmap,me=Me, 
-			 emap=gb_trees:from_orddict(sort(Emap))}},
-    {Keep0,We}.
+    {We2,Vmap} = cut_edges(Keep0, InnerEdges, Cuts, We1, Map1),
+
+    %% Create edge map and finish We.
+    Me = wings_we:new_items_as_ordset(edge, We1, We2),
+    Emap = make_emap(Me, Vmap, We0, We2, []),
+    We3 = We2#we{name=#ch{vmap=Vmap,me=Me,emap=Emap}},
+
+    %% Dissolve unneeded faces and also hide them.
+    #we{fs=Ftab} = We = wpa:face_dissolve_complement(Keep0, We3),
+    Hidden = ordsets:subtract(gb_trees:keys(Ftab), Keep0),
+    wings_we:hide_faces(Hidden, We).
+
+make_emap([ME|T], Vmap, We0, #we{es=Etab}=We, Acc) ->
+    case gb_trees:lookup(ME, Etab) of
+	none ->
+	    make_emap(T, Vmap, We0, We, Acc);
+	{value,#edge{vs=Va0,ve=Vb0}} ->
+	    Va = map_vertex(Va0, Vmap),
+	    case map_vertex(Vb0, Vmap) of
+		Va ->
+		    make_emap(T, Vmap, We0, We, Acc);
+		Vb ->
+		    E = wings_vertex:until(
+			  fun(E, _, Rec, A) ->
+				  case Rec of
+				      #edge{vs=Va,ve=Vb} -> E;
+				      #edge{vs=Vb,ve=Va} -> E;
+				      _ -> A
+				  end
+			  end, none, Va, We0),
+		    case E of
+			none -> make_emap(T, Vmap, We0, We, Acc);
+			_ -> make_emap(T, Vmap, We0, We, [{ME,E}|Acc])
+		    end
+	    end
+    end;
+make_emap([], _, _, _, Acc) -> gb_trees:from_orddict(sort(Acc)).
 
 cut_shared_vertices(Faces, Es, #we{es=Etab}=We0, InvVmap0) ->
     VsEs0 = foldl(fun(E, A) ->
@@ -772,8 +786,7 @@ do_cut_shared_1([F|Fs], V, Wid, We0, Map0) ->
     do_cut_shared_1(Fs, V, Wid, We, Map);
 do_cut_shared_1([], _, _, We, Map) -> {We,Map}.
 
-cut_edges(Faces, Cuts0, We, Map) ->
-    Inner = wings_face:inner_edges(Faces, We),
+cut_edges(Faces, Inner, Cuts0, We, Map) ->
     case ordsets:intersection(Inner, Cuts0) of
 	[] -> {We,Map};
 	Cuts -> cut_edges_1(Faces, Cuts, We, Map)
@@ -782,28 +795,43 @@ cut_edges(Faces, Cuts0, We, Map) ->
 cut_edges_1(Faces, Cuts, We0, Map0) ->
     Vs = wings_edge:to_vertices(Cuts, We0),
     {We1,Map1} = bevel_cut_vs(Vs, We0, Map0),
-    NewEdges = gb_sets:to_list(wings_we:new_items(edge, We0, We1)),
-    {We2,Map,MaybeRem} = cut_new_edges(NewEdges, We1, Map1, []),
+    CutEdges = edges_to_cut(Cuts, We1),
+    {We2,Map} = cut_new_edges(CutEdges, We1, Map1),
+    MaybeRem = wings_we:new_items_as_ordset(edge, We0, We2),
     We3 = connect_edges(Cuts, We2),
     We = cut_cleanup(Faces, MaybeRem, We3),
     {We,Map}.
 
 bevel_cut_vs([V|Vs], We0, Map0) ->
     We = wings_vertex_cmd:bevel_vertex(V, We0),
-    NewVs = wings_we:new_items(vertex, We0, We),
-    Map = add_new_vs(V, gb_sets:to_list(NewVs), Map0),
+    NewVs = wings_we:new_items_as_ordset(vertex, We0, We),
+    Map = add_new_vs(V, NewVs, Map0),
     bevel_cut_vs(Vs, We, Map);
 bevel_cut_vs([], We, Map) -> {We,Map}.
 
-cut_new_edges([Edge|Es], #we{es=Etab}=We0, Map0, MaybeRemove0) ->
+edges_to_cut(Es, #we{es=Etab}) ->
+    edges_to_cut_1(Es, Etab, []).
+
+edges_to_cut_1([E|Es], Etab, Acc) ->
+    #edge{ltpr=Lp,ltsu=Lu,rtpr=Rp,rtsu=Ru} = gb_trees:get(E, Etab),
+    edges_to_cut_1(Es, Etab, [Lp,Lu,Rp,Ru|Acc]);
+edges_to_cut_1([], _, Es) ->
+    edges_to_cut_2(sort(Es), []).
+
+edges_to_cut_2([E,E|Es], Acc) ->
+    edges_to_cut_2(Es, [E|Acc]);
+edges_to_cut_2([_|Es], Acc) ->
+    edges_to_cut_2(Es, Acc);
+edges_to_cut_2([], Acc) ->
+    ordsets:from_list(Acc).
+
+cut_new_edges([Edge|Es], #we{es=Etab}=We0, Map0) ->
     #edge{vs=Va,ve=Vb} = gb_trees:get(Edge, Etab),
     Pos = wpa:vertex_pos(Va, We0),
-    {We,NewV=NewEdge} = wings_edge:fast_cut(Edge, Pos, We0),
+    {We,NewV} = wings_edge:screaming_cut(Edge, Pos, We0),
     Map = add_new_vs(Vb, [NewV], add_new_vs(Va, [NewV], Map0)),
-    MaybeRemove = [Edge,NewEdge|MaybeRemove0],
-    cut_new_edges(Es, We, Map, MaybeRemove);
-cut_new_edges([], We, Map, MaybeRemove) ->
-    {We,Map,ordsets:from_list(MaybeRemove)}.
+    cut_new_edges(Es, We, Map);
+cut_new_edges([], We, Map) -> {We,Map}.
 
 connect_edges([E|Es], #we{es=Etab}=We0) ->
     #edge{vs=Va,ve=Vb,lf=Lf,ltpr=Lp,ltsu=Lu,
@@ -823,7 +851,9 @@ other_vertex(V, Edge, Etab) ->
 cut_cleanup(Faces, MaybeRemove, We) ->
     Es = ordsets:intersection(MaybeRemove,
 			      wings_face:to_edges(Faces, We)),
-    foldl(fun(E, W) -> wings_collapse:collapse_edge(E, W) end, We, Es).
+    foldl(fun(E, W) ->
+		  wings_collapse:fast_collapse_edge(E, W)
+	  end, We, Es).
 
 add_new_vs(OldV, NewVs, Map) ->
     foldl(fun(NewV, M) -> add_new_vs_1(OldV, NewV, M) end, Map, NewVs).
@@ -851,60 +881,65 @@ normalize_charts([], Cuts0, We, Charts) ->
     Cuts = gb_sets:intersection(Cuts0, AllInner),
     {Charts,Cuts}.
 
-normalize_chart(Fs, Cuts, We, Acc) ->
-    case gb_sets:is_empty(Fs) of
+normalize_chart(Faces, Cuts, We, Acc) ->
+    find_reachable(Faces, We, collect_fun(Cuts), Acc).
+
+find_reachable(Faces0, We, Coll, Acc) ->
+    case gb_sets:is_empty(Faces0) of
 	true -> Acc;
 	false ->
-	    {F,_} = gb_sets:take_smallest(Fs),
-	    Reachable = reachable_faces(F, Fs, Cuts, We),
-	    Rest = gb_sets:difference(Fs, Reachable),
-	    normalize_chart(Rest, Cuts, We, [gb_sets:to_list(Reachable)|Acc])
+	    {Face,Faces1} = gb_sets:take_smallest(Faces0),
+	    Ws = [Face],
+	    {Reg,Faces} = collect_reachable(Ws, We, Coll, [], Faces1),
+	    find_reachable(Faces, We, Coll, [Reg|Acc])
     end.
 
-%% reachable_faces(Face, FaceSet, EdgeSet, We) -> Faces.
-%%  Return a set containing all faces in FaceSet reachable from Face,
-%%  without crossing any edge in EdgeSet.
-reachable_faces(Face, Faces, Edges, We) ->
-    reachable_faces_1(gb_sets:singleton(Face), We, {Faces,Edges}, gb_sets:empty()).
+collect_reachable([_|_]=Ws0, We, Coll, Reg0, Faces0) ->
+    Reg = Ws0++Reg0,
+    {Ws,Faces} = wings_face:fold_faces(Coll, {[],Faces0}, Ws0, We),
+    collect_reachable(Ws, We, Coll, Reg, Faces);
+collect_reachable([], _, _, Reg, Faces) ->
+    {sort(Reg),Faces}.
 
-reachable_faces_1(Work0, We, Constraints, Acc0) ->
-    case gb_sets:is_empty(Work0) of
-	true -> Acc0;
-	false ->
-	    {Face,Work1} = gb_sets:take_smallest(Work0),
-	    Acc = gb_sets:insert(Face, Acc0),
-	    Work = collect_maybe_add(Work1, Face, Constraints, We, Acc),
-	    reachable_faces_1(Work, We, Constraints, Acc)
+collect_fun(Cuts) ->
+    fun(Face, _, Edge, Rec, {Ws,Faces}=A) ->
+	    Of = case Rec of
+		     #edge{lf=Face,rf=Of0} -> Of0;
+		     #edge{rf=Face,lf=Of0} -> Of0
+		 end,
+	    case gb_sets:is_member(Of, Faces) andalso
+		not gb_sets:is_member(Edge, Cuts) of
+		true -> {[Of|Ws],gb_sets:delete(Of, Faces)};
+		false -> A
+	    end
     end.
 
-collect_maybe_add(Work, Face, {Faces,Edges}, We, Res) ->
-    wings_face:fold(
-      fun(_, Edge, Rec, A) ->
-	      case gb_sets:is_member(Edge, Edges) of
-		  true -> A;
-		  false ->
-		      Of = wings_face:other(Face, Rec),
-		      case gb_sets:is_member(Of, Faces) andalso
-			  not gb_sets:is_member(Of, Res) of
-			  false -> A;
-			  true -> gb_sets:add(Of, A)
-		      end
-	      end
-      end, Work, Face, We).
-
-
 %%%
-%%% Build a map {F,V} => UV.
+%%% Build a map [F|V] => UV.
 %%%
 
-fv_to_uv_map(Fs, We) ->
-    fv_to_uv_map(Fs, We, []).
+fv_to_uv_map(#we{fs=Ftab}=We) ->
+    FsEs = gb_trees:to_list(Ftab),
+    fvuvmap_1(FsEs, We, [], []).
 
-fv_to_uv_map([F|Fs], We, Acc0) ->
-    Acc = [{{F,V},UV} || [V|UV] <- wings_face:vinfo_ccw(F, We)] ++ Acc0,
-    fv_to_uv_map(Fs, We, Acc);
-fv_to_uv_map([], _, Acc) ->
-    gb_trees:from_orddict(sort(Acc)).
+fvuvmap_1([{F,E}|FsEs], We, FaceAcc, Acc) ->
+    case uv_info(F, E, We) of
+	error -> fvuvmap_1(FsEs, We, FaceAcc, Acc);
+	Info -> fvuvmap_1(FsEs, We, [F|FaceAcc], Info++Acc)
+    end;
+fvuvmap_1([], _, FaceAcc, Acc) ->
+    {FaceAcc,gb_trees:from_orddict(sort(Acc))}.
+	
+uv_info(F, E, We) ->
+    uv_info_1(wings_face:vinfo_ccw(F, E, We), F, []).
+
+uv_info_1([[V|{_,_}=UV]|T], F, Acc) ->
+    uv_info_1(T, F, [{[F|V],UV}|Acc]);
+uv_info_1([[V|_]|T], F, Acc) ->
+    %% No UV coordinates for this vertex.
+    uv_info_1(T, F, [{[F|V],none}|Acc]);
+uv_info_1([_|_], _, _) -> error;
+uv_info_1([], _, Acc) -> Acc.
 
 %%%
 %%% Given a model having UV coordinates, partition it into charts.
@@ -912,72 +947,54 @@ fv_to_uv_map([], _, Acc) ->
 
 uv_to_charts(Faces0, Dict, We) ->
     Faces = gb_sets:from_list(Faces0),
-    Cuts = gb_sets:from_list(wings_face:inner_edges(Faces0, We)),
-    uv_to_charts_1(Faces, We, Dict, Cuts, []).
+    Coll = collect_chart_fun(Dict),
+    Cs = uv_to_charts_1(Faces, We, Coll, []),
+    Cuts = chart_cuts(Cs, We, Dict, []),
+    {Cs,Cuts}.
 
-uv_to_charts_1(Faces0, We, D, Cuts0, Charts) ->
-    case gb_trees:is_empty(Faces0) of
-	true ->
-	    normalize_charts(Charts, Cuts0, We);
+uv_to_charts_1(Faces0, We, Coll, Acc) ->
+    case gb_sets:is_empty(Faces0) of
+	true -> Acc;
 	false ->
 	    {Face,Faces1} = gb_sets:take_smallest(Faces0),
-	    Ws = gb_sets:singleton(Face),
-	    Empty = gb_sets:empty(),
-	    {Chart,Cuts,Faces} = collect_chart(Ws, We, D, Empty, Cuts0, Faces1),
-	    uv_to_charts_1(Faces, We, D, Cuts, [gb_sets:to_list(Chart)|Charts])
+	    Ws = [Face],
+	    {Reg,Faces} = collect_chart(Ws, We, Coll, [], Faces1),
+	    uv_to_charts_1(Faces, We, Coll, [Reg|Acc])
     end.
 
-collect_chart(Ws0, We, D, Charts0, Cuts0, Faces0) ->
-    case gb_sets:is_empty(Ws0) of
-	true ->
-	    {Charts0,Cuts0,Faces0};
-	false ->
-	    {Face,Ws1} = gb_sets:take_smallest(Ws0),
-	    Charts = gb_sets:add(Face, Charts0),
-	    Cuts = remove_non_cutting(Face, D, We, Charts, Cuts0),
-	    {Ws,Faces} = collect_adj_faces(Face, We, D, Ws1, Faces0),
-	    collect_chart(Ws, We, D, Charts, Cuts, Faces)
+collect_chart([_|_]=Ws0, We, Coll, Reg0, Faces0) ->
+    Reg = Ws0++Reg0,
+    {Ws,Faces} = wings_face:fold_faces(Coll, {[],Faces0}, Ws0, We),
+    collect_chart(Ws, We, Coll, Reg, Faces);
+collect_chart([], _, _, Reg, Faces) ->
+    {sort(Reg),Faces}.
+
+collect_chart_fun(Dict) ->
+    fun(Face, _, _, Rec, {Ws,Faces}=A) ->
+	    Of = case Rec of
+		     #edge{lf=Face,rf=Of0} -> Of0;
+		     #edge{rf=Face,lf=Of0} -> Of0
+		 end,
+	    case gb_sets:is_member(Of, Faces) andalso
+		not is_cutting_edge(Rec, Dict) of
+		true -> {[Of|Ws],gb_sets:delete(Of, Faces)};
+		false -> A
+	    end
     end.
 
-collect_adj_faces(Face, We, D, Ws, Faces0) ->
-    wings_face:fold(
-      fun(_, _, Rec, {W0,F0}=A) ->
-	      Of = wings_face:other(Face, Rec),
-	      case gb_sets:is_member(Of, F0) andalso not is_cutting_edge(Rec, D) of
-		  false -> A;
-		  true -> {gb_sets:insert(Of, W0),gb_sets:delete(Of, F0)}
-	      end
-      end, {Ws,Faces0}, Face, We).
+chart_cuts([C|Cs], #we{es=Etab}=We, D, Acc0) ->
+    Inner = wings_face:inner_edges(C, We),
+    Acc = chart_cuts_1(Inner, Etab, D, Acc0),
+    chart_cuts(Cs, We, D, Acc);
+chart_cuts([], _, _, Acc) -> gb_sets:from_list(Acc).
 
-remove_non_cutting(Face, D, We, Charts, Cuts0) ->
-    wings_face:fold(
-      fun(_, E, Rec, Cuts) ->
-	      Of = wings_face:other(Face, Rec),
-	      case gb_sets:is_member(Of, Charts) andalso
-		  not is_cutting_edge(Rec, D) of
-		  false -> Cuts;
-		  true -> gb_sets:delete_any(E, Cuts)
-	      end
-      end, Cuts0, Face, We).
+chart_cuts_1([E|Es], Etab, D, Acc) ->
+    case is_cutting_edge(gb_trees:get(E, Etab), D) of
+	false -> chart_cuts_1(Es, Etab, D, Acc);
+	true -> chart_cuts_1(Es, Etab, D, [E|Acc])
+    end;
+chart_cuts_1([], _, _, Acc) -> Acc.
 
 is_cutting_edge(#edge{vs=Va,ve=Vb,lf=Lf,rf=Rf}, D) ->
-    gb_trees:get({Lf,Va}, D) =/= gb_trees:get({Rf,Va}, D) orelse
-	gb_trees:get({Lf,Vb}, D) =/= gb_trees:get({Rf,Vb}, D).
-
-%%%
-%%% Finalize charts for use in AutoUV.
-%%%
-
-finalize_charts(Cs) ->
-    finalize_charts_1(Cs, length(Cs), []).
-
-finalize_charts_1([{Fs,We0}|Cs], Id, Acc) ->
-    We = finalize_chart(Fs, We0#we{id=Id}),
-    finalize_charts_1(Cs, Id-1, [We|Acc]);
-finalize_charts_1([], 0, Acc) -> Acc.
-
-finalize_chart(Fs, #we{fs=Ftab0}=We0) ->
-    NotNeeded = ordsets:subtract(gb_trees:keys(Ftab0), Fs),
-    #we{fs=Ftab} = We = wings_face_cmd:dissolve(NotNeeded, We0),
-    Hidden = ordsets:subtract(gb_trees:keys(Ftab), Fs),
-    wings_we:hide_faces(Hidden, We).
+    gb_trees:get([Lf|Va], D) =/= gb_trees:get([Rf|Va], D) orelse
+	gb_trees:get([Lf|Vb], D) =/= gb_trees:get([Rf|Vb], D).
