@@ -1,14 +1,15 @@
-%% File    : wpc_autouv.erl
-%% Author  : Dan Gudmundsson <dgud@erix.ericsson.se>
-%% Description : A semi-simple semi-automatic UV-mapping semi-plugin
 %%
-%% Created : 24 Jan 2002 by Dan Gudmundsson <dgud@erix.ericsson.se>
-%%-------------------------------------------------------------------
-%%  Copyright (c) 2002-2004 Dan Gudmundsson, Bjorn Gustavsson
+%%  wpc_autouv.erl --
+%%
+%%     A semi-simple semi-automatic UV-mapping semi-plugin.
+%%
+%%  Copyright (c) 2002-2005 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-%%     $Id: wpc_autouv.erl,v 1.271 2004/10/20 11:21:47 dgud Exp $
+%%
+%%     $Id: wpc_autouv.erl,v 1.302 2005/03/22 05:50:48 bjorng Exp $
+%%
 
 -module(wpc_autouv).
 
@@ -19,14 +20,14 @@
 -include("e3d_image.hrl").
 -include("auv.hrl").
  
--export([init/0,menu/2,command/2,redraw/1,window/1]).
-
+-export([init/0,menu/2,command/2,redraw/1]).
+-export([handle_event/2]). %% Debug
 -import(lists, [sort/1,keysort/2,map/2,foldl/3,reverse/1,
 		append/1,delete/2,usort/1,max/1,min/1,
 		member/2,foreach/2,keysearch/3]).
 
 %% Exports to auv_texture.
--export([get_material/3,has_texture/2]).
+-export([has_texture/2]).
 
 %% Exports to auv_seg_ui.
 -export([init_show_maps/3]).
@@ -39,34 +40,56 @@ menu({body}, Menu) ->
 	true ->
 	    Menu;
 	false ->
-	    Menu ++ [separator,
-		     {"UV Mapping", ?MODULE,
-		      "Generate or edit UV mapping or texture"}
-		    ]
+	    Menu ++ [separator,auv_menu()]
     end;
 menu({face}, Menu) ->
     case wpc_snap:active() of
 	true ->
 	    Menu;
 	false ->
-	    Menu ++ [separator,
-		     {"UV Mapping", ?MODULE,
-		      "Generate UV mapping or texture"}
-		    ]
+	    Menu ++ [separator,auv_menu()]
     end;
-
+menu({window}, Menu) ->
+    Menu ++ [separator,
+	     {"UV Editor Window",uv_editor_window,
+	      "Open a UV Editor window for each selected object"}];
 menu(_Dbg, Menu) ->
     Menu.
 
-command({body,?MODULE}, St) ->
-    start_uvmap(segment, St);
-command({face,?MODULE}, St) ->
-    start_uvmap(segment, St);
-command(_, _) -> next.
+auv_menu() ->
+    {"UV Mapping", {?MODULE, fun auv_menu/2}}.
+auv_menu(help,_) ->
+    {"Generate UV mapping or texture",
+     "",
+     "Force to segmenting mode"};
+auv_menu(1,_What) -> 
+    case wings_pref:get_value(advanced_menus) of
+	false -> 
+	    [{"Direct", segment,
+	      "Open UV-window directly if selection already contains uv-coords"},
+	     {"Force Segment", force_seg,
+	      "Delete old UV-coords and start over with segmenting"}];
+	true -> 
+	    {?MODULE, segment}
+    end;
+auv_menu(2,_) -> ignore;
+auv_menu(3,_) -> {?MODULE, force_seg}.
+
+command({body,{?MODULE, Op}} , St) ->
+    start_uvmap(Op, St);
+command({face,{?MODULE, Op}} , St) ->
+    start_uvmap(Op, St);
+command({?MODULE, Op}, St) ->
+    start_uvmap(Op, St);
+command({window,uv_editor_window}, St) ->
+    window(St);
+command(_Cmd, _) -> 
+    next.
 
 window(St) ->
     start_uvmap(edit, St).
 
+start_uvmap(edit, #st{sel=[]}) -> wings_u:error("Nothing selected");
 start_uvmap(Action, #st{sel=Sel}=St) ->
     start_uvmap_1(Sel, Action, St).
 
@@ -91,20 +114,37 @@ start_uvmap_2(Action, Name, Id, #st{shapes=Shs}=St) ->
 		       {toolbar,CreateToolbar}], Op),
     wings_wm:send(Name, {init,{Action,We}}).
 
-auv_event({init,Op}, St = #st{selmode = Mode}) ->
+auv_event({init,Op}, St = #st{selmode=Mode,sel=Sel0,shapes=Shs}) ->
     wings:init_opengl(St),
     case Op of
 	{edit,We} ->
 	    start_edit(We, St);
+	{segment,We = #we{id=Id}} when Mode == face ->
+	    UVFs = gb_sets:from_ordset(wings_we:uv_mapped_faces(We)),
+	    {value, {_, Fs}} = lists:keysearch(Id, 1, Sel0),
+	    case gb_sets:is_subset(Fs,UVFs) of
+		false -> auv_seg_ui:start(We, We, St);
+		true -> 
+		    %% Hmm this didn't work..
+		    Other = wings_sel:inverse_items(face, Fs, We),
+		    NewWe = wings_we:hide_faces(Other, We),
+		    NewSt = wings_sel:clear(St#st{shapes=gb_trees:update(Id, We,Shs)}),
+		    start_edit(NewWe, NewSt)
+	    end;
 	{segment,We} ->
-	    case wings_we:uv_mapped_faces(We) of
-		[_|_Faces] when Mode == body -> start_edit(We, St);
-		_ -> auv_seg_ui:start(We, We, St)
-	    end
+	    case wings_we:uv_mapped_faces(We) of	    
+		[] -> auv_seg_ui:start(We, We, St);
+		_ -> start_edit(We, St)
+	    end;
+	{force_seg, We} ->
+	    auv_seg_ui:start(We, We, St)
     end;
 auv_event(redraw, _) ->
     wings_wm:clear_background(),
     keep;
+auv_event({crash,Crash}, _) ->
+    wings_u:win_crash(Crash),
+    delete;
 auv_event(_Ev, _) -> keep.
 
 %%%
@@ -112,7 +152,7 @@ auv_event(_Ev, _) -> keep.
 %%%
 
 start_edit(We, St) ->
-    MatNames0 = wings_material:get_all(We),
+    MatNames0 = wings_facemat:all(We),
     MatNames1 = sofs:from_external(MatNames0, [{face,material}]),
     MatNames2 = sofs:converse(MatNames1),
     MatNames3 = sofs:relation_to_family(MatNames2),
@@ -170,6 +210,7 @@ create_uv_state(Charts, MatName, We, GeomSt) ->
     wings_wm:set_prop(wireframed_objects,
 		      gb_sets:from_list(gb_trees:keys(Charts))),
     wings_wm:set_prop(allow_rotation, false),
+    wings_wm:set_prop(select_backface, true),
 
     wings_wm:later(got_focus),
 
@@ -233,7 +274,7 @@ update_selected_uvcoords(#st{bb=Uvs}=St) ->
     Shs = gb_trees:update(Id, We, Shs0),
     GeomSt = GeomSt0#st{shapes=Shs},
     wings_wm:send(geom, {new_state,GeomSt}),
-    St#st{bb=Uvs#uvstate{st=GeomSt}}.
+    clear_temp_sel(St#st{bb=Uvs#uvstate{st=GeomSt}}).
 
 %% update_uvs(Charts, We0) -> We
 %%  Update the UV coordinates for the original model.
@@ -286,7 +327,7 @@ update_uvs_3(V, Face, UV, Etab, We) ->
 
 insert_material(Cs, MatName, We) ->
     Faces = lists:append([wings_we:visible(W) || W <- gb_trees:values(Cs)]),
-    wings_material:assign(MatName, Faces, We).
+    wings_facemat:assign(MatName, Faces, We).
 
 
 %%%%% Material handling
@@ -301,11 +342,6 @@ has_texture(MatName, Materials) ->
 	    none /= proplists:get_value(diffuse, Maps, none)
     end.
 
-get_material(Face, Materials, We) ->
-    MatName = wings_material:get(Face, We),
-    Mat = gb_trees:get(MatName, Materials),
-    proplists:get_value(diffuse, proplists:get_value(opengl, Mat)).
-
 add_material(#e3d_image{}=Tx, Name, none, St0) ->
     MatName0 = list_to_atom(Name++"_auv"),
     Mat = {MatName0,[{opengl,[]},{maps,[{diffuse,Tx}]}]},
@@ -322,18 +358,23 @@ add_material(Im = #e3d_image{}, _, MatName,St) ->
 %%%% Menus.
 
 command_menu(body, X, Y) ->
-    Rotate = rotate_directions(),
-
-    Scale =  [{"Uniform",    scale_uniform, "Scale in both directions"},
-	      {"Horizontal", scale_x, "Scale horizontally (X dir)"},
-	      {"Vertical",   scale_y, "Scale vertically (Y dir)"}],
-
     Menu = [{basic,{"Chart operations",ignore}},
 	    {basic,separator},
 	    {"Move", move, "Move selected charts"},
-	    {"Scale", {scale, Scale}, "Scale selected charts"},
-	    {"Rotate", {rotate, Rotate}, "Rotate selected charts"},
+	    {"Scale", {scale, scale_directions()++ stretch_directions()}, 
+	     "Scale selected charts"},
+	    {"Rotate", rotate, "Rotate selected charts"},
 	    separator,
+	    {"Move to", 
+	     {move_to, 
+	      [{"Center", center, "Move to Center"},
+	       {"Center X", center_x, "Move to horizontal center"},
+	       {"Center Y", center_y, "Move to vertical center"},
+	       {"Bottom", bottom, "Move to bottom border"},
+	       {"Top", top, "Move to top border"},
+	       {"Left", left, "Move to left border"},
+	       {"Right", right, "Move to right border"}
+	      ]}, "Move charts to position"},
 	    {"Flip",{flip,
 		     [{"Horizontal",horizontal,"Flip selection horizontally"},
 		      {"Vertical",vertical,"Flip selection vertically"}]},
@@ -356,21 +397,24 @@ command_menu(body, X, Y) ->
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(face, X, Y) ->
     Scale = scale_directions(),
-    Rotate = rotate_directions(),
     Menu = [{basic,{"Face operations",ignore}},
+	    {basic,separator},
 	    {"Move",move,"Move selected faces",[magnet]},
 	    {"Scale",{scale,Scale},"Scale selected faces"},
-	    {"Rotate",{rotate,Rotate},"Rotate selected faces"}
+	    {"Rotate",rotate,"Rotate selected faces"}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(edge, X, Y) ->
     Scale = scale_directions(),
-    Rotate = rotate_directions(),
+    Align = 	    
+	[{"Free",free,"Rotate selection freely"},
+	 {"Chart to X", align_x, "Rotate chart to align selected edge to X-axis"},
+	 {"Chart to Y", align_y, "Rotate chart to align selected edge to Y-axis"}],
     Menu = [{basic,{"Edge operations",ignore}},
 	    {basic,separator},
 	    {"Move",move,"Move selected edges",[magnet]},
 	    {"Scale",{scale,Scale},"Scale selected edges"},
-	    {"Rotate",{rotate,Rotate},"Rotate selected edges"},
+	    {"Rotate",{rotate,Align},"Rotate commands"},
 	    separator,
 	    {"Stitch", stitch, "Stitch edges/charts"},
 	    {"Cut", cut_edges, "Cut selected edges"}
@@ -378,12 +422,18 @@ command_menu(edge, X, Y) ->
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(vertex, X, Y) ->
     Scale = scale_directions(),
-    Rotate = rotate_directions(),
+    Align = 	    
+	[{"Free",free,"Rotate selection freely"},
+	 {"Chart to X", align_x, 
+	  "Rotate chart to align (imaginary) edge joining selected verts to X-axis"},
+	 {"Chart to Y", align_y, 
+	  "Rotate chart to align (imaginary) edge joining selected verts to Y-axis"}],
+
     Menu = [{basic,{"Vertex operations",ignore}},
 	    {basic,separator},
 	    {"Move",move,"Move selected vertices",[magnet]},
 	    {"Scale",{scale,Scale},"Scale selected vertices"},
-	    {"Rotate",{rotate,Rotate},"Rotate selected vertices"},
+	    {"Rotate",{rotate,Align},"Rotation commands"},
 	    separator,
 	    {"Flatten",{flatten,
 			[{"X", x, "Flatten horizontally"},
@@ -393,26 +443,23 @@ command_menu(vertex, X, Y) ->
 	     "Move UV coordinates towards average midpoint",
 	     [magnet]},
 	    separator, 
-	    {"Unfold", lsqcm, "Unfold the chart (without moving the selected vertices)"}
+	    {"Unfold",lsqcm,"Unfold the chart (without moving the selected vertices)"}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(_, X, Y) ->
     [_|Menu] = option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu).
 
+stretch_directions() ->
+    [{"Max Uniform",    max_uniform, "Maximize either horizontally or vertically"},
+     {"Max Horizontal", max_x, "Maximize horizontally (X dir)"},
+     {"Max Vertical",   max_y, "Maximize vertically (Y dir)"}].
+
 scale_directions() ->
     [{"Uniform",    scale_uniform, "Scale in both directions"},
      {"Horizontal", scale_x, "Scale horizontally (X dir)"},
      {"Vertical",   scale_y, "Scale vertically (Y dir)"}].
 
-rotate_directions() ->
-    [{"Free",free,"Rotate freely"},
-     {"90"++[?DEGREE]++" CW",-90,
-      "Rotate selection 90 degrees clockwise"},
-     {"90"++[?DEGREE]++" CCW",90,
-      "Rotate selection 90 degrees counter-clockwise"},
-     {"180"++[?DEGREE],180,"Rotate selection 180 degrees"}].
-
 option_menu() ->
     [separator,
      {"Create Texture",create_texture,"Make and Attach a texture to the model"}].
@@ -426,12 +473,13 @@ get_event(#st{}=St) ->
 
 get_event_nodraw(#st{}=St) ->
     wings_wm:current_state(St),
-    {replace,fun(Ev) -> handle_event(Ev, St) end}.
+    {replace,fun(Ev) -> ?MODULE:handle_event(Ev, St) end}.
 
-handle_event({crash,_}=Crash, _) ->
-    LogName = wings_util:crash_log(wings_wm:this(), Crash),
-    wings_wm:send(geom, {crash_in_other_window,LogName}),
+handle_event({crash,Crash}, _) ->
+    wings_u:win_crash(Crash),
     delete;
+handle_event({command_error,Error}, _) ->
+    wings_u:message(Error);
 handle_event(redraw, St) ->
     redraw(St),
     get_event_nodraw(St);
@@ -444,20 +492,54 @@ handle_event({new_state,St}, _) ->
     new_state(St);
 handle_event(revert_state, St) ->
     get_event(St);
+handle_event({current_state,geom_display_lists,GeomSt}, AuvSt) ->
+    new_geom_state(GeomSt, AuvSt);
+handle_event({do_tweak, Type, St =#st{sh=Sh,selmode=Mode}}, _) ->
+    case Type of 
+	temp_selection ->
+	    handle_command(move,St#st{temp_sel={Mode,Sh}});
+	_ -> 
+	    handle_command(move,St)
+    end;
+handle_event({cancel_tweak,Ev}, St) ->
+    handle_event_1(Ev,St,wings_msg:free_lmb_modifier());
 handle_event(Ev, St) ->
-    case wings_camera:event(Ev, fun() -> redraw(St) end) of
-	next -> handle_event_1(Ev, St);
-	Other -> Other
+    case wings_camera:event(Ev, St, fun() -> redraw(St) end) of
+	next ->
+	    FreeLmbMod = wings_msg:free_lmb_modifier(),
+%%	    io:format("Ev ~W~n",[Ev,3]),
+	    handle_event_0(Ev, St, FreeLmbMod);
+	Other -> 
+	    Other
     end.
 
-%% Short cut for move selected
-handle_event_1(#mousebutton{state=?SDL_PRESSED,
-			    button=?SDL_BUTTON_LEFT,
-			    mod=Mod},
-	       St = #st{sel = Sel}) 
-  when Sel /= [], (Mod band ?ALT_BITS) =/= 0 ->
-    handle_command(move,St);
-handle_event_1(Ev, St) ->
+%% Short cut for tweak like move
+handle_event_0(Ev=#mousebutton{state=?SDL_PRESSED,
+			       x=X,y=Y,
+			       button=?SDL_BUTTON_LEFT,
+			       mod=Mod},
+	       #st{sel=Sel}=St0, FreeLmbMod) 
+  when (Mod band 16#0FFF) == 0 -> %% No modifiers
+    case (Sel == []) and wings_pref:get_value(use_temp_sel) of
+	true ->
+	    case wings_pick:do_pick(X, Y, St0) of
+		{add,_,St} -> 
+		    start_tweak(temp_selection, Ev, St);
+		_ -> 
+		    handle_event_1(Ev, St0, FreeLmbMod)
+	    end;
+	false ->
+	    case wings_pick:do_pick(X,Y,St0) of
+		{delete,_,_} ->
+		    start_tweak(selection, Ev, St0);
+		_ -> 
+		    handle_event_1(Ev, St0, FreeLmbMod)
+	    end
+    end;
+handle_event_0(Ev, St, FreeLmbMod) ->
+    handle_event_1(Ev, St, FreeLmbMod).
+
+handle_event_1(Ev, St, _) ->
     case wings_pick:event(Ev, St) of
 	next -> handle_event_2(Ev, St);
 	Other -> Other
@@ -469,18 +551,20 @@ handle_event_2(Ev, St) ->
 	Cmd -> wings_wm:later({action,Cmd})
     end.
 
-handle_event_3({current_state,geom_display_lists,GeomSt}, AuvSt) ->
-    new_geom_state(GeomSt, AuvSt);
-handle_event_3(#mousebutton{state=?SDL_RELEASED,
-			    button=?SDL_BUTTON_RIGHT,
-			    x=X0,y=Y0},
+handle_event_3(#mousebutton{button=?SDL_BUTTON_RIGHT}=Ev,
 	       #st{selmode=Mode0,sel=Sel}) ->
-    {X,Y} = wings_wm:local2global(X0, Y0),
-    Mode = case Sel of 
-	       [] -> undefined; 
-	       _ -> Mode0 
-	   end,
-    command_menu(Mode, X, Y);
+    %% Note: Basic menus must be shown when the right mouse button
+    %% is PRESSED; advanced menus when the button is RELEASED.
+    %% wings_menu:is_popup_event/1 takes care of that.
+    case wings_menu:is_popup_event(Ev) of
+	no -> keep;
+	{yes,X,Y,_} ->
+	    Mode = case Sel of 
+		       [] -> undefined; 
+		       _ -> Mode0 
+		   end,
+	    command_menu(Mode, X, Y)
+    end;
 handle_event_3({drop,_,DropData}, St) ->
     handle_drop(DropData, St);
 handle_event_3({action,{auv,create_texture}},_St) ->
@@ -502,7 +586,7 @@ handle_event_3({action,{auv,{remap,Method}}}, St0) ->
     St = remap(Method, St0),
     get_event(St);
 handle_event_3({action,{auv,lsqcm}}, St0) ->
-    St = remap(lsqcm, St0),
+    St = reunfold(St0),
     get_event(St);
 
 %% Others
@@ -518,6 +602,7 @@ handle_event_3({action,{auv,quit}}, _St) ->
     cleanup_before_exit(),
     delete;
 handle_event_3({action,{auv,Cmd}}, St) ->
+%%    io:format("Cmd ~p ~n", [Cmd]),
     handle_command(Cmd, St);
 handle_event_3({action,{select,Command}}, St0) ->
     case wings_sel_cmd:command(Command, St0) of
@@ -543,21 +628,57 @@ handle_event_3({action,Ev}, St) ->
 	    handle_command({scale,scale_y},St);
 	{_, {scale,_}} ->
 	    handle_command({scale,scale_uniform},St);
+	{view,aim} ->
+	    St1 = fake_selection(St),
+	    wings_view:command(aim, St1),
+	    get_event(St);
+	{view,Cmd} when Cmd == frame ->
+	    wings_view:command(Cmd,St),
+	    get_event(St);
 	_ ->
-%%	    io:format("MissEvent ~p~n", [Ev]),
 	    keep
     end;
 handle_event_3(got_focus, _) ->
-    Msg1 = wings_util:button_format("Select"),
+    Msg1 = wings_msg:button_format("Select"),
     Msg2 = wings_camera:help(),
-    Msg3 = wings_util:button_format([], [], "Show menu"),
-    Msg4 = "[Alt]+L: Move selected",
-    Message = wings_util:join_msg([Msg1,Msg2,Msg3,Msg4]),
-    wings_wm:message(Message),
+    Msg3 = wings_msg:button_format([], [], "Show menu"),
+    Message = wings_msg:join([Msg1,Msg2,Msg3]),
+    wings_wm:message(Message, ""),
     wings_wm:dirty();
 handle_event_3(_Event, _) ->
+%%    io:format("MissEvent ~p~n", [_Event]),
     keep.
 
+clear_temp_sel(#st{temp_sel=none}=St) -> St;
+clear_temp_sel(#st{temp_sel={Mode,Sh}}=St) ->
+    St#st{temp_sel=none,selmode=Mode,sh=Sh,sel=[]}.
+
+-record(tweak, {type, st, pos, ev}).
+
+start_tweak(Type, Ev = #mousebutton{x=X,y=Y}, St0) ->
+%%    io:format("Start tweak~n", []),
+    T = #tweak{type=Type,st=St0,pos={X,Y}, ev=Ev},
+    {seq,push,get_tweak_event(T)}.
+
+get_tweak_event(T) ->
+    {replace,fun(Ev) -> tweak_event(Ev, T) end}.
+tweak_event(#mousemotion{x=X,y=Y}, #tweak{pos={Sx,Sy},type=Type,st=St}) ->
+    case (abs(X-Sx) > 2) orelse (abs(Y-Sy) > 2) of
+	true -> 
+	    if Type == temp_selection ->
+		    wings_wm:later(clear_selection);
+	       true -> ignore
+	    end,
+	    wings_wm:later({do_tweak,Type,St}),
+	    pop;
+	false ->
+	    keep
+    end;
+tweak_event(Other, #tweak{ev=Ev}) ->
+    wings_wm:later(Other),
+    wings_wm:later({cancel_tweak,Ev}),
+    pop.
+
 new_state(#st{bb=#uvstate{}=Uvs}=St0) ->
     GeomSt = update_geom_selection(St0),
     St1 = St0#st{bb=Uvs#uvstate{st=GeomSt}},
@@ -565,7 +686,7 @@ new_state(#st{bb=#uvstate{}=Uvs}=St0) ->
     get_event(St).
 
 handle_command(move, St) ->
-    drag(wings_move:setup(free, St));
+    drag(wings_move:setup(free_2d, St));
 handle_command({move,Magnet}, St) ->
     drag(wings_move:setup({free_2d,Magnet}, St));
 handle_command({scale,scale_uniform}, St) ->
@@ -574,8 +695,18 @@ handle_command({scale,scale_x}, St) ->
     drag(wings_scale:setup({x,center}, St));
 handle_command({scale,scale_y}, St) ->
     drag(wings_scale:setup({y,center}, St));
+handle_command({scale,Dir}, St0) -> %% Maximize chart
+    St1 = wpa:sel_map(fun(_, We) -> stretch(Dir,We) end, St0),
+    St = update_selected_uvcoords(St1),
+    get_event(St);
+handle_command(rotate, St) ->
+    drag(wings_rotate:setup({free,center}, St));
 handle_command({rotate,free}, St) ->
     drag(wings_rotate:setup({free,center}, St));
+handle_command({move_to,Dir}, St0) ->
+    St1 = wpa:sel_map(fun(_, We) -> move_to(Dir,We) end, St0),
+    St = update_selected_uvcoords(St1),
+    get_event(St);
 handle_command({flip,horizontal}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_horizontal(We) end, St0),
     St = update_selected_uvcoords(St1),
@@ -584,6 +715,11 @@ handle_command({flip,vertical}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_vertical(We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
+handle_command({rotate,Dir}, St0) 
+  when Dir == align_y; Dir == align_x; Dir == align_xy ->
+    St1 = align_chart(Dir, St0),
+    St = update_selected_uvcoords(St1),
+    get_event(St);
 handle_command({rotate,Deg}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> rotate_chart(Deg, We) end, St0),
     St = update_selected_uvcoords(St1),
@@ -604,7 +740,7 @@ handle_command(stitch, St0 = #st{selmode=edge}) ->
     St1 = stitch(St0),
     AuvSt = #st{bb=#uvstate{id=Id,st=Geom}} = update_selected_uvcoords(St1),
     %% Do something here, i.e. restart uvmapper.
-    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes),AuvSt,false,[]),
+    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes), AuvSt, []),
     get_event(St);
 handle_command(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
     Es = wpa:sel_fold(fun(Es,#we{name=#ch{emap=Emap}},A) ->
@@ -612,13 +748,32 @@ handle_command(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
 			       || E<-gb_sets:to_list(Es)] ++ A
 		      end, [], St0),
     %% Do something here, i.e. restart uvmapper.
-    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes),St0,false,Es),
+    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes), St0, Es),
     get_event(St);
 
 %%    get_event(St);
 handle_command(_, #st{sel=[]}) ->
     keep.
 
+fake_selection(St) ->
+    wings_dl:fold(fun(#dlo{src_sel=none}, S) ->
+			  %% No selection, try highlighting.
+			  fake_sel_1(S);
+		     (#dlo{src_we=#we{id=Id},src_sel={Mode,Els}}, S) ->
+			  S#st{selmode=Mode,sel=[{Id,Els}]}
+		  end, St).
+
+fake_sel_1(St0) ->
+    case wings_pref:get_value(use_temp_sel) of
+	false -> St0;
+	true ->
+	    {_,X,Y} = wings_wm:local_mouse_state(),
+	    case wings_pick:do_pick(X, Y, St0) of
+		{add,_,St} -> St;
+		_ -> St0
+	    end
+    end.
+
 drag({drag,Drag}) ->
     wings:mode_restriction([vertex,edge,face,body]),
     wings_wm:set_prop(show_info_text, true),
@@ -741,7 +896,6 @@ handle_drop({image,_,#e3d_image{width=W,height=H}=Im}, #st{bb=Uvs0}=St) ->
 	    get_event(St#st{bb=Uvs})
     end;
 handle_drop(_DropData, _) ->
-    %%io:format("~P\n", [_DropData,40]),
     keep.
 
 is_power_of_two(X) ->
@@ -769,101 +923,60 @@ new_geom_state_1(Shs, #st{bb=#uvstate{id=Id,st=#st{shapes=Orig}}}=AuvSt) ->
 	{none,_} -> delete;
 	{{value,We},{value,We}} -> {AuvSt,false};
 	{{value,#we{es=Etab}},{value,#we{es=Etab}}} -> {AuvSt,false};
-	{{value,We},_} -> {rebuild_charts(We, AuvSt,true,[]),true}
+	{{value,We},_} -> {rebuild_charts(We, AuvSt, []),true}
     end.
 
-rebuild_charts(We, #st{shapes=OldCharts}=St,Reuse,ExtraCuts) ->
-    Faces = wings_we:uv_mapped_faces(We),
-    FvUvMap = auv_segment:fv_to_uv_map(Faces, We),
+rebuild_charts(We, St = #st{bb=UVS=#uvstate{st=Old}}, ExtraCuts) ->
+    {Faces,FvUvMap} = auv_segment:fv_to_uv_map(We),
     {Charts0,Cuts0} = auv_segment:uv_to_charts(Faces, FvUvMap, We),
     {Charts1,Cuts} =
 	case ExtraCuts of
-	    [] ->  {Charts0,Cuts0};
+	    [] -> {Charts0,Cuts0};
 	    _ ->
-		Cuts1 = foldl(fun(Edge,Cuts) -> gb_sets:add(Edge,Cuts) end, 
-			      Cuts0, ExtraCuts),
-		auv_segment:normalize_charts(Charts0,Cuts1,We)
+		Cuts1 = gb_sets:union(Cuts0, gb_sets:from_list(ExtraCuts)),
+		auv_segment:normalize_charts(Charts0, Cuts1, We)
 	end,
     Charts2 = auv_segment:cut_model(Charts1, Cuts, We),
-    Old = if Reuse -> gb_trees:values(OldCharts);
-	     true -> []
-	  end,
-    Charts3 = update_charts(Charts2, Old),
-    Charts4 = build_map(Charts3, FvUvMap, []),
-    Charts = gb_trees:from_orddict(keysort(1, Charts4)),
+    Charts = update_uv_tab(Charts2, FvUvMap),
     wings_wm:set_prop(wireframed_objects,
-		      gb_sets:from_list(gb_trees:keys(Charts))),
-    St#st{sel=[],shapes=Charts}.
+		      gb_sets:from_ordset(lists:seq(1, length(Charts2)))),
+    St#st{sel=[],bb=UVS#uvstate{st=Old#st{sel=[]}},shapes=Charts}.
 
-update_charts(Cs, OldCs) ->
-    Id = length(Cs),
-    update_charts_1(Cs, reverse(OldCs), Id, []).
+update_uv_tab(Cs, FvUvMap) ->
+    update_uv_tab_1(Cs, FvUvMap, []).
 
-update_charts_1([{Fs,We0}|Cs], [OldWe|OldCs], Id, Acc) ->
-    We = update_chart(Fs, We0#we{id=Id}, OldWe),
-    update_charts_1(Cs, OldCs, Id-1, [We|Acc]);
-update_charts_1([{Fs,We0}|Cs], [], Id, Acc) ->
-    We = auv_segment:finalize_chart(Fs, We0#we{id=Id}),
-    update_charts_1(Cs, [], Id-1, [We|Acc]);
-update_charts_1([], _, 0, Acc) -> Acc.
-
-update_chart(Fs, We, OldWe) ->
-    case wings_we:visible(OldWe) of
-	Fs -> update_chart_1(Fs, We, OldWe);
-	_ -> auv_segment:finalize_chart(Fs, We)
-    end.
-
-update_chart_1(Fs, We, OldWe) ->
-    case same_topology(Fs, We, OldWe) of
-	false -> auv_segment:finalize_chart(Fs, We);
-	true -> OldWe
-    end.
-
-same_topology([F|Fs], We, OldWe) ->
-    case same_toplolgy_1(F, We, OldWe) of
-	false -> false;
-	true -> same_topology(Fs, We, OldWe)
-    end;
-same_topology([], _, _) -> true.
-
-same_toplolgy_1(F, #we{es=EtabA,fs=FtabA}, #we{es=EtabB,fs=FtabB}) ->
-    EdgeA = gb_trees:get(F, FtabA),
-    EdgeB = gb_trees:get(F, FtabB),
-    same_toplogy_2(F, EtabA, EdgeA, EdgeA, EtabB, EdgeB, EdgeB).
-    
-same_toplogy_2(_F, _EtabA, LastA, LastA, _EtabB, LastB, LastB) -> true;
-same_toplogy_2(F, EtabA, EdgeA, LastA, EtabB, EdgeB, LastB) ->
-    case {gb_trees:get(EdgeA, EtabA),gb_trees:get(EdgeB, EtabB)} of
-	{#edge{vs=Eq,lf=F,ltsu=NextA},#edge{vs=Eq,lf=F,ltsu=NextB}} ->
-	    same_toplogy_2(F, EtabA, NextA, LastA, EtabB, NextB, LastB);
-	{#edge{ve=Eq,rf=F,rtsu=NextA},#edge{vs=Eq,lf=F,ltsu=NextB}} ->
-	    same_toplogy_2(F, EtabA, NextA, LastA, EtabB, NextB, LastB);
-	{#edge{vs=Eq,lf=F,ltsu=NextA},#edge{ve=Eq,rf=F,rtsu=NextB}} ->
-	    same_toplogy_2(F, EtabA, NextA, LastA, EtabB, NextB, LastB);
-	{#edge{ve=Eq,rf=F,rtsu=NextA},#edge{ve=Eq,rf=F,rtsu=NextB}} ->
-	    same_toplogy_2(F, EtabA, NextA, LastA, EtabB, NextB, LastB);
-	{_,_} -> false
-    end.
-
-build_map([#we{id=Id,name=#ch{vmap=Vmap}}=We0|T], FvUvMap, Acc) ->
-    %% XXX Because auv_segment:cut_model/3 distorts the UV coordinates
-    %% (bug in wings_vertex_cmd), we must fetch the UV coordinates
-    %% from the original object.
+update_uv_tab_1([#we{id=Id,name=#ch{vmap=Vmap}}=We0|Cs], FvUvMap, Acc) ->
     Fs = wings_we:visible(We0),
-    Z = zero(),
     UVs0 = wings_face:fold_faces(
 	     fun(F, V, _, _, A) ->
 		     OrigV = auv_segment:map_vertex(V, Vmap),
-		     {X,Y} = gb_trees:get({F,OrigV}, FvUvMap),
-		     [{V,{X,Y,Z}}|A]
+		     [{V,[F|OrigV]}|A]
 	     end, [], Fs, We0),
-    UVs = gb_trees:from_orddict(orddict:from_list(UVs0)),
-    We = We0#we{vp=UVs},
-    build_map(T, FvUvMap, [{Id,We}|Acc]);
-build_map([], _, Acc) -> reverse(Acc).
+    case update_uv_tab_2(sort(UVs0), FvUvMap, 0.0, []) of
+	error ->
+	    %% No UV coordinate for at least some vertices (probably
+	    %% all) in the chart. Throw away this chart.
+	    update_uv_tab_1(Cs, FvUvMap, Acc);
+	UVs1 ->
+	    UVs = gb_trees:from_orddict(UVs1),
+	    We = We0#we{vp=UVs},
+	    update_uv_tab_1(Cs, FvUvMap, [{Id,We}|Acc])
+    end;
+update_uv_tab_1([], _, Acc) ->
+    gb_trees:from_orddict(sort(Acc)).
 
-zero() ->
-    0.0.
+update_uv_tab_2([{V,_}|T], FvUvMap, Z, [{V,_}|_]=Acc) ->
+    update_uv_tab_2(T, FvUvMap, Z, Acc);
+update_uv_tab_2([{V,Key}|T], FvUvMap, Z, Acc) ->
+    case gb_trees:get(Key, FvUvMap) of
+	{X,Y} ->
+	    Pos = {X,Y,Z},
+	    update_uv_tab_2(T, FvUvMap, Z, [{V,Pos}|Acc]);
+	_ ->
+	    %% No UV-coordinate for this vertex. Abandon the entire chart.
+	    error
+    end;
+update_uv_tab_2([], _, _, Acc) -> reverse(Acc).
 
 %% update_selection(GemoSt, AuvSt0) -> AuvSt
 %%  Update the selection in the AutoUV window given a selection
@@ -893,7 +1006,7 @@ update_selection(#st{selmode=Mode,sel=Sel}=St0,
 	     %% Different selection modes. Convert Geom selection to
 	     %% the mode in AutoUV.
 	     St = St0#st{sel=[IdElems]},
-	     #st{sel=[{Id,Elems0}]} = wings_sel:convert_selection(AuvMode, St),
+	     #st{sel=[{Id,Elems0}]} = wings_sel_conv:mode(AuvMode, St),
 	     Elems = gb_sets:to_list(Elems0),
 	     NewSel = update_selection_1(AuvMode, Elems, Charts),
 	     AuvSt0#st{sel=sort(NewSel),sh=false,bb=Uvs#uvstate{st=St0}}
@@ -976,6 +1089,8 @@ body_sel_to_body([], Sel) -> Sel.
 %%  Given the selection in the AutoUV window, update the selection
 %%  in the geometry window.
 
+update_geom_selection(#st{temp_sel={_,_},bb=#uvstate{st=GeomSt}}) ->
+    GeomSt;
 update_geom_selection(#st{sel=[],bb=#uvstate{st=GeomSt}}) ->
     wpa:sel_set(face, [], GeomSt);
 update_geom_selection(#st{selmode=body,sel=Sel,
@@ -1014,11 +1129,48 @@ update_geom_selection(#st{selmode=vertex,sel=Sel,
 
 rotate_chart(Angle, We) ->
     Center = wings_vertex:center(We),
+    rotate_chart(Angle, Center, We).
+
+rotate_chart(Angle, Center, We) ->
     Rot0 = e3d_mat:translate(e3d_vec:neg(Center)),
-    Rot1 = e3d_mat:mul(e3d_mat:rotate(float(trunc(Angle)), {0.0,0.0,1.0}), Rot0),
+    Rot1 = e3d_mat:mul(e3d_mat:rotate(float(Angle), {0.0,0.0,1.0}), Rot0),
     Rot = e3d_mat:mul(e3d_mat:translate(Center), Rot1),
     wings_we:transform_vs(Rot, We).
 
+align_chart(Dir, St = #st{selmode=Mode}) ->
+    wings_sel:map(
+      fun(Sel, We = #we{vp=Vtab,es=Etab}) ->
+	      case gb_sets:to_list(Sel) of
+		  [V1,V2] when Mode == vertex -> 
+		      align_chart(Dir,gb_trees:get(V1,Vtab),
+				  gb_trees:get(V2,Vtab),
+				  We);
+		  [E] when Mode == edge -> 
+		      #edge{vs=V1,ve=V2} = gb_trees:get(E, Etab),
+		      align_chart(Dir,gb_trees:get(V1,Vtab),
+				  gb_trees:get(V2,Vtab),
+				  We);
+		  _ -> align_error()
+	      end
+      end, St).
+
+align_chart(Dir, V1={X1,Y1,_},V2={X2,Y2,_}, We) ->
+    Deg0 = 180.0/math:pi() *
+	case Dir of
+	    align_x -> math:atan2(Y2-Y1,X2-X1);
+	    align_y -> math:atan2(X1-X2,Y2-Y1);
+	    align_xy -> math:atan2(Y2-Y1,X2-X1) -
+			    45/180*math:pi()
+	end,
+    Deg = if abs(Deg0) < 90.0 -> Deg0;
+	     true -> Deg0 + 180
+	  end,
+    Center = e3d_vec:average(V1,V2),
+    rotate_chart(-Deg,Center,We).
+
+align_error() ->
+    wings_u:error("Select two vertices or one edge").
+
 flip_horizontal(We) ->
     flip(e3d_mat:scale(-1.0, 1.0, 1.0), We).
 
@@ -1033,26 +1185,82 @@ flip(Flip, We0) ->
     We = wings_we:transform_vs(T, We0),
     wings_we:invert_normals(We).
 
-remap(Method, #st{sel=Sel,selmode=Mode}=St0) ->
+move_to(Dir,We) ->
+    [V1={X1,Y1,_},V2={X2,Y2,_}] = wings_vertex:bounding_box(We),
+    ChartCenter = {CCX,CCY,CCZ} = e3d_vec:average(V1,V2),
+    Translate
+	= case Dir of
+	      center ->   e3d_vec:sub({0.5,0.5,CCZ}, ChartCenter);
+	      center_x -> e3d_vec:sub({0.5,CCY,CCZ}, ChartCenter);
+	      center_y -> e3d_vec:sub({CCX,0.5,CCZ}, ChartCenter);
+	      bottom ->   {0.0,-Y1,0.0};
+	      top ->      {0.0,1.0-Y2,0.0};   
+	      left ->     {-X1,0.0,0.0};
+	      right ->    {1.0-X2,0.0,0.0}
+	  end,
+    T = e3d_mat:translate(Translate),
+    wings_we:transform_vs(T, We).
+
+stretch(Dir,We) ->
+    [{X1,Y1,_},{X2,Y2,_}] = wings_vertex:bounding_box(We),
+    Center = {CX,CY,CZ} = {X1+(X2-X1)/2, Y1+(Y2-Y1)/2, 0.0},
+    T0 = e3d_mat:translate(e3d_vec:neg(Center)),
+    SX0 = 1.0/(X2-X1), SY0= 1.0/(Y2-Y1),
+    {SX,SY} = case Dir of
+		  max_x -> {SX0, 1.0};
+		  max_y -> {1.0, SY0};
+		  max_uniform when SX0 < SY0 -> 
+		      {SX0, SX0};
+		  _ ->
+		      {SY0, SY0}
+	      end,
+    Stretch = e3d_mat:scale(SX, SY, 1.0),
+    T1 = e3d_mat:mul(Stretch, T0),
+    Pos = case Dir of
+	      max_uniform -> {0.5,0.5,CZ};
+	      max_x -> {0.5,CY,CZ};
+	      max_y -> {CX,0.5,CZ}
+	  end,    
+    T = e3d_mat:mul(e3d_mat:translate(Pos), T1),
+    wings_we:transform_vs(T, We).
+    
+reunfold(#st{sel=Sel,selmode=vertex}=St0) ->
+    %% Check correct pinning.
+    Ch = fun(Vs, _, _) ->
+		 case gb_sets:size(Vs) of
+		     N when N < 2 ->
+			 E = "At least two vertices per chart must be pinned",
+			 wpa:error(E);
+		     _-> ok
+		 end
+	 end,
+    wings_sel:fold(Ch, ok, St0),
+
+    %% OK. Go ahead and re-unfold.
     wings_pb:start("remapping"),
     wings_pb:update(0.001),
     N = length(Sel),
-    GetUV = fun(V,Vtab) -> 
-		    {S,T,_} = gb_trees:get(V,Vtab),
-		    {V,{S,T}}
-	    end,
-    Remap = 
-	fun(Vs, We = #we{vp=Vtab}, I) ->
-		Msg = "chart " ++ integer_to_list(I),
+    R = fun(Vs, #we{vp=Vtab}=We, I) ->
+		Msg = "chart " ++ integer_to_list(I+1),
 		wings_pb:update(I/N, Msg),
-		Pinned = case Mode of
-			     vertex -> 
-				 [GetUV(V,Vtab) || 
-				     V <- gb_sets:to_list(Vs)];
-			     _ -> none
-			 end,
-		{remap(Method, Pinned, We, St0),I+1}
+		Pinned = [begin
+			      {S,T,_} = gb_trees:get(V, Vtab),
+			      {V,{S,T}}
+			  end || V <- gb_sets:to_list(Vs)],
+		{remap(lsqcm, Pinned, We, St0),I+1}
 	end,
+    {St,_} = wings_sel:mapfold(R, 1, St0),
+    wings_pb:done(update_selected_uvcoords(St)).
+
+remap(Method, #st{sel=Sel}=St0) ->
+    wings_pb:start("remapping"),
+    wings_pb:update(0.001),
+    N = length(Sel),
+    Remap = fun(_, We, I) ->
+		    Msg = "chart " ++ integer_to_list(I+1),
+		    wings_pb:update(I/N, Msg),
+		    {remap(Method, none, We, St0),I+1}
+	    end,
     {St,_} = wings_sel:mapfold(Remap, 1, St0),
     wings_pb:done(update_selected_uvcoords(St)).
 
@@ -1066,10 +1274,8 @@ remap(Type, Pinned, #we{name=Ch}=We0, St) ->
     %% Get 3d positions (even for mapped vs).
     Vs3d = orig_pos(We0, St),
     case auv_mapping:map_chart(Type, We0#we{vp=Vs3d}, Pinned) of
-	{error, Msg} -> 
-	    wings_pb:done(),
-	    wings_util:message(Msg),
-	    We0;
+	{error,Msg} -> 
+	    wpa:error(Msg);
 	Vs0 -> 
 	    We1 = We0#we{vp=gb_trees:from_orddict(sort(Vs0))},
 	    Fs = wings_we:visible(We1),
