@@ -8,17 +8,18 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: user_default.erl,v 1.19 2004/11/12 05:29:33 bjorng Exp $
+%%     $Id: user_default.erl,v 1.25 2004/12/18 10:24:06 bjorng Exp $
 %% 
 
 -module(user_default).
 
 -export([help/0,wh/0,
-	 wx/0,wxe/0,wxu/1,wxu/3,wxunref/0,wxundef/0,
+	 wx/0,wxe/0,wxu/1,wxu/3,wxunref/0,wxundef/0,wxcs/0,
+	 wxc/1,wxq/1,
 	 wldiff/1,
 	 lm/0,mm/0]).
 
--import(lists, [foldl/3]).
+-import(lists, [foldl/3,foreach/2]).
 
 help() ->
     shell_default:help(),
@@ -37,6 +38,9 @@ wh() ->
     p("wxundef()  -- print calls to undefined functions\n"),
     p("wxu(M)     -- print uses of module M\n"),
     p("wxu(M, F, A) -- print uses of M:F/A\n"),
+    p("wxc(M)     -- print modules that M calls\n"),
+    p("wxcs()     -- print strong components\n"),
+    p("wxq(Query) -- execute an XREF query\n"),
     p("** Language support **\n"),
     p("wldiff(Lang) -- diff language files against English templates\n"),
     ok.
@@ -48,15 +52,23 @@ wh() ->
 wx() ->
     WingsLib = code:lib_dir(wings),
     WingsEbin = filename:join(WingsLib, "ebin"),
+    case whereis(s) of
+	undefined -> ok;
+	_ -> xref:stop(s)
+    end,
     xref:start(s),
     xref:set_default(s, [{verbose,false},{warnings,false},{builtins,true}]),
     xref:set_library_path(s, code:get_path() -- [WingsEbin]),
     {ok,Ms} = xref:add_directory(s, WingsEbin),
     Dirs = get_plugin_dirs(),
-    foldl(fun(D, N) -> 
-		  {ok,PMs} = xref:add_directory(s, D),
-		  N+length(PMs)
-	  end, length(Ms), Dirs).
+    N = foldl(fun(D, N) -> 
+		      {ok,PMs} = xref:add_directory(s, D),
+		      N+length(PMs)
+	      end, length(Ms), Dirs),
+    xref:q(s, "Wings := \"wings.*\":Mod + \"wp.*\":Mod + \"auv_.*\":Mod "),
+    io:put_chars(" Variable Wings = <all modules in Wings>\n"),
+    io:format(" Modules loaded: ~p\n", [N]),
+    wxundef().
 
 wxe() ->
     Dir = filename:dirname(code:which(gl)),
@@ -64,16 +76,31 @@ wxe() ->
     length(Ms).
 
 wxu(Mod) when is_atom(Mod) ->
-    result(xref:q(s, make_query("domain(E || ~p) - ~p", [Mod,Mod])));
+    wxq(make_query("domain(strict(ME || ~p))", [Mod]));
 wxu({M,_,_}=MFA) ->
-    result(xref:q(s, make_query("domain(E || ~p) - ~p", [MFA,M]))).
+    wxq(make_query("domain(E || ~p) - ~p", [MFA,M])).
 
 wxu(M, F, A) ->
-    MFA = {M,F,A},
-    result(xref:q(s, make_query("domain(E || ~p) - ~p", [MFA,M]))).
+    wxu({M,F,A}).
+
+wxc(Mod) when is_atom(Mod) ->
+    wxq(make_query("range(strict(ME|~p))*Wings", [Mod])).
+
+wxcs() ->
+    print_components(xref:q(s, make_query("components ME", []))).
+
+wxq(Q) ->
+    result(xref:q(s, Q)).
+
+print_components({ok,Cs}) ->
+    foreach(fun(C) -> io:format("~p\n", [C]) end, Cs);
+print_components(Other) -> Other.
 
 wxundef() ->
-    xref:analyze(s, undefined_function_calls).
+    case xref:analyze(s, undefined_function_calls) of
+	{ok,Undef} -> Undef;
+	Other -> Other
+    end.
 
 wxunref() ->
     {ok,Unref0} = xref:analyze(s, exports_not_used),
@@ -126,7 +153,9 @@ result({ok,List}) ->
 result(Other) -> Other.
 
 make_query(Format, Args) ->
-    lists:flatten(io_lib:format(Format, Args)).
+    R = lists:flatten(io_lib:format(Format, Args)),
+    %%io:format("~p\n", [R]),
+    R.
 
 %%%
 %%% Language support.
