@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_drag.erl,v 1.185 2004/10/15 15:40:30 bjorng Exp $
+%%     $Id: wings_drag.erl,v 1.192 2005/01/31 14:34:13 dgud Exp $
 %%
 
 -module(wings_drag).
@@ -86,15 +86,17 @@ setup_mode(Flags, Falloff) ->
     end.
     
 standard_mode_fun(Falloff) ->
-    Help0 = ?STR(standard_mode_fun,1,"[Tab] Numeric entry  [Shift] and/or [Ctrl] Constrain"),
+    Help0 = ?__(1,"[Shift] and/or [Ctrl] Constrain"),
     Help = case Falloff of
 	       none -> Help0;
 	       _ ->
-		   lists:flatten([?STR(standard_mode_fun,2,"[+] or [-] Adjust Radius  ")|Help0])
+		   M = [?__(2,"[+] or [-] Adjust Radius"),
+			"  "|Help0],
+		   lists:flatten(M)
 	   end,
     fun(help, _) -> Help;
        (_, _) -> none
-     end.
+    end.
 
 unit_scales(Units) ->
     #view{distance=D} = wings_view:current(),
@@ -315,16 +317,18 @@ do_drag(Drag0, _) ->
     keep.
 
 gl_rescale_normal() ->
-    case wings_util:is_gl_ext('GL_EXT_rescale_normal') of
+    case wings_gl:is_ext('GL_EXT_rescale_normal') of
 	true -> ?GL_RESCALE_NORMAL;
 	false -> ?GL_NORMALIZE
     end.
 
 help_message(#drag{unit=Unit,mode_fun=ModeFun,mode_data=ModeData}) ->
-    Accept = wings_util:button_format(wings_s:accept()),
+    Accept = wings_msg:button_format(wings_s:accept()),
     ZMsg = zmove_help(Unit),
-    Cancel = wings_util:button_format([], [],wings_s:cancel()),
-    Msg = wings_camera:join_msg([Accept,ZMsg,Cancel]),
+    Cancel = wings_msg:button_format([], [],wings_s:cancel()),
+    NumEntry = ?__(1,"Numeric entry"),
+    Tab = wings_util:key_format("[Tab]", NumEntry),
+    Msg = wings_msg:join([Accept,ZMsg,Cancel,Tab]),
     MsgRight = ModeFun(help, ModeData),
     wings_wm:message(Msg, MsgRight).
 
@@ -332,15 +336,15 @@ zmove_help([_]) -> [];
 zmove_help([_,_]) -> [];
 zmove_help([_,_,falloff]) -> [];
 zmove_help([_,_,dz|_]) ->
-    zmove_help_1(?STR(zmove_help_1,1,"Drag to move along Z"));
+    zmove_help_1(?__(1,"Drag to move along Z"));
 zmove_help([_,_,_|_]) ->
-    zmove_help_1(?STR(zmove_help_1,2,"Drag to adjust third parameter")).
+    zmove_help_1(?__(2,"Drag to adjust third parameter")).
 
 zmove_help_1(Msg) ->
     case wings_pref:get_value(camera_mode) of
-	tds -> wings_camera:mod_format(?CTRL_BITS, 3, Msg);
-	blender -> wings_camera:mod_format(?CTRL_BITS, 3, Msg);
-	_ -> wings_camera:mod_format(0, 2, Msg)
+	tds -> wings_msg:mod_format(?CTRL_BITS, 3, Msg);
+	blender -> wings_msg:mod_format(?CTRL_BITS, 3, Msg);
+	_ -> wings_msg:mod_format(0, 2, Msg)
     end.
 
 get_drag_event(Drag) ->
@@ -369,8 +373,8 @@ handle_drag_event(#mousebutton{button=3,state=?SDL_RELEASED,mod=Mod}=Ev,
 	true ->
 	    handle_drag_event_0(Ev, Drag)
     end;
-handle_drag_event(Event, Drag) ->
-    case wings_camera:event(Event, fun() -> redraw(Drag) end) of
+handle_drag_event(Event, Drag = #drag{st=St}) ->
+    case wings_camera:event(Event, St, fun() -> redraw(Drag) end) of
 	next -> handle_drag_event_0(Event, Drag);
 	Other ->
 	    %% Clear any potential marker for an edge about to be
@@ -407,13 +411,9 @@ handle_drag_event_1(#mousemotion{}=Ev, Drag0) ->
     Drag = motion(Ev, Drag0),
     get_drag_event(Drag);
 handle_drag_event_1(#mousebutton{button=1,x=X,y=Y,mod=Mod,state=?SDL_RELEASED}, Drag0) ->
-    ungrab(Drag0),
     Ev = #mousemotion{x=X,y=Y,state=0,mod=Mod},
-    #drag{last_move=Move} = Drag = ?SLOW(motion(Ev, Drag0)),
-    St = normalize(Move, Drag),
-    DragEnded = {new_state,St#st{drag_args=Move}},
-    wings_wm:later(DragEnded),
-    pop;
+    Drag = ?SLOW(motion(Ev, Drag0)),
+    quit_drag(Drag);
 handle_drag_event_1({drag_arguments,Move}, Drag0) ->
     ungrab(Drag0),
     Drag1 = possible_falloff_update(Move, Drag0),
@@ -432,19 +432,32 @@ handle_drag_event_1(view_changed, Drag) ->
 handle_drag_event_1({action,{drag_arguments,_}=DragArgs}, _) ->
     wings_wm:later(DragArgs);
 handle_drag_event_1(Event, #drag{st=St}=Drag0) ->
-    Drag = case wings_hotkey:event(Event) of
-	       next -> Drag0;
-	       {view,smoothed_preview} -> Drag0;
-	       {view,Cmd} ->
-		   wings_view:command(Cmd, St),
-		   view_changed(Drag0);
-	       {select,less} ->
-		   magnet_radius(-1, Drag0);
-	       {select,more} ->
-		   magnet_radius(1, Drag0);
-	       _Other -> Drag0
-	   end,
-    get_drag_event(Drag).
+    case wings_hotkey:event(Event,St) of
+	next ->
+	    get_drag_event(Drag0);
+	{view,smoothed_preview} -> 
+	    get_drag_event(Drag0);
+	{view,Cmd} ->
+	    wings_view:command(Cmd, St),
+	    Drag = view_changed(Drag0),
+	    get_drag_event(Drag);
+	{select,less} ->
+	    Drag = magnet_radius(-1, Drag0),
+	    get_drag_event(Drag);
+	{select,more} ->
+	    Drag = magnet_radius(1, Drag0),
+	    get_drag_event(Drag);
+	_Other ->
+	    wings_wm:later(Event),
+	    quit_drag(Drag0)
+    end.
+
+quit_drag(#drag{last_move=Move} = Drag) ->
+    ungrab(Drag),
+    St = normalize(Move, Drag),
+    DragEnded = {new_state,St#st{drag_args=Move}},
+    wings_wm:later(DragEnded),
+    pop.
 
 ungrab(#drag{x=Ox,y=Oy}) ->
     wings_wm:release_focus(),
@@ -458,7 +471,7 @@ numeric_input(Drag0) ->
     {_,X,Y} = wings_wm:local_mouse_state(),
     Ev = #mousemotion{x=X,y=Y,state=0,mod=0},
     {Move0,Drag} = mouse_translate(Ev, Drag0),
-    wings_ask:dialog(?STR(numeric_input,1,"Numeric Input"), 
+    wings_ask:dialog(?__(1,"Numeric Input"), 
 		     make_query(Move0, Drag),
 		     fun(Res) ->
 			     {drag_arguments,make_move(Res, Drag)}
@@ -473,7 +486,8 @@ make_query_1([U0|Units], [V|Vals]) ->
 	    [{hframe,[{text,V*100.0,percent_qrange(U0)},{label,"%"}]}|
 	     make_query_1(Units, Vals)];
 	angle ->
-	    [{hframe,[{label,?STR(make_query_1,2,"A")},{text,V,qrange(U0)},{label,[?DEGREE]}]}|
+	    [{hframe,[{label,?__(2,"A")},
+		      {text,V,qrange(U0)},{label,[?DEGREE]}]}|
 	     make_query_1(Units, Vals)];
 	U ->
 	    [{hframe,[{label,qstr(U)},{text,V,qrange(U0)}]}|
@@ -481,12 +495,12 @@ make_query_1([U0|Units], [V|Vals]) ->
     end;
 make_query_1([], []) -> [].
 
-qstr(distance) -> ?STR(qstr,1,"Dx");
-qstr(dx) -> ?STR(qstr,2,"Dx");
-qstr(dy) -> ?STR(qstr,3,"Dy");
-qstr(dz) ->  ?STR(qstr,4,"R");
-qstr(falloff) ->  ?STR(qstr,5,"A");
-qstr(angle) ->  ?STR(qstr,6,"A");
+qstr(distance) -> ?__(1,"Dx");
+qstr(dx) -> ?__(2,"Dx");
+qstr(dy) -> ?__(3,"Dy");
+qstr(dz) ->  ?__(4,"Dz");
+qstr(falloff) ->  ?__(5,"R");
+qstr(angle) ->  ?__(6,"A");
 qstr(Atom) -> atom_to_list(Atom).
 
 qrange({_,{_,_}=Range}) -> [{range,Range}];
@@ -583,7 +597,7 @@ mouse_range(#mousemotion{x=X0,y=Y0,state=Mask},
 		  unit_sc=UnitScales}=Drag) ->
     %%io:format("Mouse Range ~p ~p~n", [{X0,Y0}, {OX,OY,Xs0,Ys0}]),
     {X,Y} = wings_wm:local2global(X0, Y0),
-    case wings_util:lowpass(X- OX, Y-OY) of
+    case wings_pref:lowpass(X- OX, Y-OY) of
 	{0,0} ->
 	    {mouse_scale([Xs0,-Ys0,-Zs0], UnitScales),
 	     Drag#drag{xt=0,yt=0}};
