@@ -3,12 +3,12 @@
 %%
 %%     This module draws objects using OpenGL.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_draw.erl,v 1.206 2004/06/05 17:55:49 bjorng Exp $
+%%     $Id: wings_draw.erl,v 1.216 2005/01/15 12:51:18 bjorng Exp $
 %%
 
 -module(wings_draw).
@@ -66,7 +66,7 @@ prepare_dlists(#st{shapes=Shs}) ->
 prepare_fun(eol, [#we{perm=Perm}|Wes]) when ?IS_NOT_VISIBLE(Perm) ->
     prepare_fun(eol, Wes);
 prepare_fun(eol, [We|Wes]) ->
-    D = #dlo{src_we=We},
+    D = #dlo{src_we=We,open=wings_we:any_hidden(We)},
     {changed_we(D, D),Wes};
 prepare_fun(eol, []) ->
     eol;
@@ -78,6 +78,8 @@ prepare_fun(#dlo{}=D, [#we{perm=Perm}|Wes]) when ?IS_NOT_VISIBLE(Perm) ->
 prepare_fun(#dlo{src_we=We,split=#split{}=Split}=D, [We|Wes]) ->
     {D#dlo{src_we=We,split=Split#split{orig_we=We}},Wes};
 prepare_fun(#dlo{src_we=We}=D, [We|Wes]) ->
+    %% No real change - take the latest We for possible speed-up
+    %% of further comparisons.
     {D#dlo{src_we=We},Wes};
 prepare_fun(#dlo{src_we=#we{id=Id}}=D, [#we{id=Id}=We1|Wes]) ->
     prepare_fun_1(D, We1, Wes);
@@ -98,23 +100,26 @@ prepare_fun_1(#dlo{src_we=#we{perm=Perm0}=We0}=D, #we{perm=Perm1}=We, Wes) ->
     end.
 
 prepare_fun_2(#dlo{proxy_data=Proxy,ns=Ns}=D, We, Wes) ->
-    {changed_we(D, #dlo{src_we=We,mirror=none,proxy_data=Proxy,ns=Ns}),Wes}.
+    Open = wings_we:any_hidden(We),
+    {changed_we(D, #dlo{src_we=We,open=Open,
+			mirror=none,proxy_data=Proxy,ns=Ns}),Wes}.
 
 only_permissions_changed(#we{perm=P}, #we{perm=P}) -> false;
 only_permissions_changed(We0, We1) -> We0#we{perm=0} =:= We1#we{perm=0}.
     
 invalidate_by_mat(Changed0) ->
-    Changed = gb_sets:from_list(Changed0),
+    Changed = ordsets:from_list(Changed0),
     wings_dl:map(fun(D, _) -> invalidate_by_mat(D, Changed) end, []).
 
 invalidate_by_mat(#dlo{work=none,vs=none,smooth=none,proxy_faces=none}=D, _) ->
     %% Nothing to do.
     D;
 invalidate_by_mat(#dlo{src_we=We}=D, Changed) ->
-    Used = wings_material:used_materials_we(We),
-    case gb_sets:is_empty(gb_sets:intersection(Used, Changed)) of
-	true -> D;
-	false -> D#dlo{work=none,edges=none,vs=none,smooth=none,proxy_faces=none}
+    Used = wings_facemat:used_materials(We),
+    case ordsets:intersection(Used, Changed) of
+	[] -> D;
+	[_|_] ->
+	    D#dlo{work=none,edges=none,vs=none,smooth=none,proxy_faces=none}
     end.
 
 invalidate_sel(#dlo{src_we=#we{id=Id},src_sel=SrcSel}=D,
@@ -273,7 +278,7 @@ more_need([], _, Acc) ->
 
 wins_of_same_class() ->
     case wings_wm:get_prop(display_lists) of
-	geom_display_lists -> wings_util:geom_windows();
+	geom_display_lists -> wings_u:geom_windows();
 	_ -> [wings_wm:this()]
     end.
 
@@ -310,16 +315,16 @@ update_fun_2(smooth, #dlo{smooth=none,proxy_data=none}=D, St) ->
     {List,Tr} = smooth_dlist(D, St),
     D#dlo{smooth=List,transparent=Tr};
 update_fun_2(smooth, #dlo{smooth=none}=D, St) ->
-    We = wings_subdiv:smooth_we(D),
+    We = wings_proxy:smooth_we(D),
     {List,Tr} = smooth_dlist(We, St),
     D#dlo{smooth=List,transparent=Tr};
-update_fun_2({vertex,PtSize}, #dlo{vs=none,src_we=#we{vp=Vtab}}=D, _) ->
+update_fun_2({vertex,PtSize}, #dlo{vs=none,src_we=We}=D, _) ->
     UnselDlist = gl:genLists(1),
     gl:newList(UnselDlist, ?GL_COMPILE),
     gl:pointSize(PtSize),
     gl:color3b(0, 0, 0),
     gl:'begin'(?GL_POINTS),
-    pump_vertices(gb_trees:values(Vtab)),
+    pump_vertices(visible_vertices(We)),
     gl:'end'(),
     gl:endList(),
     D#dlo{vs=UnselDlist};
@@ -342,7 +347,7 @@ update_fun_2(edges, #dlo{edges=none,ns=Ns}=D, _) ->
 update_fun_2(normals, D, _) ->
     make_normals_dlist(D);
 update_fun_2(proxy, D, St) ->
-    wings_subdiv:update(D, St);
+    wings_proxy:update(D, St);
 update_fun_2(_, D, _) -> D.
 
 make_edge_dl(Ns) ->
@@ -398,6 +403,17 @@ force_flat({call,_,Faces}, Color) ->
     wings_draw_util:force_flat_color(Faces, Color);
 force_flat(_, _) -> none.
 
+visible_vertices(#we{vp=Vtab0}=We) ->
+    case wings_we:any_hidden(We) of
+	false -> gb_trees:values(Vtab0);
+	true ->
+	    Vis0 = wings_we:visible_vs(We),
+	    Vis = sofs:from_external(Vis0, [vertex]),
+	    Vtab = sofs:from_external(gb_trees:to_list(Vtab0),
+				      [{vertex,position}]),
+	    sofs:to_external(sofs:image(Vtab, Vis))
+    end.
+    
 %%%
 %%% Update the selection display list.
 %%%
@@ -410,11 +426,17 @@ update_sel_dlist() ->
 update_sel(#dlo{src_we=We}=D) when ?IS_LIGHT(We) -> {D,[]};
 update_sel(#dlo{sel=none,src_sel={body,_}}=D) ->
     update_sel_all(D);
-update_sel(#dlo{sel=none,src_sel={face,Faces},src_we=#we{fs=Ftab}}=D) ->
+update_sel(#dlo{split=none,sel=none,src_sel={face,Faces},
+		src_we=#we{fs=Ftab}}=D) ->
+    %% If we are not dragging (no dlists splitted), we can
+    %% optimize the showing of the selection.
     case gb_trees:size(Ftab) =:= gb_sets:size(Faces) of
 	true -> update_sel_all(D);
 	false -> update_face_sel(gb_sets:to_list(Faces), D)
     end;
+update_sel(#dlo{sel=none,src_sel={face,Faces}}=D) ->
+    %% We are dragging. Don't try to be tricky here.
+    update_face_sel(gb_sets:to_list(Faces), D);
 update_sel(#dlo{sel=none,src_sel={edge,Edges}}=D) ->
     #dlo{src_we=#we{es=Etab,vp=Vtab}} = D,
     List = gl:genLists(1),
@@ -512,16 +534,17 @@ split_1(D, Vs, St) ->
     split_2(D, Vs, update_materials(D, St)).
 
 split_2(#dlo{mirror=M,src_sel=Sel,src_we=#we{fs=Ftab}=We,
-	     proxy_data=Pd,ns=Ns0,needed=Needed}=D, Vs0, St) ->
+	     proxy_data=Pd,ns=Ns0,needed=Needed,open=Open}=D, Vs0, St) ->
     Vs = sort(Vs0),
-    Faces = vs_to_faces(Vs, We),
+    Faces = wings_we:visible(wings_face:from_vs(Vs, We), We),
     StaticVs = static_vs(Faces, Vs, We),
 
-    {Work,FtabDyn} = split_faces(D, Ftab, Faces, St),
+    VisFtab = wings_we:visible(gb_trees:to_list(Ftab), We),
+    {Work,FtabDyn} = split_faces(D, VisFtab, Faces, St),
     StaticEdgeDl = make_static_edges(Faces, D),
     {DynVs,VsDlist} = split_vs_dlist(Vs, StaticVs, Sel, We),
 
-    WeDyn = wings_material:cleanup(We#we{fs=gb_trees:from_orddict(FtabDyn)}),
+    WeDyn = wings_facemat:gc(We#we{fs=gb_trees:from_orddict(FtabDyn)}),
     DynPlan = wings_draw_util:prepare(FtabDyn, We, St),
     StaticVtab = insert_vtx_data(StaticVs, We#we.vp, []),
 
@@ -529,7 +552,7 @@ split_2(#dlo{mirror=M,src_sel=Sel,src_we=#we{fs=Ftab}=We,
 		   dyn_plan=DynPlan,orig_ns=Ns0,orig_we=We},
     #dlo{work=Work,edges=[StaticEdgeDl],mirror=M,vs=VsDlist,
 	 src_sel=Sel,src_we=WeDyn,split=Split,proxy_data=Pd,
-	 needed=Needed}.
+	 needed=Needed,open=Open}.
 
 remove_stale_ns(none, _) -> none;
 remove_stale_ns(Ns, Ftab) ->
@@ -543,16 +566,6 @@ remove_stale_ns_1([{F,_}=Pair|Fs], Ftab, Acc) ->
 remove_stale_ns_1([], _, Acc) ->
     gb_trees:from_orddict(reverse(Acc)).
 
-vs_to_faces(Vs, We) ->
-    Fun = fun(_, Face, _, A) -> [Face|A] end,
-    vs_to_faces_1(Vs, Fun, We, []).
-
-vs_to_faces_1([V|Vs], Fun, We, Acc0) ->
-    Acc = wings_vertex:fold(Fun, Acc0, V, We),
-    vs_to_faces_1(Vs, Fun, We, Acc);
-vs_to_faces_1([], _, We, Acc) ->
-    wings_we:visible(ordsets:from_list(Acc), We).
-
 static_vs(Fs, Vs, We) ->
     VsSet = gb_sets:from_ordset(Vs),
     Fun = fun(V, _, _, A) ->
@@ -568,8 +581,7 @@ static_vs_1([F|Fs], Fun, We, Acc) ->
 static_vs_1([], _, _, Acc) -> ordsets:from_list(Acc).
 
 split_faces(#dlo{needed=Need}=D, Ftab0, Fs0, St) ->
-    Ftab1 = gb_trees:to_list(Ftab0),
-    Ftab = sofs:from_external(Ftab1, [{face,data}]),
+    Ftab = sofs:from_external(Ftab0, [{face,data}]),
     Fs = sofs:from_external(Fs0, [face]),
     case member(work, Need) orelse member(smooth, Need) of
 	false ->
@@ -606,7 +618,7 @@ insert_vtx_data([V|Vs], Vtab, Acc) ->
     insert_vtx_data(Vs, Vtab, [{V,gb_trees:get(V, Vtab)}|Acc]);
 insert_vtx_data([], _, Acc) -> reverse(Acc).
 
-split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}) ->
+split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}=We) ->
     case wings_pref:get_value(vertex_size) of
 	0.0 -> {none,none};
 	PtSize -> 
@@ -624,7 +636,8 @@ split_vs_dlist(Vs, StaticVs, {vertex,SelVs0}, #we{vp=Vtab}) ->
 	    gl:'begin'(?GL_POINTS),
 	    List0 = sofs:from_external(gb_trees:to_list(Vtab), [{vertex,info}]),
 	    List1 = sofs:drestriction(List0, DynVs),
-	    List = sofs:to_external(List1),
+	    List2 = sofs:to_external(List1),
+	    List = wings_we:visible_vs(List2, We),
 	    foreach(fun({_,Pos}) ->
 			    gl:vertex3fv(Pos)
 		    end, List),
@@ -885,21 +898,13 @@ draw_uv_faces([], _) -> ok.
 %%% Smooth drawing.
 %%%
 
-smooth_dlist(#dlo{src_we=#we{he=Htab0,fs=Ftab,mirror=Face}=We,ns=Ns0}=D, St) ->
+smooth_dlist(#dlo{src_we=#we{}=We,ns=Ns0}=D, St) ->
     Ns1 = foldl(fun({F,[N|_]}, A) -> [{F,N}|A];
 		   ({F,{N,_,_}}, A) -> [{F,N}|A]
 		end, [], gb_trees:to_list(Ns0)),
     Ns = reverse(Ns1),
-    case gb_trees:is_defined(Face, Ftab) of
-	false ->
-	    Flist = wings_we:normals(Ns, We),
-	    smooth_faces(Flist, D, St);
-	true ->
-	    Edges = wings_face:outer_edges([Face], We),
-	    Htab = gb_sets:union(Htab0, gb_sets:from_list(Edges)),
-	    Flist = wings_we:normals(Ns, We#we{he=Htab}),
-	    smooth_faces(Flist, D, St)
-    end;
+    Flist = wings_we:normals(Ns, We),
+    smooth_faces(Flist, D, St);
 smooth_dlist(We, St) ->
     D = update_normals(changed_we(#dlo{}, #dlo{src_we=We})),
     smooth_dlist(D, St).
