@@ -3,66 +3,74 @@
 %%
 %%     This module contains most of the face commands.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_face_cmd.erl,v 1.109 2004/11/13 04:39:26 bjorng Exp $
+%%     $Id: wings_face_cmd.erl,v 1.132 2005/01/30 06:37:22 bjorng Exp $
 %%
 
 -module(wings_face_cmd).
 -export([menu/3,command/2]).
--export([dissolve/1,dissolve/2,outer_edge_partition/2,mirror_faces/2,
-	 set_color/2, force_bridge/5]).
+-export([mirror_faces/2,set_color/2,force_bridge/5]).
 
 -include("wings.hrl").
--import(lists, [map/2,foldl/3,reverse/1,sort/1,keysort/2,
+-import(lists, [map/2,foldl/3,reverse/1,reverse/2,sort/1,keysort/2,
 		keymember/3,keysearch/3,keydelete/3,
 		member/2,seq/2,last/1]).
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
-    Menu = [{basic,{?STR(menu,1,"Face operations"),ignore}},
+    Menu = [{basic,{?__(1,"Face operations"),ignore}},
 	    {basic,separator},
-	    {?STR(menu,2,"Move"),{move,Dir},[],[magnet]},
+	    {?__(2,"Move"),{move,Dir},[],[magnet]},
 	    wings_menu_util:rotate(St),
 	    wings_menu_util:scale(St),
 	    separator,
-	    {?STR(menu,3,"Extrude"),{extrude,Dir}},
-	    {?STR(menu,4,"Extrude Region"),{extrude_region,Dir}},
-	    {?STR(menu,5,"Extract Region"),{extract_region,Dir}},
+	    {?__(3,"Extrude"),{extrude,Dir}},
+	    {?__(4,"Extrude Region"),{extrude_region,Dir}},
+	    {?__(5,"Extract Region"),{extract_region,Dir}},
 	    separator,
 	    wings_menu_util:flatten(),
 	    separator,
-	    {?STR(menu,6,"Inset"),inset,?STR(menu,7,"Inset a face inside the selected face")},
-	    {?STR(menu,8,"Intrude"),intrude,?STR(menu,9,"Carve out interior of object, making selected faces holes")},
-	    {?STR(menu,10,"Bevel"),bevel,?STR(menu,11,"Round off edges of selected faces")},
-	    {?STR(menu,12,"Bridge"),bridge,?STR(menu,13,"Create a bridge or tunnel between two faces")},
+	    {?__(6,"Inset"),inset,
+	     ?__(7,"Inset a face inside the selected face")},
+	    {?__(8,"Intrude"),intrude,
+	     ?__(9,"Carve out interior of object, making selected faces holes")},
+	    {?__(10,"Bevel"),bevel,
+	     ?__(11,"Round off edges of selected faces")},
+	    {?__(12,"Bridge"),bridge,
+	     ?__(13,"Create a bridge or tunnel between two faces")},
 	    {advanced,separator},
-	    {?STR(menu,14,"Bump"),bump,?STR(menu,15,"Create bump of selected faces")},
-	    {advanced,{?STR(menu,16,"Lift"),{lift,lift_fun(St)}}},
-	    {advanced,{?STR(menu,17,"Put On"),put_on_fun(),
-		       {?STR(menu,18,"Move and rotate object, aligning the selected face to another element"),[],
-		        ?STR(menu,19,"Clone object on to one or more elements")},[]}},
+	    {?__(14,"Bump"),bump,
+	     ?__(15,"Create bump of selected faces")},
+	    {advanced,{?__(16,"Lift"),{lift,lift_fun(St)}}},
+	    {advanced,{?__(17,"Put On"),put_on_fun(),
+		       {?__(18,"Move and rotate object, aligning the selected face to another element"),[],
+		        ?__(19,"Clone object on to one or more elements")},[]}},
 	    separator,
-	    {?STR(menu,20,"Mirror"),mirror_fun(),
-	     {?STR(menu,21,"Mirror object around selected faces and merge to object"),[],
-	  	?STR(menu,22,"Mirror and create separate objects")},[]},
-    	    {?STR(menu,23,"Dissolve"),dissolve,?STR(menu,24,"Eliminate all edges between selected faces")},
-	    {?STR(menu,25,"Collapse"),collapse,?STR(menu,26,"Delete faces, replacing them with vertices")},
-	    {?STR(menu,27,"Smooth"),smooth,
-		?STR(menu,28,"Subdivide selected faces to smooth them (Catmull-Clark)")},
-	    {?STR(menu,29,"Tesselate"),{subdivide,wings_tesselation:submenu()}},
+	    {?__(20,"Mirror"),mirror_fun(),
+	     {?__(21,"Mirror object around selected faces and merge to object"),[],
+	      ?__(22,"Mirror and create separate objects")},[]},
+    	    {?__(23,"Dissolve"),dissolve,
+	     ?__(24,"Eliminate all edges between selected faces")},
+	    {?__(25,"Collapse"),collapse,
+	     ?__(26,"Delete faces, replacing them with vertices")},
+	    {?__(27,"Smooth"),smooth,
+	     ?__(28,"Subdivide selected faces to smooth them (Catmull-Clark)")},
+	    {?__(29,"Tesselate"),{subdivide,wings_tesselation:submenu()}},
+	    separator,
+	    {?__(32,"Hide"),hide,?__(33,"Hide the selected faces")},
 	    separator] ++ wings_material:material_menu(St) ++
-	[{?STR(menu,30,"Vertex Color"),vertex_color,
-	  ?STR(menu,31,"Apply vertex colors to selected faces")}],
+	[{?__(30,"Vertex Color"),vertex_color,
+	  ?__(31,"Apply vertex colors to selected faces")}],
     wings_menu:popup_menu(X, Y, face, Menu).
 
 lift_fun(St) ->
     fun(help, _Ns) ->
-	    {?STR(lift_fun,1,"Lift, rotating face around edge or vertex"),[],
-	     ?STR(lift_fun,2,"Lift in std. directions")};
+	    {?__(1,"Lift, rotating face around edge or vertex"),[],
+	     ?__(2,"Lift in std. directions")};
        (1, Ns) ->
 	    Funs = lift_selection(rotate, St),
 	    wings_menu:build_command({'ASK',Funs}, Ns);
@@ -136,7 +144,10 @@ command({subdivide,Subdivide}, St) ->
 command(vertex_color, St) ->
     wings_color:choose(fun(Color) ->
 			       set_color(Color, St)
-		       end).
+		       end);
+command(hide, St) ->
+    {save_state,hide_faces(St)}.
+    
 
 %%%
 %%% Extrude, Extrude Region, and Inset commands.
@@ -173,32 +184,27 @@ extrude_region_1([Faces0|Rs0]=Rs, We0, Acc) ->
 	    extrude_region_1(Rs0, We0, [Face|Acc]);
 	_Other ->
 	    We = wings_extrude_face:faces(Acc, We0),
-	    extrude_region_2(Rs, We)
+	    wings_extrude_face:regions(Rs, We)
     end;
 extrude_region_1([], We, Faces) ->
     wings_extrude_face:faces(Faces, We).
 
-extrude_region_2([Faces|Rs], We0) ->
-    We = wings_extrude_face:region(Faces, We0),
-    extrude_region_2(Rs, We);
-extrude_region_2([], We) -> We.
-
 extrude_region_vmirror(_, #we{mirror=none}=We) -> We;
 extrude_region_vmirror(OldWe, #we{mirror=Face0}=We0) ->
     %% Merge the mirror face and any newly created faces to one new mirror face
     %% and flatten it.
     FaceSet = gb_sets:singleton(Face0),
     Bordering = wings_face:extend_border(FaceSet, We0),
-    NewFaces = wings_we:new_items(face, OldWe, We0),
-    BorderingNew = gb_sets:intersection(Bordering, NewFaces),
-    case gb_sets:is_empty(BorderingNew) of
+    NewFaces = wings_we:new_items_as_gbset(face, OldWe, We0),
+    Dissolve0 = gb_sets:intersection(Bordering, NewFaces),
+    case gb_sets:is_empty(Dissolve0) of
 	true -> We0;
 	false ->
-	    Dissolve = gb_sets:union(FaceSet, BorderingNew),
-	    We1 = dissolve(Dissolve, We0),
-	    [Face] = NewFace = gb_sets:to_list(wings_we:new_items(face, We0, We1)),
-	    We = wings_material:assign('_hole_', NewFace, We1),
-	    wings_util:mirror_flatten(OldWe, We#we{mirror=Face})
+	    Dissolve = gb_sets:insert(Face0, Dissolve0),
+	    We1 = wings_dissolve:faces(Dissolve, We0),
+	    [Face] = NewFace = wings_we:new_items_as_ordset(face, We0, We1),
+	    We = wings_facemat:assign('_hole_', NewFace, We1),
+	    wings_we:mirror_flatten(OldWe, We#we{mirror=Face})
     end.
 
 %%%
@@ -208,122 +214,27 @@ extrude_region_vmirror(OldWe, #we{mirror=Face0}=We0) ->
 extract_region(Type, St0) ->
     St1 = wings_sel:fold(
 	    fun(Faces, We0, #st{sel=Sel0,onext=Oid}=S0) ->
-		    We = We0#we{mirror=none},
+		    We = wings_dissolve:complement(Faces, We0),
 		    S = wings_shape:insert(We, extract, S0),
 		    Sel = [{Oid,Faces}|Sel0],
 		    S#st{sel=Sel}
 	    end, St0#st{sel=[]}, St0),
     Sel = St1#st.sel,
-    St2 = wings_sel:set(Sel, St1),
-    St3 = extract_inverse(St2),
-    St4 = dissolve(St3),
-    St = wings_sel:set(Sel, St4),
+    St = wings_sel:set(Sel, St1),
     wings_move:setup(Type, St).
-
-extract_inverse(St) ->
-    Sel = wings_sel:fold(
-	    fun(Faces, #we{id=Id}=We, A) ->
-		    Diff = wings_sel:inverse_items(face, Faces, We),
-		    case gb_sets:is_empty(Diff) of
-			true -> A;
-			false -> [{Id,Diff}|A]
-		    end
-	    end, [], St),
-    wings_sel:set(Sel, St).
     
 %%%
 %%% The Dissolve command.
 %%%
 
 dissolve(St0) ->
-    {St,Sel} = wings_sel:mapfold(fun dissolve/3, [], St0),
+    {St,Sel} = wings_sel:mapfold(fun dissolve_sel/3, [], St0),
     wings_sel:set(Sel, St).
 
-dissolve(Faces, We0) ->
-    {We,_} = dissolve(Faces, We0, []),
-    We.
-
-dissolve(Faces, #we{id=Id}=We0, Acc) ->
-    We = dissolve_1(Faces, We0),
-    case wings_we:is_consistent(We) of
-	true ->
-	    Sel = wings_we:new_items(face, We0, We),
-	    {We,[{Id,Sel}|Acc]};
-	false ->
-	    wings_util:error(?STR(dissolve,1,"Dissolving would cause an inconsistent object structure."))
-    end.
-		  
-dissolve_1(Faces, We) ->
-    case gb_sets:is_empty(Faces) of
-	true -> We;
-	false -> wings_we:vertex_gc(dissolve_2(Faces, We#we{vc=undefined}))
-    end.
-
-dissolve_2(Faces, We) ->
-    Parts = wings_sel:face_regions(Faces, We),
-    dissolve_3(Parts, We).
-
-dissolve_3([Faces|T], We0) ->
-    Face = gb_sets:smallest(Faces),
-    Mat = wings_material:get(Face, We0),
-    We1 = wings_material:delete_faces(Faces, We0),
-    Parts = outer_edge_partition(Faces, We1),
-    We2 = do_dissolve(Faces, Parts, Mat, We0, We1),
-    We = foldl(fun(_, bad_edge) -> bad_edge;
-		  (F, W) -> wings_face:delete_if_bad(F, W)
-	       end, We2, gb_sets:to_list(wings_we:new_items(face, We0, We2))),
-    dissolve_3(T, We);
-dissolve_3([], We) -> We.
-
-do_dissolve(Faces, Ess, Mat, WeOrig, We0) ->
-    We1 = do_dissolve_faces(Faces, We0),
-    Inner = wings_face:inner_edges(Faces, WeOrig),
-    We2 = delete_inner(Inner, We1),
-    #we{he=Htab0} = We = do_dissolve_1(Ess, Mat, WeOrig, We2),
-    Htab = gb_sets:difference(Htab0, gb_sets:from_list(Inner)),
-    We#we{he=Htab}.
-
-do_dissolve_1([EdgeList|Ess], Mat, WeOrig, #we{es=Etab0,fs=Ftab0}=We0) ->
-    {Face,We1} = wings_we:new_id(We0),
-    Ftab = gb_trees:insert(Face, hd(EdgeList), Ftab0),
-    Last = last(EdgeList),
-    Etab = update_outer([Last|EdgeList], EdgeList, Face, WeOrig,
-			Ftab, Etab0),
-    We2 = We1#we{es=Etab,fs=Ftab},
-    We = wings_material:assign(Mat, [Face], We2),
-    do_dissolve_1(Ess, Mat, WeOrig, We);
-do_dissolve_1([], _Mat, _WeOrig, We) -> We.
-
-do_dissolve_faces(Faces, #we{fs=Ftab0}=We) ->
-    Ftab = foldl(fun(Face, Ft) ->
-			 gb_trees:delete(Face, Ft)
-		 end, Ftab0, gb_sets:to_list(Faces)),
-    We#we{fs=Ftab}.
-
-delete_inner(Inner, #we{es=Etab0}=We) ->
-    Etab = foldl(fun(Edge, Et) ->
-			 gb_trees:delete(Edge, Et)
-		 end, Etab0, Inner),
-    We#we{es=Etab}.
-
-update_outer([Pred|[Edge|Succ]=T], More, Face, WeOrig, Ftab, Etab0) ->
-    #edge{rf=Rf} = R0 = gb_trees:get(Edge, Etab0),
-    Rec = case gb_trees:is_defined(Rf, Ftab) of
-	      true ->
-		  ?ASSERT(false == gb_trees:is_defined(R0#edge.lf, Ftab)),
-		  LS = succ(Succ, More),
-		  R0#edge{lf=Face,ltpr=Pred,ltsu=LS};
-	      false ->
-		  ?ASSERT(true == gb_trees:is_defined(R0#edge.lf, Ftab)),
-		  RS = succ(Succ, More),
-		  R0#edge{rf=Face,rtpr=Pred,rtsu=RS}
-	  end,
-    Etab = gb_trees:update(Edge, Rec, Etab0),
-    update_outer(T, More, Face, WeOrig, Ftab, Etab);
-update_outer([_], _More, _Face, _WeOrig, _Ftab, Etab) -> Etab.
-
-succ([Succ|_], _More) -> Succ;
-succ([], [Succ|_]) -> Succ.
+dissolve_sel(Faces, #we{id=Id}=We0, Acc) ->
+    We = wings_dissolve:faces(Faces, We0),
+    Sel = wings_we:new_items_as_gbset(face, We0, We),
+    {We,[{Id,Sel}|Acc]}.
 
 %%%
 %%% The Intrude command.
@@ -345,7 +256,7 @@ intrude(Faces0, #we{id=Id,es=Etab,fs=Ftab,next_id=Wid}=We0, SelAcc) ->
     {We1,RootSet} = wings_we:renumber(We0, Wid, RootSet0),
     We2 = wings_we:invert_normals(We1),
     We3 = wings_we:merge(We0, We2),
-    Sel0 = wings_we:new_items(face, We0, We3),
+    Sel0 = wings_we:new_items_as_gbset(face, We0, We3),
     BridgeFaces = [F || {face,F} <- RootSet0 ++ RootSet],
     Sel = gb_sets:difference(Sel0, gb_sets:from_list(BridgeFaces)),
     We4 = intrude_bridge(RootSet0, RootSet, We3),
@@ -425,8 +336,8 @@ mirror_move_vs({V,Pos0}, PlaneNormal, Center, A) ->
 mirror_weld(0, _IterA0, FaceA, _IterB0, FaceB, _WeOrig, #we{fs=Ftab0}=We0) ->
     Ftab1 = gb_trees:delete(FaceA, Ftab0),
     Ftab = gb_trees:delete(FaceB, Ftab1),
-    We = wings_material:delete_faces([FaceA,FaceB], We0#we{fs=Ftab}),
-    wings_we:vertex_gc(We);
+    We = wings_facemat:delete_faces([FaceA,FaceB], We0#we{fs=Ftab}),
+    wings_we:rebuild(We#we{vc=undefined});
 mirror_weld(N, IterA0, FaceA, IterB0, FaceB, WeOrig, We0) ->
     %% We will remove FaceA and FaceB, as well as all edges and vertices
     %% surrounding FaceB.
@@ -584,13 +495,13 @@ smooth(St0) ->
 
 smooth(Faces0, #we{id=Id}=We0, Acc) ->
     Rs = wings_sel:face_regions(Faces0, We0),
-    wings_pb:start(?STR(smooth,1,"smoothing")),
+    wings_pb:start(?__(1,"smoothing")),
     We1 = wings_pb:done(smooth_regions(Rs, 1, length(Rs), We0)),
-    NewFaces = wings_we:new_items(face, We0, We1),
-    NewVs = wings_we:new_items(vertex, We0, We1),
-    We2 = smooth_connect(NewVs, NewFaces, We1),
-    We = wings_util:mirror_flatten(We0, We2),
-    {We,[{Id,NewFaces}|Acc]}.
+    NewSelFaces = wings_we:new_items_as_ordset(face, We0, We1),
+    NewVs = wings_we:new_items_as_ordset(vertex, We0, We1),
+    We2 = smooth_connect(NewVs, NewSelFaces, We1),
+    We = wings_we:mirror_flatten(We0, We2),
+    {We,[{Id,gb_sets:from_ordset(NewSelFaces)}|Acc]}.
 
 smooth_regions([Faces0|Rs], I, N, #we{he=Htab}=We0) ->
     wings_pb:update(I/N, io_lib:format("~p/~p\n", [I,N])),
@@ -610,48 +521,60 @@ all_edges(Faces, We) ->
     {ordsets:from_list(Vs),ordsets:from_list(Es)}.
 
 smooth_connect(Vs, Faces0, #we{mirror=Mirror}=We0) ->
-    Faces1 = gb_sets:add(Mirror, Faces0),
-    Faces = sofs:from_external(gb_sets:to_list(Faces1), [face]),
+    Faces1 = ordsets:add_element(Mirror, Faces0),
+    Faces = sofs:from_external(Faces1, [face]),
     FaceVs0 = wings_vertex:per_face(Vs, We0),
     FaceVs1 = sofs:from_external(FaceVs0, [{face,[vertex]}]),
     FaceVs2 = sofs:drestriction(FaceVs1, Faces),
     FaceVs = sofs:to_external(FaceVs2),
-    smooth_connect_0(FaceVs, We0).
+    {We,Hide} = smooth_connect_0(FaceVs, [], We0),
+    wings_we:hide_faces(Hide, We).
 
-smooth_connect_0([{Face,Vs}|Fvs], We0) ->
-    case wings_material:get(Face, We0) of
+smooth_connect_0([{Face,Vs}|Fvs], Hide0, We0) ->
+    case wings_facemat:face(Face, We0) of
 	'_hole_' ->
-	    smooth_connect_0(Fvs, We0);
+	    smooth_connect_0(Fvs, Hide0, We0);
 	_ ->
-	    We = smooth_connect_1(Face, Vs, We0),
-	    smooth_connect_0(Fvs, We)
+	    {We,Hide} = smooth_connect_1(Face, Vs, Hide0, We0),
+	    smooth_connect_0(Fvs, Hide, We)
     end;
-smooth_connect_0([], We) -> We.
+smooth_connect_0([], Hide, We) -> {We,Hide}.
 
-smooth_connect_1(Face, [V], We) ->
+smooth_connect_1(Face, [V], Hide, We) ->
     Iter0 = wings_face:iterator(Face, We),
     IterCw = wings_face:skip_to_cw(V, Iter0),
     IterCcw = wings_face:skip_to_ccw(V, Iter0),
-    smooth_connect_2(IterCw, IterCcw, V, Face, We);
-smooth_connect_1(Face, Vs, We) ->
-    wings_vertex:connect(Face, Vs, We).
-
-smooth_connect_2(IterCw0, IterCcw0, V, Face, We0) ->
+    smooth_connect_2(IterCw, IterCcw, V, Face, Hide, We);
+smooth_connect_1(Face, Vs, Hide, We0) ->
+    We = wings_vertex:connect(Face, Vs, We0),
+    if
+	Face < 0 ->
+	    {We,wings_we:new_items_as_ordset(face, We0, We)++Hide};
+	true ->
+	    {We,Hide}
+    end.
+ 
+smooth_connect_2(IterCw0, IterCcw0, V, Face, Hide, We0) ->
     case {wings_face:next_cw(IterCw0),wings_face:next_ccw(IterCcw0)} of
 	{{_,Edge,_,_},{_,Edge,_,_}} ->
-	    {We1,NewV} = wings_edge:cut(Edge, 2, We0),
-	    {We,_} = wings_vertex:force_connect(V, NewV, Face, We1),
-	    We;
+	    {We,NewV} = wings_edge:cut(Edge, 2, We0),
+	    smooth_connect_3(V, NewV, Face, Hide, We);
 	{{Va,_,_,IterCw},{Vb,_,Rec,IterCcw}} ->
 	    case wings_vertex:other(Vb, Rec) of
 		Va when Va =/= V ->
-		    {We,_} = wings_vertex:force_connect(V, Va, Face, We0),
-		    We;
+		    smooth_connect_3(V, Va, Face, Hide, We0);
 		_Other ->
-		    smooth_connect_2(IterCw, IterCcw, V, Face, We0)
+		    smooth_connect_2(IterCw, IterCcw, V, Face, Hide, We0)
 	    end
     end.
 
+smooth_connect_3(Va, Vb, Face, Hide, We0) ->
+    {We,NewFace} = wings_vertex:force_connect(Va, Vb, Face, We0),
+    if
+	Face < 0 -> {We,[NewFace|Hide]};
+	true -> {We,Hide}
+    end.
+    
 %%%
 %%% The Bridge command.
 %%%
@@ -690,7 +613,8 @@ bridge(_St) ->
 
 unify_modes(#we{mode=Mode}, #we{mode=Mode}) -> Mode;
 unify_modes(_, _) ->
-    wings_util:error(?STR(unify_modes,1,"An object with vertex colors cannot be bridged with an object with materials.")).
+    wings_u:error(?__(1,
+			 "An object with vertex colors cannot be bridged with an object with materials.")).
 
 bridge_null_uvs(Mode, #we{mode=Mode}=We) -> We;
 bridge_null_uvs(uv, #we{es=Etab0}=We) ->
@@ -707,17 +631,17 @@ bridge(FaceA, FaceB, #we{vp=Vtab}=We) ->
     VsB = wings_face:vertices_ccw(FaceB, We),
     if
 	length(VsA) =/= length(VsB) ->
-	    bridge_error(?STR(bridge,1,"Faces must have the same number of vertices."));
+	    bridge_error(?__(1,"Faces must have the same number of vertices."));
 	true ->
 	    An = wings_face:face_normal_cw(VsA, Vtab),
 	    Bn = wings_face:face_normal_cw(VsB, Vtab),
 	    case e3d_vec:dot(An, Bn) of
 		Dot when Dot > 0.99 ->
-		    bridge_error(?STR(bridge,2,"Faces must not point in the same direction."));
+		    bridge_error(?__(2,"Faces must not point in the same direction."));
 		_Dot ->
 		    case wings_face:are_neighbors(FaceA, FaceB, We) of
 			true ->
-			    bridge_error(?STR(bridge,3,"Faces must not be neighbors."));
+			    bridge_error(?__(3,"Faces must not be neighbors."));
 			false ->
 			    bridge(FaceA, VsA, FaceB, VsB, We)
 		    end
@@ -768,7 +692,7 @@ force_bridge(FaceA, Va, FaceB, Vb, We0) ->
 do_bridge(0, _Va, FaceA, _IterA, _Vb, FaceB, _IterB, _, #we{fs=Ftab0}=We) ->
     Ftab1 = gb_trees:delete(FaceA, Ftab0),
     Ftab = gb_trees:delete(FaceB, Ftab1),
-    wings_material:delete_faces([FaceA,FaceB], We#we{fs=Ftab});
+    wings_facemat:delete_faces([FaceA,FaceB], We#we{fs=Ftab});
 do_bridge(N, Va0, FaceA, IterA0, Vb0, FaceB, IterB0, Ids0, We0) ->
     #we{es=Etab0,fs=Ftab0} = We0,
     NewEdge = wings_we:id(2, Ids0),
@@ -810,8 +734,8 @@ do_bridge(N, Va0, FaceA, IterA0, Vb0, FaceB, IterB0, Ids0, We0) ->
 			  rf=RightFace,rtpr=EdgeB,rtsu=EdgeA},
     Etab = gb_trees:enter(NewEdge, NewRec, Etab3),
 
-    Mat = wings_material:get(FaceA, We0),
-    We1 = wings_material:assign(Mat, [RightFace], We0),
+    Mat = wings_facemat:face(FaceA, We0),
+    We1 = wings_facemat:assign(Mat, [RightFace], We0),
     Ftab = gb_trees:insert(RightFace, NewEdge, Ftab0),
     
     We = We1#we{es=Etab,fs=Ftab},
@@ -827,10 +751,10 @@ get_edge(Edge, Etab) ->
     end.
 
 bridge_error() ->
-    bridge_error(?STR(bridge_error,1,"Exactly two faces must be selected.")).
+    bridge_error(?__(1,"Exactly two faces must be selected.")).
 
 bridge_error(Error) ->
-    wings_util:error(Error).
+    wings_u:error(Error).
 
 bridge_color(Edge, Face, Iter) ->
     Etab = wings_face:iter2etab(Iter),
@@ -844,7 +768,7 @@ bridge_color(Edge, Face, Iter) ->
 %%%
 
 lift_selection(Dir, OrigSt) ->
-    Desc = ?STR(lift_selection,1,"Select edge or vertex to act as hinge"),
+    Desc = ?__(1,"Select edge or vertex to act as hinge"),
     Fun = fun(check, St) ->
 		  lift_check_selection(St, OrigSt);
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}=St}) ->
@@ -867,7 +791,7 @@ lift_check_selection(#st{selmode=edge,sel=EdgeSel}, OrigSt) ->
 	    end, EdgeSel, OrigSt),
     case Res of
 	[] -> {none,""};
-	_ -> {none,?STR(lift_check_selection,1,"Face and edge selections don't match.")}
+	_ -> {none,?__(1,"Face and edge selections don't match.")}
     end;
 lift_check_selection(#st{selmode=vertex,sel=VsSel}, OrigSt) ->
     Res = wings_sel:fold(
@@ -881,7 +805,7 @@ lift_check_selection(#st{selmode=vertex,sel=VsSel}, OrigSt) ->
 	    end, VsSel, OrigSt),
     case Res of
 	[] -> {none,""};
-	_ -> {none,?STR(lift_check_selection,2,"Face and vertex selections don't match.")}
+	_ -> {none,?__(2,"Face and vertex selections don't match.")}
     end.
 
 lift({'ASK',Ask}, St) ->
@@ -893,6 +817,13 @@ lift({Dir,vertex,VertexSel}, St) ->
 lift(Dir, St) ->
     wings:ask(lift_selection(Dir, St), St, fun lift/2).
 
+lift_setup_drag(Tvs, rotate, St) ->
+    wings_drag:setup(Tvs, [angle], St);
+lift_setup_drag(Tvs, free, St) ->
+    wings_drag:setup(Tvs, [dx,dy,dz], [screen_relative], St);
+lift_setup_drag(Tvs, _, St) ->
+    wings_drag:setup(Tvs, [distance], St).
+
 %%%
 %%% Lift from edge.
 %%%
@@ -905,17 +836,12 @@ lift_from_edge(Dir, EdgeSel, St0) ->
 	       (_, _, _) -> lift_sel_mismatch()
 	    end, {EdgeSel,[]}, St0),
     case Res of
-	{St,{[],Tvs}} when Dir == rotate ->
-	    wings_drag:setup(Tvs, [angle], St);
-	{St,{[],Tvs}} when Dir == free ->
-	    wings_drag:setup(Tvs, [dx,dy,falloff], [screen_relative], St);
-	{St,{[],Tvs}} ->
-	    wings_drag:setup(Tvs, [distance], St);
+	{St,{[],Tvs}} -> lift_setup_drag(Tvs, Dir, St);
 	{_,_} -> lift_sel_mismatch()
     end.
 
 lift_sel_mismatch() ->
-    wings_util:error(?STR(lift_sel_mismatch,1,"Face and edge selections don't match.")).
+    wings_u:error(?__(1,"Face and edge selections don't match.")).
 	
 lift_from_edge(Dir, Faces, Edges, We0, Tv) ->
     case lift_face_edge_pairs(Faces, Edges, We0) of
@@ -1001,17 +927,12 @@ lift_from_vertex(Dir, VsSel, St0) ->
 	       (_, _, _) -> lift_vtx_sel_mismatch()
 	    end, {VsSel,[]}, St0),
     case Res of
-	{St,{[],Tvs}} when Dir == rotate ->
-	    wings_drag:setup(Tvs, [angle], St);
-	{St,{[],Tvs}} when Dir == free ->
-	    wings_drag:setup(Tvs, [dx,dy,falloff], [screen_relative], St);
-	{St,{[],Tvs}} ->
-	    wings_drag:setup(Tvs, [distance], St);
+	{St,{[],Tvs}} -> lift_setup_drag(Tvs, Dir, St);
 	{_,_} -> lift_vtx_sel_mismatch()
     end.
 
 lift_vtx_sel_mismatch() ->
-    wings_util:error(?STR(lift_vtx_sel_mismatch,1,"Face and vertex selections don't match.")).
+    wings_u:error(?__(1,"Face and vertex selections don't match.")).
 
 lift_from_vertex(Dir, Faces, Vs, We, Tv) ->
     case lift_face_vertex_pairs(Faces, Vs, We) of
@@ -1032,7 +953,7 @@ lift_from_vertex_2(Dir, Face, V, #we{id=Id,next_id=Next}=We0, Tv) ->
 		   wings_collapse:collapse_edge(Edge, V, W);
 	      (_, _, _, W) -> W
 	   end, We1, V, We1),
-    FaceVs = wings_we:new_items(vertex, We0, We),
+    FaceVs = wings_we:new_items_as_ordset(vertex, We0, We),
     case Dir of
 	rotate ->
 	    Vpos = wings_vertex:pos(V, We),
@@ -1050,8 +971,7 @@ lift_from_vertex_2(Dir, Face, V, #we{id=Id,next_id=Next}=We0, Tv) ->
 	    M = e3d_vec:norm(e3d_vec:add(Vecs)),
 	    N = wings_face:normal(Face, We),
 	    Axis = e3d_vec:cross(M, N),
- 	    Rot = wings_rotate:rotate(Axis, Vpos, gb_sets:to_list(FaceVs),
-				      We, Tv),
+ 	    Rot = wings_rotate:rotate(Axis, Vpos, FaceVs, We, Tv),
 	    {We,Rot};
 	_Other ->
 	    Vec = wings_util:make_vector(Dir),
@@ -1088,13 +1008,13 @@ put_on(#st{sel=[{_,Faces}]}=St) ->
 	1 ->
 	    wings:ask(put_on_selection(St), St, fun put_on/2);
 	_ ->
-	    wings_util:error(?STR(put_on,1,"There must only be one face selected."))
+	    wings_u:error(?__(1,"There must only be one face selected."))
     end;
 put_on(_) ->
-    wings_util:error(?STR(put_on,1,"There must only be one face selected.")).
+    wings_u:error(?__(1,"There must only be one face selected.")).
 
 put_on_selection(OrigSt) ->
-    Desc = ?STR(put_on_selection,1,"Select target element on which to put source object"),
+    Desc = ?__(1,"Select target element on which to put source object"),
     Fun = fun(check, St) -> put_on_check_selection(St, OrigSt);
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}=St}) ->
 		  case put_on_check_selection(St, OrigSt) of
@@ -1105,14 +1025,14 @@ put_on_selection(OrigSt) ->
     {[{Fun,Desc}],[],[],[face,edge,vertex]}.
 
 put_on_check_selection(#st{sel=[{Id,_}]}, #st{sel=[{Id,_}]}) ->
-    {none,?STR(put_on_check_selection,1,"Selection must not be in the same object.")};
+    {none,?__(1,"Selection must not be in the same object.")};
 put_on_check_selection(#st{sel=[{_,Elems}]}, _) ->
     case gb_trees:size(Elems) of
 	1 -> {none,""};
-	_ -> {none,?STR(put_on_check_selection,2,"Select only one element.")}
+	_ -> {none,?__(2,"Select only one element.")}
     end;
 put_on_check_selection(_, _) ->
-    {none,?STR(put_on_check_selection,2,"Select only one element.")}.
+    {none,?__(2,"Select only one element.")}.
 
 put_on({Mode,[{Id,Els}]}, #st{shapes=Shs}=St0) ->
     We0 = gb_trees:get(Id, Shs),
@@ -1143,13 +1063,13 @@ clone_on(#st{sel=[{_,Faces}]}=St) ->
 	1 ->
 	    wings:ask(clone_on_selection(), St, fun clone_on/2);
 	_ ->
-	    wings_util:error(?STR(clone_on,1,"There must only be one face selected."))
+	    wings_u:error(?__(1,"There must only be one face selected."))
     end;
 clone_on(_) ->
-    wings_util:error(?STR(clone_on,1,"There must only be one face selected.")).
+    wings_u:error(?__(1,"There must only be one face selected.")).
 
 clone_on_selection() ->
-    Desc = ?STR(clone_on_selection,1,"Select target elements on which to put clones"),
+    Desc = ?__(1,"Select target elements on which to put clones"),
     Fun = fun(check, _) ->
 		  {none,""};
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}}) ->
@@ -1188,6 +1108,17 @@ clone_3(El, We, Tr, N, Clone, #st{selmode=Mode}=St) ->
     NewWe = wings_we:transform_vs(M, Clone),
     wings_shape:insert(NewWe, clone, St).
 
+hide_faces(St0) ->
+    St = wings_sel:map(fun hide_faces_fun/2, St0),
+    wings_sel:clear(St).
+
+hide_faces_fun(Fs, We0) ->
+    We = wings_we:hide_faces(Fs, We0),
+    case wings_we:all_hidden(We) of
+	true -> We0#we{perm=[]};		%Hide entire object.
+	false -> We
+    end.
+
 %%
 %% Common help function.
 %%
@@ -1229,81 +1160,3 @@ set_color_1([F|Fs], Color, #we{es=Etab0}=We) ->
 	     end, Etab0, F, We),
     set_color_1(Fs, Color, We#we{es=Etab});
 set_color_1([], _, We) -> We.
-
-    
-%% outer_edge_partition(FaceSet, WingedEdge) -> [[Edge]].
-%%  Partition all outer edges. Outer edges are all edges
-%%  between one face in the set and one outside.
-
-outer_edge_partition(Faces, We) when is_list(Faces) ->
-    collect_outer_edges(Faces, gb_sets:from_list(Faces), We);
-outer_edge_partition(Faces, We) ->
-    collect_outer_edges(gb_sets:to_list(Faces), Faces, We).
-
-collect_outer_edges(Fs, Faces, We) ->
-    F0 = wings_face:fold_faces(
-	   fun(Face, _, Edge, #edge{vs=V,ve=OtherV,lf=Face,rf=Other}, Acc) ->
-		   case gb_sets:is_member(Other, Faces) of
-		       false -> [{V,{Edge,V,OtherV}}|Acc];
-		       true -> Acc
-		   end;
-	      (Face, _, Edge, #edge{vs=OtherV,ve=V,rf=Face,lf=Other}, Acc) ->
-		   case gb_sets:is_member(Other, Faces) of
-		       false -> [{V,{Edge,V,OtherV}}|Acc];
-		       true -> Acc
-		   end
-	   end, [], Fs, We),
-    F = gb_trees:from_orddict(wings_util:rel2fam(F0)),
-    partition_edges(F, Faces, We, []).
-
-partition_edges(Es0, Faces, We, Acc) ->
-    case gb_trees:is_empty(Es0) of
-	true -> Acc;
-	false ->
-	    {Key,Val,Es1} = gb_trees:take_smallest(Es0),
-	    {Part,Es} = partition_edges(Key, Val, Faces, Es1, We, []),
-	    partition_edges(Es, Faces, We, [Part|Acc])
-    end.
-
-partition_edges(Va, [{Edge,Va,Vb}], Faces, Es0, We, Acc0) ->
-    Acc = [Edge|Acc0],
-    case gb_trees:lookup(Vb, Es0) of
-	none ->
-	    {Acc,Es0};
-	{value,Val} ->
-	    Es = gb_trees:delete(Vb, Es0),
-	    partition_edges(Vb, Val, Faces, Es, We, Acc)
-    end;
-partition_edges(Va, [Val|More], Faces, Es0, We, []) ->
-    Es = gb_trees:insert(Va, More, Es0),
-    partition_edges(Va, [Val], Faces, Es, We, []);
-partition_edges(Va, Edges, Faces, Es, We, Acc) ->
-    part_try_all_edges(Va, Edges, Faces, Es, We, Acc, [], none).
-    
-%% Here we have multiple choices of edges. Use the shortest path
-%% that don't return us to the Va vertex.
-%% (We want edge loops without repeated vertices.)
-part_try_all_edges(Va, [Val|More], Faces, Es0, We, Acc, Done0, Path0) ->
-    Es1 = gb_trees:insert(Va, [{repeated,Va,fake}], Es0),
-    Done = [Val|Done0],
-    case partition_edges(Va, [Val], Faces, Es1, We, Acc) of
-	none ->
-	    part_try_all_edges(Va, More, Faces, Es0, We, Acc, Done, Path0);
-	{[repeated|_],_} ->
-	    part_try_all_edges(Va, More, Faces, Es0, We, Acc, Done, Path0);
-	{_,_}=Found ->
-	    Path1 = part_shortest_path(Path0, Found),
-	    Path = {Path1,Done0,More},
-	    part_try_all_edges(Va, More, Faces, Es0, We, Acc, Done, Path)
-    end;
-part_try_all_edges(Va, [], _, _, _, _, _, {{Path,Es},Alt0,Alt1}) ->
-    case Alt0++Alt1 of
-	[] -> {Path,gb_trees:delete(Va, Es)};
-	Alt -> {Path,gb_trees:enter(Va, Alt, Es)}
-    end;
-part_try_all_edges(_Va, [], _, _, _, _, _Alt, none) -> none.
-
-part_shortest_path(none, Path) -> Path;
-part_shortest_path({{AccA,_},_,_}=A, {{AccB,_},_,_})
-  when length(AccA) < length(AccB) -> A;
-part_shortest_path(_, B) -> B.
