@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_ff_wings.erl,v 1.57 2004/10/08 06:02:29 dgud Exp $
+%%     $Id: wings_ff_wings.erl,v 1.62 2004/12/30 05:27:12 bjorng Exp $
 %%
 
 -module(wings_ff_wings).
@@ -23,36 +23,36 @@
 %% Load a wings file.
 
 import(Name, St) ->
-    wings_pb:start(?STR(import,1,"opening wings file")),
-    wings_pb:update(0.07, ?STR(import,2,"reading file")),
+    wings_pb:start(?__(1,"opening wings file")),
+    wings_pb:update(0.07, ?__(2,"reading file")),
     wings_pb:done(import_1(Name, St)).
 
 import_1(Name, St0) ->
     case file:read_file(Name) of
 	{ok,<<?WINGS_HEADER,Sz:32,Data/binary>>} when size(Data) =:= Sz ->
-	   wings_pb:update(0.08, ?STR(import_1,1,"converting binary")),
+	   wings_pb:update(0.08, ?__(1,"converting binary")),
 	    case catch binary_to_term(Data) of
 		{wings,0,_Shapes} ->
-                    {error, ?STR(import_1,2,"Pre-0.80 Wings format no longer supported.")};
+                    {error, ?__(2,"Pre-0.80 Wings format no longer supported.")};
 		{wings,1,_,_,_} ->
                      %% Pre-0.92. No longer supported.
-                    {error,?STR(import_1,3,"Pre-0.92 Wings format no longer supported.")};
+                    {error,?__(3,"Pre-0.92 Wings format no longer supported.")};
 		{wings,2,{Shapes,Materials,Props}} ->
                     import_vsn2(Shapes, Materials, Props, St0);
 		{wings,_,_} ->
-		    {error,?STR(import_1,4,"unknown wings format")};
+		    {error,?__(4,"unknown wings format")};
 		Other ->
 		    io:format("~P\n", [Other,20]),
-                    {error,?STR(import_1,5,"corrupt Wings file")}
+                    {error,?__(5,"corrupt Wings file")}
 	    end;
 	{ok,_Bin} ->
-	    {error,?STR(import_1,6,"not a Wings file (or old Wings format)")};
+	    {error,?__(6,"not a Wings file (or old Wings format)")};
 	{error,Reason} ->
 	    {error,file:format_error(Reason)}
     end.
 
 import_vsn2(Shapes, Materials0, Props, St0) ->
-    wings_pb:update(0.10, ?STR(import_vsn2,1,"images and materials")),
+    wings_pb:update(0.10, ?__(1,"images and materials")),
     Images = import_images(Props),
     Materials1 = translate_materials(Materials0),
     Materials = translate_map_images(Materials1, Images),
@@ -60,7 +60,7 @@ import_vsn2(Shapes, Materials0, Props, St0) ->
     NameMap1 = gb_trees:from_orddict(sort(NameMap0)),
     NameMap = optimize_name_map(Materials, NameMap1, []),
     St = import_props(Props, St1),
-    wings_pb:update(1.0,?STR(import_vsn2,2,"objects")),
+    wings_pb:update(1.0,?__(2,"objects")),
     import_objects(Shapes, NameMap, St).
 
 optimize_name_map([{Name,_}|Ms], NameMap, Acc) ->
@@ -88,7 +88,8 @@ import_objects([Sh0|Shs], Mode, NameMap, Oid, ShAcc) ->
     Mirror = proplists:get_value(mirror_face, Props, none),
     We = #we{es=Etab,vp=Vtab,he=Htab,perm=Perm,
 	     id=Oid,name=Name,mode=ObjMode,mirror=Mirror,mat=FaceMat},
-    import_objects(Shs, Mode, NameMap, Oid+1, [We|ShAcc]);
+    HiddenFaces = proplists:get_value(num_hidden_faces, Props, 0),
+    import_objects(Shs, Mode, NameMap, Oid+1, [{HiddenFaces,We}|ShAcc]);
 import_objects([], _Mode, _NameMap, Oid, Objs0) ->
     %%io:format("flat_size: ~p\n", [erts_debug:flat_size(Objs0)]),
     Objs = share_list(Objs0),
@@ -126,7 +127,7 @@ import_edge([], Rec) -> Rec.
 
 import_face_mat([F|Fs], NameMap, Face, Acc) ->
     Mat = import_face_mat_1(F, NameMap, default),
-    import_face_mat(Fs, NameMap, Face+1, [{Mat,Face}|Acc]);
+    import_face_mat(Fs, NameMap, Face+1, [{Face,Mat}|Acc]);
 import_face_mat([], _, _, Acc) -> reverse(Acc).
 
 import_face_mat_1([{material,Name}|T], NameMap, _) ->
@@ -164,7 +165,7 @@ import_perm(Props) ->
 import_object_mode(Ps) ->
     case proplists:get_value(mode, Ps, material) of
 	undefined ->
-	    io:format(?STR(import_object_mode,1,"Changed undefined mode to material\n")),
+	    io:format(?__(1,"Changed undefined mode to material\n")),
 	    material;
 	uv -> material;
 	Other -> Other
@@ -175,7 +176,7 @@ import_props([{selection,{Mode,Sel0}}|Ps], St) ->
     import_props(Ps, St#st{selmode=Mode,sel=Sel});
 import_props([{saved_selection,{Mode,Sel0}}|Ps], St0) ->
     Sel = import_sel(Sel0, St0),
-    St = new_sel_group(?STR(import_props,1,"<Stored Selection>"), Mode, Sel, St0),
+    St = new_sel_group(?__(1,"<Stored Selection>"), Mode, Sel, St0),
     import_props(Ps, St);
 import_props([{{selection_group,Name},{Mode,Sel0}}|Ps], St0) ->
     Sel = import_sel(Sel0, St0),
@@ -227,7 +228,7 @@ import_images_1([{Id0,Im}|T], Map) ->
 import_images_1([], Map) -> Map.
 
 import_image(Im) ->
-    Name = proplists:get_value(name, Im, ?STR(import_image,1,"unnamed image")),
+    Name = proplists:get_value(name, Im, ?__(1,"unnamed image")),
     case proplists:get_value(filename, Im) of
 	undefined ->
 	    W = proplists:get_value(width, Im, 0),
@@ -236,7 +237,7 @@ import_image(Im) ->
 	    Pixels = proplists:get_value(pixels, Im),
 	    if
 		W*H*PP =:= size(Pixels) -> ok;
-		true -> wings_util:error( ?STR(import_image,2,"Bad image: ~p\n"), [Name])
+		true -> wings_u:error(?__(2,"Bad image: ~p\n"), [Name])
 	    end,
 	    MaskSize = proplists:get_value(mask_size, Im),
 	    Type = case PP of
@@ -250,7 +251,7 @@ import_image(Im) ->
 		       alignment=1,bytes_pp=PP,image=Pixels};
 	Filename ->
 	    Ps = [{filename,Filename}],
-	    case wpa:image_read(Ps) of
+	    case wings_image:image_read(Ps) of
 		#e3d_image{}=E3D ->
 		    E3D#e3d_image{name=Name,filename=Filename};
 		{error,_} ->
@@ -273,8 +274,8 @@ translate_map_images_2([{Type,Im0}|T], Mat, ImMap) when is_integer(Im0) ->
     case gb_trees:lookup(Im0, ImMap) of
 	none ->
 	    %% Something wrong here.
-	    io:format( ?STR(translate_map_images_2,1,"Material ~p, ~p texture: reference to non-existing image ~p\n"),
-		      [Mat,Type,Im0]),
+	    io:format( ?__(1,"Material ~p, ~p texture: reference to non-existing image ~p\n"),
+		       [Mat,Type,Im0]),
 	    translate_map_images_2(T, Mat, ImMap);
 	{value,Im} ->
 	    [{Type,Im}|translate_map_images_2(T, Mat, ImMap)]
@@ -288,7 +289,7 @@ translate_map_images_2([], _, _) -> [].
 %%%
 
 share_list(Wes) ->
-    Tabs0 = [{Vtab,Etab} || #we{vp=Vtab,es=Etab} <- Wes],
+    Tabs0 = [{Vtab,Etab} || {_,#we{vp=Vtab,es=Etab}} <- Wes],
     Floats = share_floats(Tabs0, tuple_to_list(wings_color:white())),
     Tabs = share_list_1(Tabs0, Floats, gb_trees:empty(), []),
     share_list_2(Tabs, Wes, []).
@@ -299,11 +300,18 @@ share_list_1([{Vtab0,Etab0}|Ts], Floats, Tuples0, Acc) ->
     share_list_1(Ts, Floats, Tuples, [{Vtab,Etab}|Acc]);
 share_list_1([], _, _, Ts) -> reverse(Ts).
 
-share_list_2([{Vtab0,Etab0}|Ts], [#we{id=Id,mat=FaceMat}=We0|Wes], Acc) ->
+share_list_2([{Vtab0,Etab0}|Ts],
+	     [{NumHidden,#we{id=Id,mat=FaceMat}=We0}|Wes], Acc) ->
     Vtab = gb_trees:from_orddict(Vtab0),
     Etab = gb_trees:from_orddict(Etab0),
     We1 = wings_we:rebuild(We0#we{vp=Vtab,es=Etab,mat=default}),
-    We = wings_material:assign_materials(FaceMat, We1),
+    We2 = wings_facemat:assign(FaceMat, We1),
+    We = if
+	     NumHidden =:= 0 -> We2;
+	     true ->
+		 Hidden = lists:seq(0, NumHidden-1),
+		 wings_we:hide_faces(Hidden, We2)
+	 end,
     share_list_2(Ts, Wes, [{Id,We}|Acc]);
 share_list_2([], [], Wes) -> sort(Wes).
 
@@ -413,16 +421,16 @@ trans({Key,{R,G,B}}, Opac) -> {Key,{R,G,B,Opac}}.
 %%%
 
 export(Name, St0) ->
-    wings_pb:start( ?STR(export,1,"saving")),
-    wings_pb:update(0.01, ?STR(export,2,"lights")),
+    wings_pb:start( ?__(1,"saving")),
+    wings_pb:update(0.01, ?__(2,"lights")),
     Lights = wings_light:export(St0),
     #st{shapes=Shs0,views={CurrentView,_}} = St = 
 	remove_lights(St0),
     Sel0 = collect_sel(St),
-    wings_pb:update(0.65, ?STR(export,3,"renumbering")),
+    wings_pb:update(0.65, ?__(3,"renumbering")),
     {Shs1,Sel} = renumber(gb_trees:to_list(Shs0), Sel0, 0, [], []),
     Shs = foldl(fun shape/2, [], Shs1),
-    wings_pb:update(0.98, ?STR(export,4,"objects")),
+    wings_pb:update(0.98, ?__(4,"objects")),
     Materials = wings_material:used_materials(St),
     Props0 = export_props(Sel),
     Props1 = case Lights of
@@ -443,9 +451,9 @@ export(Name, St0) ->
 	     end,
     Props  = [{scene_prefs,wings_pref:get_scene_value()}|Props4],
     Wings = {wings,2,{Shs,Materials,Props}},
-    wings_pb:update(0.99, ?STR(export,5,"compressing")),
+    wings_pb:update(0.99, ?__(5,"compressing")),
     Bin = term_to_binary(Wings, [compressed]),
-    wings_pb:update(1.0, ?STR(export,6,"writing file")),
+    wings_pb:update(1.0, ?__(6,"writing file")),
     wings_pb:done(write_file(Name, Bin)).
 
 remove_lights(#st{sel=Sel0,shapes=Shs0}=St) ->
@@ -471,11 +479,14 @@ collect_sel_groups([{{Mode,Name},Sel}|Gs], St, Acc0) ->
 collect_sel_groups([], _, Acc) -> Acc.
 
 renumber([{Id,We0}|Shs], [{Id,Root0}|Sel], NewId, WeAcc, RootAcc) ->
+    Hidden = wings_we:num_hidden(We0),
     {We,Root} = wings_we:renumber(We0, 0, Root0),
-    renumber(Shs, Sel, NewId+1, [We|WeAcc], [{NewId,Root}|RootAcc]);
+    renumber(Shs, Sel, NewId+1, [{Hidden,We}|WeAcc],
+	     [{NewId,Root}|RootAcc]);
 renumber([{_,We0}|Shs], Sel, NewId, WeAcc, RootAcc) ->
+    Hidden = wings_we:num_hidden(We0),
     We = wings_we:renumber(We0, 0),
-    renumber(Shs, Sel, NewId+1, [We|WeAcc], RootAcc);
+    renumber(Shs, Sel, NewId+1, [{Hidden,We}|WeAcc], RootAcc);
 renumber([], [], _NewId, WeAcc, RootAcc) ->
     {WeAcc,RootAcc}.
 
@@ -500,7 +511,7 @@ write_file(Name, Bin) ->
 	{error,Reason} -> {error,file:format_error(Reason)}
     end.
 
-shape(#we{mode=ObjMode,name=Name,vp=Vs0,es=Es0,he=Htab}=We, Acc) ->
+shape({Hidden,#we{mode=ObjMode,name=Name,vp=Vs0,es=Es0,he=Htab}=We}, Acc) ->
     Vs1 = foldl(fun export_vertex/2, [], gb_trees:values(Vs0)),
     Vs = reverse(Vs1),
     UvFaces = gb_sets:from_ordset(wings_we:uv_mapped_faces(We)),
@@ -508,16 +519,20 @@ shape(#we{mode=ObjMode,name=Name,vp=Vs0,es=Es0,he=Htab}=We, Acc) ->
 			export_edge(E, UvFaces, A)
 		end, [], gb_trees:values(Es0)),
     Es = reverse(Es1),
-    Fs1 = foldl(fun export_face/2, [], wings_material:get_all(We)),
+    Fs1 = foldl(fun export_face/2, [], wings_facemat:all(We)),
     Fs = reverse(Fs1),
     He = gb_sets:to_list(Htab),
     Props0 = [{mode,ObjMode}|export_perm(We)],
-    Props = mirror(We, Props0),
+    Props1 = hidden_faces(Hidden, Props0),
+    Props = mirror(We, Props1),
     [{object,Name,{winged,Es,Fs,Vs,He},Props}|Acc].
 
 mirror(#we{mirror=none}, Props) -> Props;
 mirror(#we{mirror=Face}, Props) -> [{mirror_face,Face}|Props].
-    
+
+hidden_faces(0, Props) -> Props;
+hidden_faces(N, Props) -> [{num_hidden_faces,N}|Props].
+
 export_perm(#we{perm=[]}) ->
     [{state,hidden_locked}];	     %Only for backward compatibility.
 export_perm(#we{perm=0}) -> [];
