@@ -8,11 +8,12 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_file.erl,v 1.151 2004/10/17 06:37:43 bjorng Exp $
+%%     $Id: wings_file.erl,v 1.156 2004/12/18 19:36:20 bjorng Exp $
 %%
 
 -module(wings_file).
 -export([init/0,init_autosave/0,menu/1,command/2]).
+-export([import_filename/2,export_filename/2,export_filename/3]).
 
 -include("wings.hrl").
 -include("e3d.hrl").
@@ -24,6 +25,61 @@
 
 -define(WINGS, ".wings").
 
+%% export_filename([Prop], St, Continuation).
+%%   The St will only be used to setup the default filename.
+%%   The Continuation fun will be called like this: Continuation(Filename).
+export_filename(Prop, #st{file=undefined}, Cont) ->
+    export_filename(Prop, Cont);
+export_filename(Prop0, #st{file=File}, Cont) ->
+    Prop = case proplists:get_value(ext, Prop0) of
+	       undefined -> Prop0;
+	       Ext ->
+		   Def = filename:rootname(filename:basename(File), ".wings") ++ Ext,
+		   [{default_filename,Def}|Prop0]
+	   end,
+    export_filename(Prop, Cont).
+
+%% import_filename([Prop], Continuation).
+%%   The Continuation fun will be called like this: Continuation(Filename).
+import_filename(Ps0, Cont) ->
+    This = wings_wm:this(),
+    Dir = wings_pref:get_value(current_directory),
+    Ps = Ps0 ++ [{title,?__(1,"Import")},{directory,Dir}],
+    Fun = fun(Name) ->
+		  case catch Cont(Name) of
+		      {command_error,Error} ->
+			  wings_u:message(Error);
+		      #st{}=St ->
+			  wings_wm:send(This, {new_state,St});
+		      Tuple when is_tuple(Tuple) ->
+			  wings_wm:send(This, {action,Tuple});
+		      ignore -> keep;
+		      keep -> keep
+		  end
+	  end,
+    wings_plugin:call_ui({file,open_dialog,Ps,Fun}).
+
+%% export_filename([Prop], Continuation).
+%%   The Continuation fun will be called like this: Continuation(Filename).
+export_filename(Prop0, Cont) ->
+    This = wings_wm:this(),
+    Dir = wings_pref:get_value(current_directory),
+    Prop = Prop0 ++ [{directory,Dir}],
+    Fun = fun(Name) ->
+		  case catch Cont(Name) of
+		      {command_error,Error} ->
+			  wings_u:message(Error);
+		      #st{}=St ->
+			  wings_wm:send(This, {new_state,St});
+		      Tuple when is_tuple(Tuple) ->
+			  wings_wm:send(This, {action,Tuple});
+		      ignore -> keep;
+		      keep -> keep;
+		      ok -> keep
+		  end
+	  end,
+	wings_plugin:call_ui({file,save_dialog,Prop++[{title,?__(1,"Export")}],Fun}).
+
 init() ->
     case wings_pref:get_value(current_directory) of
 	undefined ->
@@ -43,28 +99,37 @@ init() ->
 menu(_) ->
     ImpFormats = [{"Nendo (.ndo)...",ndo}],
     ExpFormats = [{"Nendo (.ndo)...",ndo}],
-    [{?STR(menu,3,"New"),new,?STR(menu,4,"Create a new, empty scene")},
-     {?STR(menu,5,"Open..."),open,?STR(menu,6,"Open a previously saved scene")},
-     {?STR(menu,7,"Merge..."),merge,?STR(menu,8,"Merge a previously saved scene into the current scene")},
+    Tail = [{?__(25,"Exit"),quit,?__(28,"Exit Wings 3D")}],
+    [{?__(3,"New"),new,
+      ?__(4,"Create a new, empty scene")},
+     {?__(5,"Open..."),open,
+      ?__(6,"Open a previously saved scene")},
+     {?__(7,"Merge..."),merge,
+      ?__(8,"Merge a previously saved scene into the current scene")},
       separator,
-     {?STR(menu,9,"Save"),save,?STR(menu,10,"Save the current scene")},
-     {?STR(menu,11,"Save As..."),save_as,?STR(menu,12,"Save the current scene under a new name")},
-     {?STR(menu,13,"Save Selected..."),save_selected,?STR(menu,14,"Save only the selected objects or faces")},
-     {?STR(menu,15,"Save Incrementally"),save_incr},
+     {?__(9,"Save"),save,
+      ?__(10,"Save the current scene")},
+     {?__(11,"Save As..."),save_as,
+      ?__(12,"Save the current scene under a new name")},
+     {?__(13,"Save Selected..."),save_selected,
+      ?__(14,"Save only the selected objects or faces")},
+     {?__(15,"Save Incrementally"),save_incr,
+      ?__(26,"Generate new filename and save")},
+     separator,
+     {?__(16,"Revert"),revert,
+      ?__(17,"Revert current scene to the saved contents")},
       separator,
-     {?STR(menu,16,"Revert"),revert,?STR(menu,17,"Revert current scene to the saved contents")},
-      separator,
-     {?STR(menu,18,"Import"),{import,ImpFormats}},
-     {?STR(menu,19,"Export"),{export,ExpFormats}},
-     {?STR(menu,20,"Export Selected"),{export_selected,ExpFormats}},
-      separator,
-     {?STR(menu,21,"Import Image..."),import_image,?STR(menu,22,"Import an image file")},
-      separator,
-     {?STR(menu,23,"Render"),{render,[]}},
-      separator,
-     {?STR(menu,24,"Install Plug-In"),install_plugin},
-      separator|recent_files([{?STR(menu,25,"Exit"),quit}])].
-												
+     {?__(18,"Import"),{import,ImpFormats}},
+     {?__(19,"Export"),{export,ExpFormats}},
+     {?__(20,"Export Selected"),{export_selected,ExpFormats}},
+     separator,
+     {?__(21,"Import Image..."),import_image,?__(22,"Import an image file")},
+     separator,
+     {?__(23,"Render"),{render,[]}},
+     separator,
+     {?__(24,"Install Plug-In"),install_plugin,
+      ?__(27,"Install a plug-in")},
+     separator|recent_files(Tail)].
     
 command(new, St) ->
     new(St);
@@ -99,7 +164,7 @@ command(save_incr, St) ->
 command(revert, St0) ->
     case revert(St0) of
 	{error,Reason} ->
-	 wings_util:error(?STR(command,1,"Revert failed: ") ++ Reason),
+	    wings_u:error(?__(1,"Revert failed: ") ++ Reason),
 	    St0;
 	#st{}=St -> {save_state,St}
     end;
@@ -112,9 +177,9 @@ command(import_image, _St) ->
 command({import_image,Name}, _) ->
     import_image(Name);
 command({export,ndo}, St) ->
-    export_ndo(export, ?STR(command,2,"Export"), St);
+    export_ndo(export, ?__(2,"Export"), St);
 command({export_selected,ndo}, St) ->
-    export_ndo(export_selected, ?STR(command,3,"Export Selected"), St);
+    export_ndo(export_selected, ?__(3,"Export Selected"), St);
 command({export,{ndo,Filename}}, St) ->
     do_export_ndo(Filename, St);
 command({export_selected,{ndo,Filename}}, St) ->
@@ -130,9 +195,9 @@ command({install_plugin,Filename}, _St) ->
 command(quit, #st{saved=true}) ->
     quit;
 command(quit, _) ->
-    wings_util:yes_no_cancel(?STR(command,4,"Do you want to save your changes before quitting?"),
-			     fun() -> {file,{save,{file,quit}}} end,
-			     fun() -> {file,confirmed_quit} end);
+    wings_u:yes_no_cancel(?__(4,"Do you want to save your changes before quitting?"),
+			  fun() -> {file,{save,{file,quit}}} end,
+			  fun() -> {file,confirmed_quit} end);
 command(confirmed_quit, _) ->
     quit;
 command(Key, St) when is_integer(Key), 1 =< Key ->
@@ -144,7 +209,7 @@ command(Key, St) when is_integer(Key), 1 =< Key ->
 	false ->
 	    Recent = delete_nth(Recent0, Key),
 	    wings_pref:set_value(recent_files, Recent),
-	    wings_util:error(?STR(command,5,"This file has been moved or deleted."))
+	    wings_u:error(?__(5,"This file has been moved or deleted."))
     end.
 
 delete_nth([_|T], 1) -> T;
@@ -159,22 +224,22 @@ confirmed_new(#st{file=File}=St) ->
 new(#st{saved=true}=St0) ->
     St1 = clean_st(St0#st{file=undefined}),
     St = clean_images(wings_undo:init(St1)),
-    wings:caption(St),
+    wings_u:caption(St),
     {new,St#st{saved=true}};
 new(St0) ->			     %File is not saved or autosaved.
-    wings:caption(St0#st{saved=false}), 
-    wings_util:yes_no_cancel(str_save_changes(),
-			     fun() -> {file,{save,{file,new}}} end,
-			     fun() -> {file,confirmed_new} end).
+    wings_u:caption(St0#st{saved=false}), 
+    wings_u:yes_no_cancel(str_save_changes(),
+			  fun() -> {file,{save,{file,new}}} end,
+			  fun() -> {file,confirmed_new} end).
 
 open(#st{saved=true}) ->
     confirmed_open_dialog();
 open(St) ->
-    wings:caption(St#st{saved=false}),		%Clear any autosave flag.
+    wings_u:caption(St#st{saved=false}),		%Clear any autosave flag.
     Confirmed = {file,confirmed_open_dialog},
-    wings_util:yes_no_cancel(str_save_changes(),
-			     fun() -> {file,{save,Confirmed}} end,
-			     fun() -> Confirmed end).
+    wings_u:yes_no_cancel(str_save_changes(),
+			  fun() -> {file,{save,Confirmed}} end,
+			  fun() -> Confirmed end).
 
 confirmed_open_dialog() ->
     %% All confirmation questions asked. The former contents has either
@@ -183,8 +248,9 @@ confirmed_open_dialog() ->
 
     Cont = fun(Filename) -> {file,{confirmed_open,Filename}} end,
     Dir = wings_pref:get_value(current_directory),
-    Ps = [{directory,Dir},{title,?STR(confirmed_open_dialog,1,"Open")}|wings_prop()],
-    wpa:import_filename(Ps, Cont).
+    Ps = [{directory,Dir},
+	  {title,?__(1,"Open")}|wings_prop()],
+    import_filename(Ps, Cont).
 
 confirmed_open(Name, St0) ->
     Fun = fun(File) ->
@@ -198,10 +264,10 @@ confirmed_open(Name, St0) ->
 			  set_cwd(dirname(File)),
 			  St = clean_images(St3),
 			  add_recent(Name),
-			  wings:caption(St#st{saved=true,file=Name});
+			  wings_u:caption(St#st{saved=true,file=Name});
 		      {error,Reason} ->
 			  clean_new_images(St2),
-			  wings_util:error(?STR(confirmed_open,1,"Read failed: ") ++ Reason)
+			  wings_u:error(?__(1,"Read failed: ") ++ Reason)
 		  end
 	  end,
     use_autosave(Name, Fun).
@@ -209,20 +275,20 @@ confirmed_open(Name, St0) ->
 named_open(Name, #st{saved=true}=St) ->
     confirmed_open(Name, St);
 named_open(Name, St) ->
-    wings:caption(St#st{saved=false}),		%Clear any autosave flag.
+    wings_u:caption(St#st{saved=false}),		%Clear any autosave flag.
     Confirmed = {file,{confirmed_open,Name}},
-    wings_util:yes_no_cancel(str_save_changes(),
-			     fun() -> {file,{save,Confirmed}} end,
-			     fun() -> Confirmed end).
+    wings_u:yes_no_cancel(str_save_changes(),
+			  fun() -> {file,{save,Confirmed}} end,
+			  fun() -> Confirmed end).
 
 str_save_changes() ->
-    ?STR(str_save_changes,1,"Do you want to save your changes?").
+    ?__(1,"Do you want to save your changes?").
 
 merge() ->
     Cont = fun(Filename) -> {file,{merge,Filename}} end,
     Dir = wings_pref:get_value(current_directory),
-    Ps = [{title,?STR(merge,1,"Merge")},{directory,Dir}|wings_prop()],
-    wpa:import_filename(Ps, Cont).
+    Ps = [{title,?__(1,"Merge")},{directory,Dir}|wings_prop()],
+    import_filename(Ps, Cont).
 
 merge(Name, St0) ->
     Fun = fun(File) ->
@@ -233,7 +299,7 @@ merge(Name, St0) ->
 		  case ?SLOW(wings_ff_wings:import(File, St0)) of
 		      {error,Reason} ->
 			  clean_new_images(St1),
-			  wings_util:error(?STR(merge,2,"Read failed: ") ++ Reason);
+			  wings_u:error(?__(2,"Read failed: ") ++ Reason);
 		      #st{}=St ->
 			  set_cwd(dirname(Name)),
 			  St#st{saved=false}
@@ -253,8 +319,8 @@ save_as(Next, St) ->
 		   set_cwd(dirname(Name)),
 		   {file,{save_as,{Name,Next}}}
 	   end,
-    Ps = [{title,?STR(save_as,1,"Save")}|wings_prop()],
-    wpa:export_filename(Ps, St, Cont).
+    Ps = [{title,?__(1,"Save")}|wings_prop()],
+    export_filename(Ps, St, Cont).
 
 save_now(Next, #st{file=Name}=St) ->
     Backup = backup_filename(Name),
@@ -265,20 +331,20 @@ save_now(Next, #st{file=Name}=St) ->
 	    set_cwd(dirname(Name)),
 	    add_recent(Name),
 	    maybe_send_action(Next),
-	    {saved,wings:caption(St#st{saved=true})};
+	    {saved,wings_u:caption(St#st{saved=true})};
 	{error,Reason} ->
-	    wings_util:error(?STR(save_now,1,"Save failed: ") ++ Reason)
+	    wings_u:error(?__(1,"Save failed: ") ++ Reason)
     end.
 
 maybe_send_action(ignore) -> keep;
 maybe_send_action(Action) -> wings_wm:later({action,Action}).
     
 save_selected(#st{sel=[]}) ->
-    wings_util:error(?STR(save_selected,1,"This command requires a selection."));
+    wings_u:error(?__(1,"This command requires a selection."));
 save_selected(St) ->
-    Ps = [{title,?STR(save_selected,2,"Save Selected")}|wings_prop()],
+    Ps = [{title,?__(2,"Save Selected")}|wings_prop()],
     Cont = fun(Name) -> {file,{save_selected,Name}} end,
-    wpa:export_filename(Ps, St, Cont).
+    export_filename(Ps, St, Cont).
 
 save_selected(Name, #st{shapes=Shs0,sel=Sel}=St0) ->
     Shs = [Sh || {Id,_}=Sh <- gb_trees:to_list(Shs0),
@@ -286,7 +352,7 @@ save_selected(Name, #st{shapes=Shs0,sel=Sel}=St0) ->
     St = St0#st{shapes=gb_trees:from_orddict(Shs)},
     case ?SLOW(wings_ff_wings:export(Name, St)) of
 	ok -> keep;
-	{error,Reason} -> wings_util:error(Reason)
+	{error,Reason} -> wings_u:error(Reason)
     end.
 
 %%%
@@ -352,9 +418,9 @@ use_autosave_1(#file_info{mtime=SaveTime0}, File, Body) ->
 	    AutoTime = calendar:datetime_to_gregorian_seconds(AutoInfo0),
 	    if
 		AutoTime > SaveTime ->
-		    Msg = ?STR(use_autosave_1,1,"An autosaved file with a later time stamp exists; do you want to load the autosaved file instead?"),
-		    wings_util:yes_no(Msg, autosave_fun(Body, Auto),
-				      autosave_fun(Body, File));
+		    Msg = ?__(1,"An autosaved file with a later time stamp exists; do you want to load the autosaved file instead?"),
+		    wings_u:yes_no(Msg, autosave_fun(Body, Auto),
+				   autosave_fun(Body, File));
 		true ->
 		    Body(File)
 	    end;
@@ -413,10 +479,11 @@ autosave(#st{file=Name}=St) ->
     %% But I don't want to copy a really big model either.
     case ?SLOW(wings_ff_wings:export(Auto, St)) of
 	ok ->
-	    wings:caption(St#st{saved=auto});
+	    wings_u:caption(St#st{saved=auto});
 	{error,Reason} ->
-	    Msg = lists:flatten(io_lib:format(?STR(autosave,1,"Autosaving \"~s\" failed: ~s"), [Auto,Reason])),
-	    wings_util:message(Msg)
+	    F = ?__(1,"Autosaving \"~s\" failed: ~s"),
+	    Msg = lists:flatten(io_lib:format(F, [Auto,Reason])),
+	    wings_u:message(Msg)
     end.
 
 autosave_filename(File) ->
@@ -450,15 +517,17 @@ update_recent(Old, New) ->
 add_recent(File, [A,B,C,D,E|_]) -> [File,A,B,C,D,E];
 add_recent(File, Recent) -> [File|Recent].
 
-recent_files(Rest) ->
+recent_files(Tail) ->
     case wings_pref:get_value(recent_files, []) of
-	[] -> Rest;
-	Files -> number_files(Files, 1, [separator|Rest])
+	[] -> Tail;
+	Files ->
+	    Help = ?__(1,"Open this recently used file"),
+	    recent_files_1(Files, 1, Help, [separator|Tail])
     end.
 
-number_files([{Base,_}|T], I, Rest) ->
-    [{Base,I}|number_files(T, I+1, Rest)];
-number_files([], _I, Rest) -> Rest.
+recent_files_1([{Base,_}|T], I, Help, Tail) ->
+    [{Base,I,Help}|recent_files_1(T, I+1, Help, Tail)];
+recent_files_1([], _, _, Tail) -> Tail.
 
 %%
 %% The Revert command.
@@ -480,28 +549,28 @@ revert(#st{file=File}=St0) ->
 import_ndo() ->
     Ps = [{ext,".ndo"},{ext_desc,"Nendo File"}],
     Cont = fun(Name) -> {file,{import,{ndo,Name}}} end,
-    wpa:import_filename(Ps, Cont).
+    import_filename(Ps, Cont).
 
 import_ndo(Name, St0) ->
     case ?SLOW(wings_ff_ndo:import(Name, St0)) of
 	#st{}=St ->
 	    {save_state,St};
 	{error,Reason} ->
-	    wings_util:error(?STR(import_ndo,1,"Import failed: ") ++ Reason),
+	    wings_u:error(?__(1,"Import failed: ") ++ Reason),
 	    St0
     end.
 
 import_image() ->
-    Ps = [{extensions,wpa:image_formats()}],
+    Ps = [{extensions,wings_image:image_formats()}],
     Cont = fun(Name) -> {file,{import_image,Name}} end,
-    wpa:import_filename(Ps, Cont).
+    import_filename(Ps, Cont).
 
 import_image(Name) ->
     case wings_image:from_file(Name) of
 	Im when is_integer(Im) ->
 	    keep;
 	{error,Error} ->
-	    wings_util:error(?STR(import_image,1,"Failed to load \"~s\": ~s\n"),
+	    wings_u:error(?__(1,"Failed to load \"~s\": ~s\n"),
 			     [Name,file:format_error(Error)])
     end.
 
@@ -512,12 +581,12 @@ import_image(Name) ->
 export_ndo(Cmd, Title, St) ->
     Ps = [{title,Title},{ext,".ndo"},{ext_desc,"Nendo File"}],
     Cont = fun(Name) -> {file,{Cmd,{ndo,Name}}} end,
-    wpa:export_filename(Ps, St, Cont).
+    export_filename(Ps, St, Cont).
 
 do_export_ndo(Name, St) ->
     case wings_ff_ndo:export(Name, St) of
 	ok -> keep;
-	{error,Reason} -> wings_util:error(Reason)
+	{error,Reason} -> wings_u:error(Reason)
     end.
 
 %%%
@@ -525,14 +594,14 @@ do_export_ndo(Name, St) ->
 %%%
 
 install_plugin() ->
-    Props = [{title,?STR(install_plugin,1,"Install Plug-In")},
+    Props = [{title,?__(1,"Install Plug-In")},
 	     {extensions,
-	      [{".gz",?STR(install_plugin,2,"GZip Compressed File")},
-	       {".tar",?STR(install_plugin,3,"Tar File")},
-	       {".tgz",?STR(install_plugin,4,"Compressed Tar File")},
-	       {".beam",?STR(install_plugin,5,"Beam File")}]}],
+	      [{".gz",?__(2,"GZip Compressed File")},
+	       {".tar",?__(3,"Tar File")},
+	       {".tgz",?__(4,"Compressed Tar File")},
+	       {".beam",?__(5,"Beam File")}]}],
     Cont = fun(Name) -> {file,{install_plugin,Name}} end,
-    wpa:import_filename(Props, Cont).
+    import_filename(Props, Cont).
 
 %%%    
 %%% Utilities.
@@ -541,7 +610,7 @@ install_plugin() ->
 clean_st(St) ->
     foreach(fun(Win) ->
 		    wings_wm:set_prop(Win, wireframed_objects, gb_sets:empty())
-	    end, wings_util:geom_windows()),
+	    end, wings_u:geom_windows()),
     DefMat = wings_material:default(),
     Empty = gb_trees:empty(),
     Limit = wings_image:next_id(),
