@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_image.erl,v 1.42 2004/10/22 07:38:38 dgud Exp $
+%%     $Id: wings_image.erl,v 1.46 2005/01/31 12:53:42 dgud Exp $
 %%
 
 -module(wings_image).
@@ -19,6 +19,7 @@
 	 next_id/0,delete_older/1,delete_from/1,delete/1,
 	 update/2,update_filename/2,draw_preview/5,
 	 window/1]).
+-export([image_formats/0,image_read/1,image_write/1]).
 
 -export([loop/1]).
 -define(NEED_OPENGL, 1).
@@ -34,13 +35,30 @@ init() ->
 init_opengl() ->
     req(init_opengl).
 
+%%%
+%%% Interface against plug-ins.
+%%%
+image_formats() ->
+    wings_plugin:call_ui({image,formats,[]}).
+
+image_read(Ps) ->
+    wings_plugin:call_ui({image,read,Ps}).
+
+image_write(Ps) ->
+    case catch wings_plugin:call_ui({image,write,Ps}) of
+	{'EXIT',Reason} ->
+	    {error,{none,?MODULE,{crash,Reason}}};
+	Result -> Result
+    end.
+
+
 %%%
 %%% Client API.
 %%%
 
 from_file(Filename) ->
     Props = [{filename,Filename},{alignment,1}],
-    case wpa:image_read(Props) of
+    case image_read(Props) of
 	#e3d_image{}=Image ->
 	    Name = filename:rootname(filename:basename(Filename)),
 	    req({new,Image#e3d_image{filename=Filename,name=Name},false});
@@ -71,7 +89,7 @@ screenshot() ->
     gl:readPixels(0, 0, W, H, ?GL_RGB, ?GL_UNSIGNED_BYTE, Mem),
     ImageBin = sdl_util:getBin(Mem),
     Image = #e3d_image{image=ImageBin,width=W,height=H},
-    Id = new_temp(?STR(screenshot,1,"<<Screenshot>>"), Image),
+    Id = new_temp(?__(1,"<<Screenshot>>"), Image),
     wings_image:window(Id).
 
 bumpid(Id) ->
@@ -284,11 +302,12 @@ create_default(Type) when Type == diffuse; Type == gloss ->
 		     image= <<255,255,255>>}, % White and full specular.
     make_texture({?DEFAULT,Type},Img);
 create_default(Type) -> 
-    io:format(?STR(create_default,1,"~p Don't know about the type ~p; ignoring\n"), [?MODULE,Type]),
+    io:format(?__(1,"~p Don't know about the type ~p; ignoring\n"),
+	      [?MODULE,Type]),
     none.
 
 create_normal_cube_map() ->
-    case wings_util:is_gl_ext('GL_ARB_texture_cube_map') of
+    case wings_gl:is_ext('GL_ARB_texture_cube_map') of
 	true ->	
 	    [CubeMap] = gl:genTextures(1),
 	    gl:bindTexture(?GL_TEXTURE_CUBE_MAP, CubeMap),
@@ -477,7 +496,7 @@ window(Id) ->
 
 window_params(Id) ->
     #e3d_image{width=W0,height=H0,name=Name,bytes_pp=BytesPerPixel} = info(Id),
-    Title = flatten(io_lib:format(?STR(window_params,1,"Image: ~s [~wx~wx~w]"),
+    Title = flatten(io_lib:format(?__(1,"Image: ~s [~wx~wx~w]"),
 				  [Name,W0,H0,8*BytesPerPixel])),
     {DeskW,DeskH} = wings_wm:win_size(desktop),
     W = if
@@ -518,14 +537,14 @@ event(close, _) ->
     delete;
 event(got_focus, _) ->
     Msg2 = wings_camera:help(),
-    Msg3 = wings_util:button_format([], [],?STR(event,1,"Show menu")),
-    Message = wings_util:join_msg([Msg2,Msg3]),
+    Msg3 = wings_msg:button_format([], [],?__(1,"Show menu")),
+    Message = wings_msg:join([Msg2,Msg3]),
     wings_wm:message(Message),
     wings_wm:dirty();
 event({action,{viewer,Cmd}}, Id) ->
     command(Cmd, Id);
 event(Ev, Id) ->
-    case wings_camera:event(Ev, fun() -> redraw(Id) end) of
+    case wings_camera:event(Ev, #st{shapes=gb_trees:empty()},fun() -> redraw(Id) end) of
 	next -> event_1(Ev, Id);
 	Other -> Other
     end.
@@ -538,7 +557,7 @@ event_1(Ev, _) ->
 		    {"25%",25},
 		    {"50%",50},
 		    separator,
-		    {"100%",100,?STR(event_1,1,"Show in natural size")},
+		    {"100%",100,?__(1,"Show in natural size")},
 		    separator,
 		    {"200%",200},
 		    {"400%",400},
@@ -646,17 +665,17 @@ draw_image(X, Y, W, H, TxId) ->
 %%%
 
 create_image() ->
-    Qs = [{?STR(create_image,1,"Width"),256,[{range,{8,1024}}]},
-	  {?STR(create_image,2,"Height"),256,[{range,{8,1024}}]},
-	  {?STR(create_image,3,"Pattern"),
-	   {menu,[{?STR(create_image,4,"Grid"),grid},
-		  {?STR(create_image,5,"Checkerboard"),checkerboard},
-		  {?STR(create_image,6,"Vertical Bars"),vbars},
-		  {?STR(create_image,7,"Horizontal Bars"),hbars},
-		  {?STR(create_image,8,"White"),white},
-		  {?STR(create_image,9,"Black"),black}],
+    Qs = [{?__(1,"Width"),256,[{range,{8,1024}}]},
+	  {?__(2,"Height"),256,[{range,{8,1024}}]},
+	  {?__(3,"Pattern"),
+	   {menu,[{?__(4,"Grid"),grid},
+		  {?__(5,"Checkerboard"),checkerboard},
+		  {?__(6,"Vertical Bars"),vbars},
+		  {?__(7,"Horizontal Bars"),hbars},
+		  {?__(8,"White"),white},
+		  {?__(9,"Black"),black}],
 	    grid}}],
-    wings_ask:ask(?STR(create_image,10,"Create Image"), Qs,
+    wings_ask:ask(?__(10,"Create Image"), Qs,
 		  fun([W,H,Pattern]) ->
 			  create_image_1(Pattern, W, H),
 			  ignore
