@@ -8,11 +8,12 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_magnet.erl,v 1.50 2004/10/08 06:02:29 dgud Exp $
+%%     $Id: wings_magnet.erl,v 1.52 2004/12/18 19:36:20 bjorng Exp $
 %%
 
 -module(wings_magnet).
--export([setup/3,transform/2,recalc/3,flags/2,dialog/1,dialog/2]).
+-export([setup/3,transform/2,recalc/3,flags/2,dialog/1,dialog/2,
+	 info_string/0]).
 
 -include("wings.hrl").
 -import(lists, [map/2,foldr/3,foldl/3,sort/1,concat/1,reverse/1]).
@@ -29,6 +30,11 @@ setup({magnet,Type,Route,Point}, VsSel, We) ->
     Affected = foldl(fun({V,_,_,_}, A) -> [V|A] end, [], VsInf),
     {VsInf,Magnet,Affected}.
 
+info_string() ->
+    ["(",?__(1,"Magnet route:"),
+     atom_to_list(wings_pref:get_value(magnet_distance_route)),
+     ")"].
+
 transform(Trans, VsInf) ->
     transform_1(Trans, VsInf, []).
 
@@ -61,8 +67,8 @@ magnet_mode_fun() ->
 
 dialog(Fun) ->
     R0 = wings_pref:get_value(magnet_radius),
-	wings_ask:dialog(?STR(dialog,1,"Magnet Options"),
-      [{hframe,[{text,R0}],[{title,?STR(dialog,2,"Influence Radius")}]}|common_dialog()],
+	wings_ask:dialog(?__(1,"Magnet Options"),
+      [{hframe,[{text,R0}],[{title,?__(2,"Influence Radius")}]}|common_dialog()],
       fun([R,Route]) ->
 	      wings_pref:set_value(magnet_distance_route, Route),
 	      Type = wings_pref:get_value(magnet_type),
@@ -71,7 +77,7 @@ dialog(Fun) ->
       end).
 
 dialog(Point, Fun) ->
-    wings_ask:dialog(?STR(dialog,3,"Magnet Options"),
+    wings_ask:dialog(?__(3,"Magnet Options"),
 		     common_dialog(),
 		     fun([Route]) ->
 			     wings_pref:set_value(magnet_distance_route, Route),
@@ -82,13 +88,13 @@ dialog(Point, Fun) ->
 
 common_dialog() ->
     Route = wings_pref:get_value(magnet_distance_route),
-    [{hradio,[{?STR(common_dialog,1,"Shortest"),shortest},
-	      {?STR(common_dialog,2,"Midpoint"),midpoint},
-	      {?STR(common_dialog,3,"Surface"),surface}],
-      Route, [{title,?STR(common_dialog,4,"Distance Route")}]}].
+    [{hradio,[{?__(1,"Shortest"),shortest},
+	      {?__(2,"Midpoint"),midpoint},
+	      {?__(3,"Surface"),surface}],
+      Route, [{title,?__(4,"Distance Route")}]}].
 
 drag_help(Type) ->
-    ?STR(drag_help,1,"[+] or [-] Adjust Radius  ") ++
+    ?__(1,"[+] or [-] Adjust Radius  ") ++
 	help_1(Type, [{1,bell},{2,dome},{3,straight},{4,spike}]).
 
 help_1(Type, [{Digit,Type}|T]) ->
@@ -117,7 +123,7 @@ mf(spike, D0, R) when is_float(D0), is_float(R) ->
     D*D.
 
 check_radius(R) when R < 1.0E-6 ->
-    wings_util:error(?STR(check_radius,1,"Too short influence radius."));
+    wings_u:error(?__(1,"Too short influence radius."));
 check_radius(_) -> ok.
 
 %%%
