@@ -8,22 +8,16 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_material.erl,v 1.120 2004/10/22 07:43:07 dgud Exp $
+%%     $Id: wings_material.erl,v 1.127 2004/12/29 14:24:18 bjorng Exp $
 %%
 
 -module(wings_material).
 -export([material_menu/1,command/2,new/1,color/4,default/0,
-	 mat_faces/2,add_materials/2,add_materials/3,
+	 add_materials/2,add_materials/3,
 	 update_materials/2,
 	 update_image/4,used_images/1,
-	 get/2,get_all/1,delete_face/2,delete_faces/2,cleanup/1,
-	 assign/3,replace_materials/2,assign_materials/2,
-	 used_materials/1,used_materials_we/1,
-	 apply_material/2,is_transparent/2,
-	 renumber/2,merge/1]).
-
-%% For debugging.
--export([validate/1]).
+	 used_materials/1,
+	 apply_material/2,is_transparent/2]).
 
 -define(NEED_OPENGL, 1).
 -define(NEED_ESDL, 1).
@@ -34,17 +28,17 @@
 		keyreplace/4,keydelete/3,keysearch/3,flatten/1]).
 
 material_menu(St) ->
-    [{basic,{?STR(material_menu,1,"Material"),
+    [{basic,{?__(1,"Material"),
 	     {material,
-	      [{?STR(material_menu,2,"New..."),new,
-		?STR(material_menu,3,"Create a new material and assign to selected faces")},
+	      [{?__(2,"New..."),new,
+		?__(3,"Create a new material and assign to selected faces")},
 	       separator|mat_list(St)]}}},
-     {advanced,{?STR(material_menu,4,"Material"),{material,material_fun(St)}}}].
+     {advanced,{?__(4,"Material"),{material,material_fun(St)}}}].
 
 material_fun(St) ->
     fun(help, _Ns) ->
-	    {?STR(material_fun,1,"Assign existing material to selection"),[],
-	     ?STR(material_fun,2,"Create and assign new material")};
+	    {?__(1,"Assign existing material to selection"),[],
+	     ?__(2,"Create and assign new material")};
        (1, _Ns) ->
 	    mat_list(St);
        (3, _) ->
@@ -66,9 +60,9 @@ new(_) ->
     new_1(new).
 
 new_1(Act) ->
-    wings_ask:ask(?STR(new_1,1,"New Material"),
-		  [{?STR(new_1,2,"Material Name"),
-		    ?STR(new_1,3,"New Material")}],
+    wings_ask:ask(?__(1,"New Material"),
+		  [{?__(2,"Material Name"),
+		    ?__(3,"New Material")}],
 		  fun([Name]) ->
 			  Action = {action,{material,{Act,Name}}},
 			  wings_wm:send_after_redraw(geom, Action),
@@ -140,7 +134,7 @@ delete_material([], St) ->
 
 rename(Mats, St) ->
     Qs = rename_qs(Mats),
-    wings_ask:dialog(?STR(rename,1,"Rename"), Qs,
+    wings_ask:dialog(?__(1,"Rename"), Qs,
 		     fun(NewNames) ->
 			     rename_1(NewNames, St, [])
 		     end).
@@ -208,11 +202,12 @@ select_material(Mat, St) ->
 
 set_material(Mat, #st{selmode=face}=St) ->
     wings_sel:map(fun(Faces, We) ->
-			  assign(Mat, Faces, We#we{mode=material})
+			  wings_facemat:assign(Mat, Faces, We#we{mode=material})
 		  end, St);
 set_material(Mat, #st{selmode=body}=St) ->
     wings_sel:map(fun(_, #we{fs=Ftab}=We) ->
-			  assign(Mat, gb_trees:keys(Ftab), We#we{mode=material})
+			  wings_facemat:assign(Mat, gb_trees:keys(Ftab),
+					       We#we{mode=material})
 		  end, St);
 set_material(_, St) -> St.
 
@@ -314,16 +309,15 @@ load_map(MapName, Dir) ->
 	    none
     end.
 
-load_map_1(none, _) -> none;
 load_map_1(File0, Dir) ->
     File = filename:absname(File0, Dir),
     Ps = [{filename,File},{order,lower_left},{alignment,1}],
-    case wpa:image_read(Ps) of
+    case wings_image:image_read(Ps) of
 	#e3d_image{}=Im ->
 	    Name = filename:rootname(filename:basename(File)),
 	    wings_image:new(Name, Im#e3d_image{filename=File});
 	{error,Error} ->
-	    io:format(?STR(load_map_1,1,"Failed to load") ++ " \"~s\": ~s\n",
+	    io:format(?__(1,"Failed to load") ++ " \"~s\": ~s\n",
 		      [File,file:format_error(Error)]),
 	    none
     end.
@@ -387,7 +381,7 @@ apply_texture_1(Image, TxId) ->
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_MIN_FILTER, ?GL_LINEAR),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_S, ?GL_REPEAT),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_T, ?GL_REPEAT),    
-    case wings_util:is_gl_ext({1,2}) of
+    case wings_gl:is_ext({1,2}) of
 	true ->
 	    %% Calculate specular color correctly on textured models.
 	    gl:lightModeli(?GL_LIGHT_MODEL_COLOR_CONTROL,?GL_SEPARATE_SPECULAR_COLOR);
@@ -406,7 +400,7 @@ apply_texture_1(Image, TxId) ->
     true.
 
 no_texture() ->
-    case wings_util:is_gl_ext({1,2}) of
+    case wings_gl:is_ext({1,2}) of
 	true ->
 	    gl:lightModeli(?GL_LIGHT_MODEL_COLOR_CONTROL, ?GL_SINGLE_COLOR);
 	false -> 
@@ -420,21 +414,14 @@ no_texture() ->
 
 used_materials(#st{shapes=Shs,mat=Mat0}) ->
     Used0 = foldl(fun(We, A) ->
-			  [used_materials_we(We)|A]
+			  [wings_facemat:used_materials(We)|A]
 		  end, [], gb_trees:values(Shs)),
-    Used1 = gb_sets:union(Used0),
-    Used2 = sofs:from_external(gb_sets:to_list(Used1), [name]),
+    Used1 = ordsets:union(Used0),
+    Used2 = sofs:from_external(Used1, [name]),
     Mat = sofs:relation(gb_trees:to_list(Mat0), [{name,data}]),
     Used = sofs:restriction(Mat, Used2),
     sofs:to_external(Used).
 
-used_materials_we(#we{mat=Mat}) when is_atom(Mat) ->
-    gb_sets:singleton(Mat);
-used_materials_we(#we{mat=MatTab}) ->
-    Used0 = sofs:from_external(MatTab, [{face,material}]),
-    Used = sofs:range(Used0),
-    gb_sets:from_list(sofs:to_external(Used)).
-
 %% Return all image ids used by materials.
 
 used_images(#st{mat=Mat}) ->
@@ -466,7 +453,8 @@ is_mat_transparent(Mat) ->
 edit(Name, Assign, #st{mat=Mtab}=St) ->
     Mat = gb_trees:get(Name, Mtab),
     {dialog,Qs,Fun} = edit_dialog(Name, Assign, St, Mat),
-    wings_ask:dialog(?STR(edit,1,"Material Properties: ")++atom_to_list(Name), Qs, Fun).
+    wings_ask:dialog(?__(1,"Material Properties: ")++atom_to_list(Name),
+		     Qs, Fun).
     
 
 edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
@@ -485,10 +473,10 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 	     {hframe, 
 	      [{custom,?PREVIEW_SIZE,?PREVIEW_SIZE+5,Preview},
 	       {vframe,
-		[{label,?STR(edit_dialog,1,"Diffuse")},
-		 {label,?STR(edit_dialog,2,"Ambient")},
-		 {label,?STR(edit_dialog,3,"Specular")},
-		 {label,?STR(edit_dialog,4,"Emission")}]
+		[{label,?__(1,"Diffuse")},
+		 {label,?__(2,"Ambient")},
+		 {label,?__(3,"Specular")},
+		 {label,?__(4,"Emission")}]
 	       },
 	       {vframe,
 		[{slider,{color,Diff0,[{key,diffuse}]}},
@@ -496,8 +484,8 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 		 {slider,{color,Spec0,[{key,specular}]}},
 		 {slider,{color,Emiss0,[{key,emission}]}}
 		]}]},
-	     {hframe, [{vframe, [{label,?STR(edit_dialog,5,"Shininess")},
-				 {label,?STR(edit_dialog,6,"Opacity")}]},
+	     {hframe, [{vframe, [{label,?__(5,"Shininess")},
+				 {label,?__(6,"Opacity")}]},
 		       {vframe, [{slider,{text,Shine0,
 					  [{range,{0.0,1.0}},
 					   {key,shininess}]}},
@@ -508,7 +496,7 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 	   }],
     Qs2 = wings_plugin:dialog({material_editor_setup,Name,Mat0}, Qs1),
     Qs = {hframe,[{vframe,Qs2},
-		  {vframe,[{button,?STR(edit_dialog,7,"OK"),done,[ok,{key,material_editor_ok}]},
+		  {vframe,[{button,?__(7,"OK"),done,[ok,{key,material_editor_ok}]},
 			   {button,wings_s:cancel(),cancel,[cancel]}]}]},
     Ask = fun([{diffuse,Diff},{ambient,Amb},{specular,Spec},
 	       {emission,Emiss},{shininess,Shine},{opacity,Opacity}|More0]) ->
@@ -538,9 +526,9 @@ plugin_results(Name, Mat0, Res0) ->
 	{Mat,[{material_editor_ok,false}]} ->
 	    {again,Mat};
 	{_,Res} ->
-	    io:format(?STR(plugin_results,1,"Material editor plugin(s) left garbage:~n    ~P~n"), 
+	    io:format(?__(1,"Material editor plugin(s) left garbage:~n    ~P~n"), 
 		      [Res,20]),
-		      wings_util:error(?STR(plugin_results,2,"Plugin(s) left garbage"))
+		      wings_u:error(?__(2,"Plugin(s) left garbage"))
     end.
 
 update_maps(Mat0, More0) ->
@@ -560,18 +548,18 @@ show_maps(Mat) ->
 	[] -> [];
 	Maps ->
 	    MapDisp = [show_map(M) || M <- sort(Maps)],
-	    [{vframe,MapDisp,[{title,?STR(show_maps,1,"Textures")}]}]
+	    [{vframe,MapDisp,[{title,?__(1,"Textures")}]}]
     end.
 
 show_map({Type,Image}) ->
     Texture = 
 	case wings_image:info(Image) of
 	    none ->
-		[{label,flatten(io_lib:format(?STR(show_map,1,"~p: <image deleted>"), [Type]))}];
+		[{label,flatten(io_lib:format(?__(1,"~p: <image deleted>"), [Type]))}];
 	    #e3d_image{name=Name,width=W,height=H,bytes_pp=PP} ->
-		Label = flatten(io_lib:format(?STR(show_map,2,"~p: ~p [~px~px~p]"),
+		Label = flatten(io_lib:format(?__(2,"~p: ~p [~px~px~p]"),
 					      [Type,Name,W,H,PP*8])),
-		[{label, Label},{button,?STR(show_map,3,"Delete"),done}]
+		[{label, Label},{button,?__(3,"Delete"),done}]
 	end,
     {hframe, Texture}.
 
@@ -644,7 +632,7 @@ preview_mat(Key, Colors, Alpha) ->
 %%% Return color in texture for the given UV coordinates.
 
 color(Face, UV, We, #st{mat=Mtab}) ->
-    Name = get(Face, We),
+    Name = wings_facemat:face(Face, We),
     Props = gb_trees:get(Name, Mtab),
     Maps = prop_get(maps, Props),
     case prop_get(diffuse, Maps, none) of
@@ -665,165 +653,9 @@ color_1({U0,V0}, #e3d_image{width=W,height=H,image=Bits}) ->
     Pos = V*W*3 + U*3,
     <<_:Pos/binary,R:8,G:8,B:8,_/binary>> = Bits,
     wings_util:share(R/255, G/255, B/255).
-    
+
 prop_get(Key, Props) ->
     proplists:get_value(Key, Props).
 
 prop_get(Key, Props, Def) ->
     proplists:get_value(Key, Props, Def).
-
-%%%
-%%% New low-level interface introduced when the #face record was
-%%% removed. Ideally, the functions below should be the only ones
-%%% that exactly know how materials are implemented.
-%%%
-
-%% mat_faces([{Face,Info}], We) -> [{Mat,[{Face,Info}]}]
-%%  Group face tab into groups based on material.
-%%  Used for displaying objects.
-mat_faces(Ftab, #we{mirror=none}=We) ->
-    mat_faces_1(Ftab, We);
-mat_faces(Ftab, #we{mirror=Face}=We) ->
-    mat_faces_1(lists:keydelete(Face, 1, Ftab), We).
-
-mat_faces_1(Ftab, We) ->
-    case wings_pref:get_value(show_materials) of
-	false -> [{default,Ftab}];
-	true -> mat_faces_2(Ftab, We)
-    end.
-
-mat_faces_2(Ftab, #we{mat=AtomMat}) when is_atom(AtomMat) ->
-    [{AtomMat,Ftab}];
-mat_faces_2(Ftab0, #we{mat=MatTab}) ->
-    Ftab = mat_join(Ftab0, MatTab, []),
-    wings_util:rel2fam(Ftab).
-
-mat_join([{F1,_}|_]=Fs, [{F2,_}|Ms], Acc) when F2 < F1 ->
-    mat_join(Fs, Ms, Acc);
-mat_join([{F,Info}|Fs], [{F,Mat}|Ms], Acc) ->
-    mat_join(Fs, Ms, [{Mat,{F,Info}}|Acc]);
-mat_join([], _, Acc) -> Acc.
-
-get_all(#we{mat=Mat,fs=Ftab}) ->
-    force_list(Mat, Ftab).
-
-get(_, #we{mat=Atom}) when is_atom(Atom) -> Atom;
-get(Face, #we{mat=Tab}) ->
-    {value,{_,Mat}} = keysearch(Face, 1, Tab),
-    Mat.
-
-delete_face(_, #we{mat=AtomMat}=We) when is_atom(AtomMat) -> We;
-delete_face(Face, #we{mat=MatTab0}=We) ->
-    MatTab = orddict:erase(Face, MatTab0),
-    We#we{mat=MatTab}.
-
-delete_faces(_, #we{mat=AtomMat}=We) when is_atom(AtomMat) -> We;
-delete_faces(Faces0, #we{mat=MatTab0}=We) when is_list(Faces0) ->
-    Faces = sofs:from_external(Faces0, [face]),
-    MatTab1 = sofs:from_external(MatTab0, [{face,mat}]),
-    MatTab2 = sofs:drestriction(MatTab1, Faces),
-    MatTab = sofs:to_external(MatTab2),
-    We#we{mat=MatTab};
-delete_faces(Faces, We) ->
-    delete_faces(gb_sets:to_list(Faces), We).
-
-cleanup(#we{mat=Mat}=We) when is_atom(Mat) -> We;
-cleanup(#we{mat=Mat0,fs=Ftab}=We) ->
-    Fs = sofs:from_external(gb_trees:keys(Ftab), [face]),
-    Mat1 = sofs:from_external(Mat0, [{face,material}]),
-    Mat2 = sofs:restriction(Mat1, Fs),
-    Mat = sofs:to_external(Mat2),
-    case mat_all_same(Mat) of
-	true ->
-	    [{_,M}|_] = Mat,
-	    We#we{mat=M};
-	false ->
-	    We#we{mat=Mat}
-    end.
-
-mat_all_same([]) -> false;
-mat_all_same([{_,M}|T]) -> mat_all_same_1(T, M).
-
-mat_all_same_1([{_,M}|T], M) -> mat_all_same_1(T, M);
-mat_all_same_1([], _) -> true;
-mat_all_same_1(_, _) -> false.
-
-replace_materials(FaceMat, We) ->
-    We#we{mat=sort(FaceMat)}.
-    
-assign_materials([{M,F}|_]=MatFace, We) when is_atom(M), is_integer(F) ->
-    foldl(fun({Mat,Faces}, W) ->
-		  assign(Mat, Faces, W)
-	  end, We, wings_util:rel2fam(MatFace));
-assign_materials([{F,M}|_]=MatFace0, We) when is_integer(F), is_atom(M) ->
-    MatFace1 = sofs:relation(MatFace0),
-    MatFace2 = sofs:converse(MatFace1),
-    MatFace = sofs:to_external(MatFace2),
-    assign_materials(MatFace, We);
-assign_materials([], We) -> We.
-
-assign(Mat, _, #we{mat=Mat}=We) -> We;
-assign(Mat, Faces, #we{mat=Tab0,fs=Ftab}=We) when is_list(Faces) ->
-    case length(Faces) =:= gb_trees:size(Ftab) of
-	true ->
-	    We#we{mat=Mat};
-	false ->
-	    Tab = force_list(Tab0, Ftab),
-	    NewTab = sort(make_tab(Mat, Faces)),
-	    MatTab = mat_merge(NewTab, Tab, []),
-	    We#we{mat=MatTab}
-    end;
-assign(Mat, Faces, We) ->
-    assign(Mat, gb_sets:to_list(Faces), We).
-
-force_list(L, _) when is_list(L) -> L;
-force_list(M, Ftab) when is_atom(M) ->
-    reverse(make_tab(M, gb_trees:keys(Ftab))).
-
-make_tab(M, List) ->
-    foldl(fun(F, A) -> [{F,M}|A] end, [], List).
-
-mat_merge([{Fn,_}|_]=Fns, [{Fo,_}=Fold|Fos], Acc) when Fo < Fn ->
-    mat_merge(Fns, Fos, [Fold|Acc]);
-mat_merge([{Fn,_}=Fnew|Fns], [{Fo,_}|_]=Fos, Acc) when Fo > Fn ->
-    mat_merge(Fns, Fos, [Fnew|Acc]);
-mat_merge([Fnew|Fns], [_|Fos], Acc) -> % Equality
-    mat_merge(Fns, Fos, [Fnew|Acc]);
-mat_merge([], Fos, Acc) ->
-    reverse(Acc, Fos);
-mat_merge(Fns, [], Acc) ->
-    reverse(Acc, Fns).
-
-renumber(Mat, _) when is_atom(Mat) -> Mat;
-renumber(L, Fmap) -> renumber_1(L, Fmap, []).
-
-renumber_1([{F,M}|T], Fmap, Acc) ->
-    renumber_1(T, Fmap, [{gb_trees:get(F, Fmap),M}|Acc]);
-renumber_1([], _, Acc) -> reverse(Acc).
-
-merge([{M,_}|T]=L) ->
-    case all_same(T, M) of
-	true -> M;
-	false -> merge_1(L, [])
-    end.
-
-merge_1([{M,#we{es=Etab}}|T], Acc) when is_atom(M) ->
-    FsM = merge_2(gb_trees:values(Etab), M, []),
-    merge_1(T, [FsM|Acc]);
-merge_1([{FsM,_}|T], Acc) ->
-    merge_1(T, [FsM|Acc]);
-merge_1([], Acc) -> lists:merge(Acc).
-
-merge_2([#edge{lf=Lf,rf=Rf}|T], M, Acc) ->
-    merge_2(T, M, [{Lf,M},{Rf,M}|Acc]);
-merge_2([], _, Acc) -> ordsets:from_list(Acc).
-
-all_same([{M,_}|T], M) ->
-    all_same(T, M);
-all_same([], _) -> true;
-all_same([_|_], _) -> false.
-
-validate(#we{mat=Mat}) when is_atom(Mat) -> ok;
-validate(#we{mat=Mat,fs=Ftab}) ->
-    Faces = gb_trees:keys(Ftab),
-    Faces = [F || {F,_} <- Mat].
