@@ -3,12 +3,12 @@
 %%
 %%     This module handles picking using OpenGL.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_pick.erl,v 1.142 2004/11/02 07:20:34 bjorng Exp $
+%%     $Id: wings_pick.erl,v 1.148 2005/03/16 20:18:12 dgud Exp $
 %%
 
 -module(wings_pick).
@@ -105,7 +105,7 @@ handle_hilite_event(redraw, #hl{redraw=#st{sel=[]}=St,prev={_,Where,{_,Elem}}})
 	       mirror ->
 		   io_lib:format("#~p ~s", 
 				 [Elem,
-				  ?STR(handle_hilite_event,2,"(in mirror)")])
+				  ?__(2,"(in mirror)")])
 	   end,
     wings:redraw(Info, St),
     keep;
@@ -162,7 +162,14 @@ insert_hilite_dl_1(#dlo{src_we=#we{id=Id}}=D, {Mode,_,{Id,Item}=Hit}, St) ->
     List = gl:genLists(1),
     gl:newList(List, ?GL_COMPILE),
     hilite_color(Hit, St),
-    hilit_draw_sel(Mode, Item, D),
+    case wings_wm:lookup_prop(select_backface) of
+	{value,true} ->
+	    gl:disable(?GL_CULL_FACE),
+	    hilit_draw_sel(Mode, Item, D),
+	    gl:enable(?GL_CULL_FACE);
+	_ ->	    
+	    hilit_draw_sel(Mode, Item, D)
+    end,
     gl:endList(),
     D#dlo{hilite=List};
 insert_hilite_dl_1(#dlo{hilite=none}=D, _, _) -> D;
@@ -220,12 +227,12 @@ hilit_draw_sel(body, _, #dlo{src_we=We}=D) ->
 clear_hilite_marquee_mode(#marquee{st=St}=Pick) ->
     Ctrl = wings_s:key(ctrl),
     Shift = wings_s:key(shift),
-    CtrlMsg = ?STR(clear_hilite_marquee_mode,ctrl_action,"Deselect"),
-    ShiftMsg = ?STR(clear_hilite_marquee_mode,shift_action,
-		    "(De)select only elements wholly inside marquee"),
+    CtrlMsg = ?__(ctrl_action,"Deselect"),
+    ShiftMsg = ?__(shift_action,
+		   "(De)select only elements wholly inside marquee"),
     Mctrl = wings_util:key_format(Ctrl, CtrlMsg),
     Mshift = wings_util:key_format(Shift, ShiftMsg),
-    Message = wings_util:join_msg(Mctrl, Mshift),
+    Message = wings_msg:join(Mctrl, Mshift),
     wings_wm:message(Message),
     wings_wm:dirty_mode(front),
     {seq,push,
@@ -302,7 +309,7 @@ marquee_pick(true, X, Y0, W, H, St0) ->
 	    Hits1 = wings_util:rel2fam(Hits0),
 	    HitsOrig = [Hit || {Id,_}=Hit <- Hits1, Id > 0],
 	    HitsMirror = [Hit || {Id,_}=Hit <- Hits1, Id < 0],
-	    {MM,PM,ViewPort} = wings_util:get_matrices(0, original),
+	    {MM,PM,ViewPort} = wings_u:get_matrices(0, original),
 	    {_,_,_,Wh} = ViewPort,
 	    Y = Wh - Y0,
 	    RectData = {MM,PM,ViewPort,X-W/2,Y-H/2,X+W/2,Y+H/2},
@@ -317,7 +324,7 @@ marquee_convert([{Id,Faces}|Hits], RectData0,
     We = gb_trees:get(abs(Id), Shs),
     RectData = if
 		   Id < 0 ->
-		       {MM,PM,_} = wings_util:get_matrices(-Id, mirror),
+		       {MM,PM,_} = wings_u:get_matrices(-Id, mirror),
 		       RectData1 = setelement(2, RectData0, PM),
 		       setelement(1, RectData1, MM);
 		   true -> RectData0
@@ -337,7 +344,8 @@ marquee_convert_1(Faces0, face, Rect, #we{vp=Vtab}=We) ->
 		     [{V,Face}|A]
 	     end, [], Faces0, We),
     Vfs = wings_util:rel2fam(Vfs0),
-    Kill0 = [Fs || {V,Fs} <- Vfs, not is_inside_rect(gb_trees:get(V, Vtab), Rect)],
+    Kill0 = [Fs || {V,Fs} <- Vfs,
+		   not is_inside_rect(gb_trees:get(V, Vtab), Rect)],
     Kill1 = sofs:set(Kill0, [[face]]),
     Kill = sofs:union(Kill1),
     Faces1 = sofs:from_external(Faces0, [face]),
@@ -393,8 +401,7 @@ marquee_update_sel(Op, Hits0, St) ->
     Hits = sofs:relation_to_family(Hits1),
     marquee_update_sel_1(Op, Hits, St).
 
-marquee_update_sel_1(add, Hits0, #st{sel=Sel0}=St) ->
-    Hits = marquee_filter_hits(Hits0, St),
+marquee_update_sel_1(add, Hits, #st{sel=Sel0}=St) ->
     Sel1 = [{Id,gb_sets:to_list(Items)} || {Id,Items} <- Sel0],
     Sel2 = sofs:from_external(Sel1, [{id,[data]}]),
     Sel3 = sofs:family_union(Sel2, Hits),
@@ -409,18 +416,6 @@ marquee_update_sel_1(delete, Hits, #st{sel=Sel0}=St) ->
     Sel = [{Id,gb_sets:from_list(Items)} || {Id,Items} <- Sel4, Items =/= []],
     St#st{sel=Sel}.
 
-%% Filter out any mirror face from the hits.
-marquee_filter_hits(Hits0, #st{selmode=face,shapes=Shs}) ->
-    Type = sofs:type(Hits0),
-    Hits = map(fun({Id,Faces}=Hit) ->
-		case gb_trees:get(Id, Shs) of
-		    #we{mirror=none} -> Hit;
-		    #we{mirror=Face} -> {Id,delete(Face, Faces)}
-		end
-	       end, sofs:to_external(Hits0)),
-    sofs:from_external(Hits, Type);
-marquee_filter_hits(Hits, _) -> Hits.
-
 %%
 %% Drag picking.
 %%
@@ -465,9 +460,14 @@ raw_pick(X0, Y0, #st{selmode=Mode}=St) ->
     glu:pickMatrix(X, Y, S, S, {0,0,W,H}),
     wings_view:projection(),
     wings_view:modelview(),
-    gl:enable(?GL_CULL_FACE),
-    draw(),
-    gl:disable(?GL_CULL_FACE),
+    case wings_wm:lookup_prop(select_backface) of
+	{value,true} ->
+	    draw();
+	_ ->
+	    gl:enable(?GL_CULL_FACE),
+	    draw(),
+	    gl:disable(?GL_CULL_FACE)
+    end,
     Hits = get_hits(HitBuf),
     case best_face_hit(Hits, Mode) of
 	none -> none;
@@ -572,7 +572,7 @@ best_face_hit_2(#we{id=AbsId}=We, Ns, Ray, {[{AbsId,Id,Face}|Hits],Hit0,T0}) ->
     end,
     P = if 
 	    Id < 0 ->
-		e3d_mat:mul_point(wings_util:mirror_matrix(AbsId), P0);
+		e3d_mat:mul_point(wings_dl:mirror_matrix(AbsId), P0);
 	    true ->
 		P0
 	end,
@@ -613,7 +613,7 @@ convert_hit_1(body, _X, _Y, Id, _Face, MM, _We) ->
 convert_hit_1(face, _X, _Y, Id, Face, MM, _We) ->
     {face,MM,{Id,Face}};
 convert_hit_1({auto,_}, X, Y, Id, Face, MM, We) ->
-    Trans = wings_util:get_matrices(Id, MM),
+    Trans = wings_u:get_matrices(Id, MM),
     Vs = sort(find_vertex(Face, We, X, Y, Trans)),
     [{Vdist0,{Xva,Yva},V},{_,{Xvb,Yvb},_}|_] = Vs,
     Vdist = math:sqrt(Vdist0),
@@ -632,7 +632,7 @@ convert_hit_1({auto,_}, X, Y, Id, Face, MM, We) ->
 	     end,
     check_restriction(Hilite, Id, V, Edge, Face);
 convert_hit_1(Mode, X, Y, Id, Face, MM, We) ->
-    Trans = wings_util:get_matrices(Id, MM),
+    Trans = wings_u:get_matrices(Id, MM),
     case Mode of
 	vertex ->
 	    {_,_,V} = min(find_vertex(Face, We, X, Y, Trans)),
@@ -663,11 +663,12 @@ find_edge(Face, We, Cx, Cy, Trans) ->
 		      Ydist = By-Ay,
 		      L = Xdist*Xdist+Ydist*Ydist,
 		      {Px,Py} =
-			  case catch ((Cx-Ax)*Xdist+(Cy-Ay)*Ydist)/L of
-			      {'EXIT',_} -> {Ax,Ay};
+			  try ((Cx-Ax)*Xdist+(Cy-Ay)*Ydist)/L of
 			      R when R =< 0 -> {Ax,Ay};
 			      R when R >= 1 -> {Bx,By};
 			      R -> {Ax+R*Xdist,Ay+R*Ydist}
+			  catch
+			      error:badarith -> {Ax,Ay}
 			  end,
 		      Xdiff = Px-Cx,
 		      Ydiff = Py-Cy,
@@ -748,17 +749,34 @@ marquee_draw(#st{selmode=edge}) ->
 	   end,
     marquee_draw_1(Draw);
 marquee_draw(#st{selmode=vertex}) ->
-    Draw = fun(#we{vp=Vtab}) ->
-		   foreach(fun({V,Pos}) ->
-				   gl:loadName(V),
-				   gl:'begin'(?GL_POINTS),
-				   gl:vertex3fv(Pos),
-				   gl:'end'()
-			   end, gb_trees:to_list(Vtab))
+    Draw = fun(#we{vp=Vtab}=We) ->
+		   case wings_we:any_hidden(We) of
+		       false ->
+			   marquee_draw_all_vs(gb_trees:to_list(Vtab));
+		       true ->
+			   marquee_draw_some_vs(wings_we:visible_vs(We),
+						Vtab)
+		   end
 	   end,
     marquee_draw_1(Draw);
 marquee_draw(_) -> draw().
 
+marquee_draw_all_vs([{V,Pos}|VsPos]) ->
+    gl:loadName(V),
+    gl:'begin'(?GL_POINTS),
+    gl:vertex3fv(Pos),
+    gl:'end'(),
+    marquee_draw_all_vs(VsPos);
+marquee_draw_all_vs([]) -> ok.
+
+marquee_draw_some_vs([V|Vs], Vtab) ->
+    gl:loadName(V),
+    gl:'begin'(?GL_POINTS),
+    gl:vertex3fv(gb_trees:get(V, Vtab)),
+    gl:'end'(),
+    marquee_draw_some_vs(Vs, Vtab);
+marquee_draw_some_vs([], _) -> ok.
+
 marquee_draw_edges([{Edge,#edge{vs=Va,ve=Vb,lf=Lf,rf=Rf}}|Es], Vtab, Vis) ->
     case gb_sets:is_member(Lf, Vis) orelse gb_sets:is_member(Rf, Vis) of
 	false -> ok;
@@ -838,14 +856,13 @@ draw_dlist(#dlo{mirror=Matrix,pick=Pick,src_we=#we{id=Id}}=D) ->
     gl:frontFace(?GL_CCW),
     D.
 
-draw_1(#dlo{ns=Ns0,src_we=#we{perm=Perm,mirror=Mirror}=We})
+draw_1(#dlo{ns=Ns0,src_we=#we{perm=Perm}=We})
   when ?IS_SELECTABLE(Perm) ->
     Ns = wings_we:visible(gb_trees:to_list(Ns0), We),
     gl:pushName(0),
-    foreach(fun({Face,Info}) when Face =/= Mirror ->
+    foreach(fun({Face,Info}) ->
 		    gl:loadName(Face),
-		    face(Info);
-	       (_) -> ok
+		    face(Info)
 	    end, Ns),
     gl:popName();
 draw_1(_) -> ok.
