@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_vec.erl,v 1.109 2004/10/15 15:40:30 bjorng Exp $
+%%     $Id: wings_vec.erl,v 1.112 2004/12/18 19:36:22 bjorng Exp $
 %%
 
 -module(wings_vec).
@@ -76,10 +76,10 @@ add_help_text([{_,_}=Pair|T]) ->
 add_help_text([Type|T]) ->
     Val = {Type,
 	   case Type of
-	       axis_point ->?STR(add_help_text,1,"Pick axis (and point)");
-	       axis -> ?STR(add_help_text,2,"Pick axis");
-	       point -> ?STR(add_help_text,3,"Pick point");
-	       magnet -> ?STR(add_help_text,4,"Pick outer boundary point for magnet influence");
+	       axis_point ->?__(1,"Pick axis (and point)");
+	       axis -> ?__(2,"Pick axis");
+	       point -> ?__(3,"Pick point");
+	       magnet -> ?__(4,"Pick outer boundary point for magnet influence");
 	       _ -> []
 	   end},
     [Val|add_help_text(T)];
@@ -95,22 +95,22 @@ magnet_possible_now(Pl, _) ->
 
 common_message(Msg, More, MagnetPossible, Right) ->
     Rmb = case More of
-	      [] -> ?STR(common_message,1,"Execute");
-	      [_|_] ->?STR(common_message,2,"Continue")
+	      [] -> ?__(1,"Execute");
+	      [_|_] ->?__(2,"Continue")
 	  end,
-    Message = wings_util:join_msg(wings_util:button_format(Msg, [], Rmb),
-				  common_magnet_message(MagnetPossible)),
+    Message = wings_msg:join(wings_msg:button_format(Msg, [], Rmb),
+			     common_magnet_message(MagnetPossible)),
     wings_wm:message(Message, Right).
 
 common_magnet_message(no) -> [];
 common_magnet_message(inactive) ->
-    [$\s,wings_util:rmb_format(?STR(common_magnet_message,1,"Magnet"))];
+    [$\s,wings_msg:rmb_format(?__(1,"Magnet"))];
 common_magnet_message(active) ->
-    ["  "|wings_util:magnet_string()].
+    ["  "|wings_magnet:info_string()].
 
 magnet_message(Msg, Right) ->
-    Message = wings_util:join_msg(wings_util:button_format(Msg, [], ?STR(magnet_message,1,"Execute ")),
-	     wings_util:rmb_format(?STR(magnet_message,2,"Magnet options"))),
+    Message = wings_msg:join(wings_msg:button_format(Msg, [], ?__(1,"Execute ")),
+			     wings_msg:rmb_format(?__(2,"Magnet options"))),
     wings_wm:message(Message, Right).
 
 mode_restriction(Modes, #st{selmode=Mode}=St) ->
@@ -146,7 +146,7 @@ handle_event({ask_init,Do,Done}, #ss{selmodes=Modes}=Ss,
     wings_dl:map(fun(#dlo{orig_sel=none,sel=Dlist}=D, _) ->
 			 D#dlo{orig_sel=Dlist,orig_mode=Mode}
 		 end, []),
-    wings_util:menu_restriction(wings_wm:this(), [view,select]),
+    wings_u:menu_restriction(wings_wm:this(), [view,select]),
     St = wings_sel:reset(mode_restriction(Modes, St0)),
     pick_next(Do, Done, Ss, St);
 handle_event(Event, Ss, St) ->
@@ -336,7 +336,7 @@ exit_menu(X, Y, Mod, #ss{f=Exit,vec=Vec}=Ss, St) ->
     case Exit(exit, {Mod,Vec,St}) of
 	error ->
 	    Menu = [{wings_s:cancel(),abort,
-		     ?STR(exit_menu,2,"Cancel current command")}],
+		     ?__(2,"Cancel current command")}],
 	    wings_menu:popup_menu(X, Y, secondary_selection, Menu);
 	keep ->
 	    keep;
@@ -347,7 +347,7 @@ exit_menu(X, Y, Mod, #ss{f=Exit,vec=Vec}=Ss, St) ->
 common_exit(_, none, _, _, _, _, _) ->
     error;
 common_exit(Type, Vec, Mod, More, Acc, inactive, _St) ->
-    RmbMod = wings_camera:free_rmb_modifier(),
+    RmbMod = wings_msg:free_rmb_modifier(),
     if
 	Mod band RmbMod =:= 0 ->
 	    common_exit_1(Type, Vec, More, Acc);
@@ -371,7 +371,7 @@ common_exit_1(point, Point, PickList, Acc) ->
     {PickList,add_to_acc(Point, Acc)}.
 
 add_magnet(More) ->
-    More ++ [{magnet,?STR(add_magnet,1,"Pick outer boundary point for magnet influence")}].
+    More ++ [{magnet,?__(1,"Pick outer boundary point for magnet influence")}].
 
 add_to_acc(Vec, [radial]) -> [{radial,Vec}];
 add_to_acc(Vec, Acc) -> [Vec|Acc].
@@ -394,7 +394,7 @@ check_vector(#st{selmode=Mode,sel=[{Id,Elems0}],shapes=Shs}) ->
     We = gb_trees:get(Id, Shs),
     Elems = gb_sets:to_list(Elems0),
     get_vec(Mode, Elems, We);
-check_vector(_) -> {none,?STR(check_vector,1,"Select parts of one object only")}.
+check_vector(_) -> {none,?__(1,"Select parts of one object only")}.
 
 %% Use single edge as axis
 get_vec(edge, [Edge], #we{es=Etab,vp=Vtab}=We) ->
@@ -407,9 +407,9 @@ get_vec(edge, [Edge], #we{es=Etab,vp=Vtab}=We) ->
     Rn = wings_face:normal(Rf, We),
     Normal = e3d_vec:norm(e3d_vec:add(Ln, Rn)),
     [{{Center,Vec},
-      {?STR(get_vec,1,"Edge saved as axis"),?STR(get_vec,2,"Save edge normal")}},
+      {?__(1,"Edge saved as axis"),?__(2,"Save edge normal")}},
      {{Center,Normal},
-      {?STR(get_vec,3,"Edge normal saved as axis"),?STR(get_vec,4,"Save edge direction")}}];
+      {?__(3,"Edge normal saved as axis"),?__(4,"Save edge direction")}}];
 %% Use direction between two edges
 get_vec(edge, [Edge1,Edge2], #we{es=Etab,vp=Vtab}) ->
     #edge{vs=Va1,ve=Vb1} = gb_trees:get(Edge1, Etab),
@@ -422,23 +422,23 @@ get_vec(edge, [Edge1,Edge2], #we{es=Etab,vp=Vtab}) ->
     Center2 = e3d_vec:average([Va2Pos,Vb2Pos]),
     Center = e3d_vec:average([Center1,Center2]),
     Vec = e3d_vec:norm_sub(Center1, Center2),
-    [{{Center,Vec},?STR(get_vec,5,"Direction between edges saved as axis.")}];
+    [{{Center,Vec},?__(5,"Direction between edges saved as axis.")}];
 %% Use edge-loop normal.
 get_vec(edge, Edges, #we{vp=Vtab}=We) ->
     case wings_edge_loop:edge_loop_vertices(Edges, We) of
 	[Vs] -> 
 	    Center = wings_vertex:center(Vs, We),
 	    Vec = wings_face:face_normal_ccw(Vs, Vtab),
-	    [{{Center,Vec},?STR(get_vec,6,"Edge loop normal saved as axis.")}];
+	    [{{Center,Vec},?__(6,"Edge loop normal saved as axis.")}];
 	_Other ->
-	    [{none,?STR(get_vec,7,"Multi-edge selection must form a single closed edge loop.")}]
+	    [{none,?__(7,"Multi-edge selection must form a single closed edge loop.")}]
     end;
 
 %% Vertex normal
 get_vec(vertex, [V], We) ->
     Vec = wings_vertex:normal(V, We),
     Center = wings_vertex:center([V], We),
-    [{{Center,Vec},?STR(get_vec,8,"Vertex normal saved.")}];
+    [{{Center,Vec},?__(8,"Vertex normal saved.")}];
 %% Direction between 2 vertices as axis
 get_vec(vertex, [Va,Vb]=Vs, We) ->
     VaPos = wings_vertex:pos(Va, We),
@@ -448,17 +448,17 @@ get_vec(vertex, [Va,Vb]=Vs, We) ->
     Normal = e3d_vec:norm(e3d_vec:add(wings_vertex:normal(Va, We),
 				      wings_vertex:normal(Vb, We))),
     [{{Center,Vec},
-      {?STR(get_vec,9,"Direction between vertices saved as axis."),
-       ?STR(get_vec,10,"Use average of vertex normals as axis")}},
+      {?__(9,"Direction between vertices saved as axis."),
+       ?__(10,"Use average of vertex normals as axis")}},
      {{Center,Normal},
-      {?STR(get_vec,11,"Average of vertex normals saved as axis."),
-       ?STR(get_vec,12,"Use direction between vertices as axis")}}];
+      {?__(11,"Average of vertex normals saved as axis."),
+       ?__(12,"Use direction between vertices as axis")}}];
 
 %% 3-point (defines face) perpendicular
 get_vec(vertex, [_,_,_]=Vs, #we{vp=Vtab}=We) ->
     Vec = wings_face:face_normal_ccw(Vs, Vtab),
     Center = wings_vertex:center(Vs, We),
-    [{{Center,Vec},?STR(get_vec,13,"3-point perp. normal saved as axis.")}];
+    [{{Center,Vec},?__(13,"3-point perp. normal saved as axis.")}];
 %% Take the edge loop normal.
 get_vec(vertex, Vs0, #we{vp=Vtab}=We) ->
     Edges = find_edges(Vs0, We),
@@ -466,9 +466,9 @@ get_vec(vertex, Vs0, #we{vp=Vtab}=We) ->
 	[Vs] -> 
 	    Center = wings_vertex:center(Vs, We),
 	    Vec = wings_face:face_normal_cw(Vs, Vtab),
-	    [{{Center,Vec},?STR(get_vec,14,"Vertex loop normal saved as axis.")}];
+	    [{{Center,Vec},?__(14,"Vertex loop normal saved as axis.")}];
 	_Other ->
-	    [{none,?STR(get_vec,15,"Multi-vertex selection must form a single closed edge loop.")}]
+	    [{none,?__(15,"Multi-vertex selection must form a single closed edge loop.")}]
     end;
 
 %% Face normal
@@ -476,7 +476,7 @@ get_vec(face, [Face], We) ->
     Vec = wings_face:normal(Face, We),
     Vs = wings_face:to_vertices([Face], We),
     Center = wings_vertex:center(Vs, We),
-    [{{Center,Vec},?STR(get_vec,16,"Face normal saved as axis.")}];
+    [{{Center,Vec},?__(16,"Face normal saved as axis.")}];
 %% Direction between two faces as axis
 get_vec(face, [Face1,Face2], We) ->
     Center1 = wings_face:center(Face1, We),
@@ -487,22 +487,22 @@ get_vec(face, [Face1,Face2], We) ->
     Face2n = wings_face:normal(Face2, We),
     Normal = e3d_vec:norm(e3d_vec:add(Face1n, Face2n)),
     [{{Center,Vec},
-      {?STR(get_vec,17,"Direction between face centers saved as axis."),
-       ?STR(get_vec,18,"Use average of face normals")}},
+      {?__(17,"Direction between face centers saved as axis."),
+       ?__(18,"Use average of face normals")}},
      {{Center,Normal},
-      {?STR(get_vec,19,"Average of face normals saved as axis."),
-       ?STR(get_vec,20,"Use direction between face centers")}}];
+      {?__(19,"Average of face normals saved as axis."),
+       ?__(20,"Use direction between face centers")}}];
 get_vec(face, Faces, #we{vp=Vtab}=We) ->
     case wings_vertex:outer_partition(Faces, We) of
 	[Vs] ->
 	    Center = wings_vertex:center(Vs, We),
 	    Vec = wings_face:face_normal_cw(Vs, Vtab),
-	    [{{Center,Vec},?STR(get_vec,21,"Edge loop normal for region saved as axis.")}];
+	    [{{Center,Vec},?__(21,"Edge loop normal for region saved as axis.")}];
 	_Other ->
-	    [{none,?STR(get_vec,22,"Multi-face selection must have a single edge loop.")}]
+	    [{none,?__(22,"Multi-face selection must have a single edge loop.")}]
     end;
 
-get_vec(_, _, _) -> {none,?STR(get_vec,23,"Select vertices, edges, or faces.")}.
+get_vec(_, _, _) -> {none,?__(23,"Select vertices, edges, or faces.")}.
 
 %%%
 %%% Point functions.
@@ -513,23 +513,23 @@ check_point(St) ->
     case kill_mirror(St) of
 	St ->
 	    Center = e3d_vec:average(wings_sel:bounding_box(St)),
-	    [{Center,?STR(check_point,1,"Midpoint of selection saved.")}];
+	    [{Center,?__(1,"Midpoint of selection saved.")}];
 	NoMirror ->
 	    Center = e3d_vec:average(wings_sel:bounding_box(St)),
 	    NoMirrorCenter = e3d_vec:average(wings_sel:bounding_box(NoMirror)),
 	    [{Center,
-	      {?STR(check_point,2,"Midpoint of selection saved."),
-	       ?STR(check_point,3,"Disregard virtual mirror in reference point calculation")}},
+	      {?__(2,"Midpoint of selection saved."),
+	       ?__(3,"Disregard virtual mirror in reference point calculation")}},
 	     {NoMirrorCenter,
-	      {?STR(check_point,4,"Midpoint of selection saved."),
-	       ?STR(check_point,5,"Include virtual mirror in reference point calculation")}}]
+	      {?__(4,"Midpoint of selection saved."),
+	       ?__(5,"Include virtual mirror in reference point calculation")}}]
     end.
 
 check_magnet_point(#st{sel=[]}) -> {none,""};
 check_magnet_point(St0) ->
     St = kill_mirror(St0),
     Center = e3d_vec:average(wings_sel:bounding_box(St)),
-    [{Center,?STR(check_magnet_point,1,"Midpoint of selection saved.")}].
+    [{Center,?__(1,"Midpoint of selection saved.")}].
 
 kill_mirror(#st{shapes=Shs0}=St) ->
     Shs = kill_mirror_1(gb_trees:values(Shs0), []),
@@ -547,7 +547,7 @@ kill_mirror_1([], Acc) ->
 %%%
 
 exit_magnet(Vec, Mod, Acc) ->
-    case wings_camera:free_rmb_modifier() of
+    case wings_msg:free_rmb_modifier() of
 	ModRmb when Mod band ModRmb =/= 0 ->
 	    Fun = fun(Mag) ->
 			  {[],[Mag|Acc]}
