@@ -5,16 +5,16 @@
 %%     we records (winged-edged records, the central data structure
 %%     in Wings 3D).
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_we.erl,v 1.82 2004/10/09 05:44:36 bjorng Exp $
+%%     $Id: wings_we.erl,v 1.108 2005/01/30 11:34:56 bjorng Exp $
 %%
 
 -module(wings_we).
--export([build/2,rebuild/1,vertex_gc/1,
+-export([build/2,rebuild/1,
 	 new_wrap_range/3,id/2,bump_id/1,
 	 new_id/1,new_ids/2,
 	 invert_normals/1,
@@ -25,30 +25,117 @@
 	 transform_vs/2,
 	 separate/1,
 	 normals/2,
-	 new_items/3,
+	 new_items_as_ordset/3,new_items_as_gbset/3,
 	 is_consistent/1,is_face_consistent/2,
-	 hide_faces/2,show_faces/1,any_hidden/1,visible/1,visible/2]).
+	 hide_faces/2,show_faces/1,num_hidden/1,
+	 any_hidden/1,all_hidden/1,
+	 visible/1,visible/2,visible_vs/1,visible_vs/2,
+	 visible_edges/1,visible_edges/2,
+	 validate_mirror/1,mirror_flatten/2]).
 
 -include("wings.hrl").
 -include("e3d.hrl").
 -import(lists, [map/2,foreach/2,foldl/3,sort/1,keysort/2,
 		last/1,reverse/1,duplicate/2,seq/2,filter/2,zip/2]).
 
-rebuild(#we{vc=undefined,fs=undefined,es=Etab0}=We) ->
+%%%
+%%% API.
+%%%
+
+build(Mode, #e3d_mesh{fs=Fs0,vs=Vs,tx=Tx,he=He}) when is_atom(Mode) ->
+    Fs = translate_faces(Fs0, list_to_tuple(Tx), []),
+    build(Mode, Fs, Vs, He);
+build(Fs, Vs) ->
+    build(material, Fs, Vs, []).
+
+%% rebuild(We) -> We'
+%%  Rebuild any missing 'vc' and 'fs' tables. If there are
+%%  fewer elements in the 'vc' table than in the 'vp' table,
+%%  remove redundant entries in the 'vp' table. Updated id
+%%  bounds.
+rebuild(#we{vc=undefined,fs=undefined,es=Etab0}=We0) ->
     Etab = gb_trees:to_list(Etab0),
-    Vct = rebuild_vct(Etab),
     Ftab = rebuild_ftab(Etab),
-    rebuild(We#we{vc=Vct,fs=Ftab});
+    VctList = rebuild_vct(Etab),
+    We = We0#we{vc=gb_trees:from_orddict(VctList),fs=Ftab},
+    rebuild_1(VctList, We);
 rebuild(#we{vc=undefined,es=Etab}=We) ->
-    Vct = rebuild_vct(gb_trees:to_list(Etab)),
-    rebuild(We#we{vc=Vct});
+    VctList = rebuild_vct(gb_trees:to_list(Etab), []),
+    rebuild_1(VctList, We#we{vc=gb_trees:from_orddict(VctList)});
 rebuild(#we{fs=undefined,es=Etab}=We) ->
     Ftab = rebuild_ftab(gb_trees:to_list(Etab)),
     rebuild(We#we{fs=Ftab});
 rebuild(We) -> update_id_bounds(We).
 
+%%% Utilities for allocating IDs.
+
+new_wrap_range(Items, Inc, #we{next_id=Id}=We) ->
+    NumIds = Items*Inc,
+    {{0,Id,Inc,NumIds},We#we{next_id=Id+NumIds}}.
+
+id(N, {Current,BaseId,_Inc,NumIds}) ->
+    BaseId + ((Current+N) rem NumIds).
+
+bump_id({Id,BaseId,Inc,NumIds}) ->
+    {Id+Inc,BaseId,Inc,NumIds}.
+
+new_id(#we{next_id=Id}=We) ->
+    {Id,We#we{next_id=Id+1}}.
+
+new_ids(N, #we{next_id=Id}=We) ->
+    {Id,We#we{next_id=Id+N}}.
+
+%%% Returns sets of newly created items.
+
+%% new_items_as_ordset(vertex|edge|face, OldWe, NewWe) -> NewItemsSet.
+%% new_items_as_gbset(vertex|edge|face, OldWe, NewWe) -> NewItemsSet.
+%%  Return all items in NewWe that are not in OldWe.
+
+new_items_as_gbset(Type, OldWe, NewWe) ->
+    gb_sets:from_ordset(new_items_as_ordset(Type, OldWe, NewWe)).
+
+new_items_as_ordset(vertex, #we{next_id=Wid}, #we{next_id=NewWid,vp=Tab}) ->
+    new_items_as_ordset_1(Tab, Wid, NewWid);
+new_items_as_ordset(edge, #we{next_id=Wid}, #we{next_id=NewWid,es=Tab}) ->
+    new_items_as_ordset_1(Tab, Wid, NewWid);
+new_items_as_ordset(face, #we{next_id=Wid}, #we{next_id=NewWid,fs=Tab}) ->
+    new_items_as_ordset_1(Tab, Wid, NewWid).
+
+%%% Hiding/showing faces.
+
+hide_faces(Fs, We) when is_list(Fs) ->
+    hide_faces_1(gb_sets:from_list(Fs), We);
+hide_faces(Fs, We) ->
+    hide_faces_1(Fs, We).
+
+num_hidden(#we{fs=Ftab}=We) ->
+    case any_hidden(We) of
+	false -> 0;
+	true -> num_hidden_1(gb_trees:keys(Ftab), 0)
+    end.
+
+any_hidden(#we{fs=Ftab}) ->
+    not gb_trees:is_empty(Ftab) andalso
+	wings_util:gb_trees_smallest_key(Ftab) < 0.
+
+all_hidden(#we{fs=Ftab}) ->
+    not gb_trees:is_empty(Ftab) andalso
+	wings_util:gb_trees_largest_key(Ftab) < 0.
+
+%%%
+%%% Local functions.
+%%%
+
+rebuild_1(VctList, #we{vc=Vct,vp=Vtab0}=We) ->
+    case {gb_trees:size(Vct),gb_trees:size(Vtab0)} of
+	{Same,Same} -> rebuild(We);
+	{Sz1,Sz2} when Sz1 < Sz2 ->
+	    Vtab = vertex_gc_1(VctList, gb_trees:to_list(Vtab0), []),
+	    rebuild(We#we{vp=Vtab})
+    end.
+
 rebuild_vct(Es) ->
-    gb_trees:from_orddict(rebuild_vct(Es, [])).
+    rebuild_vct(Es, []).
 
 rebuild_vct([{Edge,#edge{vs=Va,ve=Vb}}|Es], Acc0) ->
     Acc = rebuild_maybe_add(Va, Vb, Edge, Acc0),
@@ -66,25 +153,15 @@ rebuild_ftab_1([], FtoE) ->
     gb_trees:from_orddict(build_incident_tab(FtoE)).
 
 rebuild_maybe_add(Ka, Kb, E, [_,{Ka,_}|_]=Acc) ->
-    rebuild_maybe_add_1(Kb, E, Acc);
+    [{Kb,E}|Acc];
+rebuild_maybe_add(Ka, Kb, E, [_,{Kb,_}|_]=Acc) ->
+    [{Ka,E}|Acc];
 rebuild_maybe_add(Ka, Kb, E, [{Ka,_}|_]=Acc) ->
-    rebuild_maybe_add_1(Kb, E, Acc);
+    [{Kb,E}|Acc];
+rebuild_maybe_add(Ka, Kb, E, [{Kb,_}|_]=Acc) ->
+    [{Ka,E}|Acc];
 rebuild_maybe_add(Ka, Kb, E, Acc) ->
-    [{Ka,E}|rebuild_maybe_add_1(Kb, E, Acc)].
-
-rebuild_maybe_add_1(K, _, [_,{K,_}|_]=Acc) -> Acc;
-rebuild_maybe_add_1(K, _, [{K,_}|_]=Acc) -> Acc;
-rebuild_maybe_add_1(K, E, Acc) -> [{K,E}|Acc].
-
-%% vertex_gc(We) -> We'
-%%  Remove vertices in the 'vc' and 'vp' tables that are no
-%%  longer referenced by any edge in the edge table.
-vertex_gc(#we{es=Etab,vp=Vtab0}=We) ->
-    Es = gb_trees:to_list(Etab),
-    Vct0 = rebuild_vct(Es, []),
-    Vtab = vertex_gc_1(Vct0, gb_trees:to_list(Vtab0), []),
-    Vct = gb_trees:from_orddict(Vct0),
-    We#we{vc=Vct,vp=Vtab}.
+    [{Ka,E},{Kb,E}|Acc].
 
 vertex_gc_1([{V,_}|Vct], [{V,_}=Vtx|Vpos], Acc) ->
     vertex_gc_1(Vct, Vpos, [Vtx|Acc]);
@@ -94,27 +171,21 @@ vertex_gc_1([], _, Acc) ->
     gb_trees:from_orddict(reverse(Acc)).
 
 %%%
-%%% Hide the faces given in the list.
+%%% Handling of hidden faces.
 %%%
 
-hide_faces(Fs, We) when is_list(Fs) ->
-    hide_faces_1(gb_sets:from_list(Fs), We);
-hide_faces(Fs, We) ->
-    hide_faces_1(Fs, We).
-
 hide_faces_1(Fs, #we{es=Etab0}=We0) ->
-    Etab1 = gb_trees:to_list(Etab0),
-    Etab2 = foldl(fun({E,#edge{lf=Lf0,rf=Rf0}=R0}, A) ->
-			  Lf = hide_map_face(Lf0, Fs),
-			  Rf = hide_map_face(Rf0, Fs),
-			  case R0#edge{lf=Lf,rf=Rf} of
-			      R0 -> [{E,R0}|A];
-			      R -> [{E,R}|A]
-			  end
-		  end, [], Etab1),
-    Etab = gb_trees:from_orddict(reverse(Etab2)),
+    Map = fun(_, #edge{lf=Lf0,rf=Rf0}=R0) ->
+		  Lf = hide_map_face(Lf0, Fs),
+		  Rf = hide_map_face(Rf0, Fs),
+		  case R0#edge{lf=Lf,rf=Rf} of
+		      R0 -> R0;
+		      R -> R
+		  end
+	  end,
+    Etab = wings_util:gb_trees_map(Map, Etab0),
     We = We0#we{es=Etab,fs=undefined},
-    rebuild(We).
+    wings_facemat:hide_faces(rebuild(We)).
 
 hide_map_face(F, Fs) ->
     case gb_sets:is_member(F, Fs) of
@@ -122,15 +193,23 @@ hide_map_face(F, Fs) ->
 	true -> -F-1
     end.
 
-any_hidden(#we{fs=Ftab}) ->
-    not gb_trees:is_empty(Ftab) andalso
-	wings_util:gb_trees_smallest_key(Ftab) < 0.
+num_hidden_1([F|Fs], N) when F < 0 ->
+    num_hidden_1(Fs, N+1);
+num_hidden_1(_, N) -> N.
+	    
 
-visible(#we{fs=Ftab}) ->
-    visible_2(gb_trees:keys(Ftab)).
+visible(#we{mirror=none,fs=Ftab}) ->
+    visible_2(gb_trees:keys(Ftab));
+visible(#we{mirror=Face,fs=Ftab}) ->
+    visible_2(gb_trees:keys(gb_trees:delete(Face, Ftab))).
 
-visible([{_,_}|_]=Fs, _) -> visible_1(Fs);
-visible(Fs, _) -> visible_2(Fs).
+visible([{_,_}|_]=Fs, #we{mirror=none}) ->
+    visible_1(Fs);
+visible([{_,_}|_]=Fs0, #we{mirror=Face}) ->
+    Fs = lists:keydelete(Face, 1, Fs0),
+    visible_1(Fs);
+visible(Fs, #we{mirror=none}) -> visible_2(Fs);
+visible(Fs, #we{mirror=Face}) -> visible_2(Fs--[Face]).
 
 visible_1([{F,_}|Fs]) when F < 0 -> visible_1(Fs);
 visible_1(Fs) -> Fs.
@@ -138,6 +217,83 @@ visible_1(Fs) -> Fs.
 visible_2([F|Fs]) when F < 0 -> visible_2(Fs);
 visible_2(Fs) -> Fs.
 
+visible_vs(#we{mirror=Face,vc=Vct,es=Etab}=We) ->
+    case any_hidden(We) of
+	false -> gb_trees:keys(Vct);
+	true -> visible_vs_1(gb_trees:values(Etab), Face, [])
+    end.
+
+visible_vs_1([#edge{lf=Mirror,rf=Rf}|Es], Mirror, Acc) when Rf < 0 ->
+    visible_vs_1(Es, Mirror, Acc);
+visible_vs_1([#edge{rf=Mirror,lf=Lf}|Es], Mirror, Acc) when Lf < 0 ->
+    visible_vs_1(Es, Mirror, Acc);
+visible_vs_1([#edge{lf=Lf,rf=Rf}|Es], Mirror, Acc) when Lf < 0, Rf < 0 ->
+    visible_vs_1(Es, Mirror, Acc);
+visible_vs_1([#edge{vs=Va,ve=Vb}|Es], Mirror, Acc) ->
+    visible_vs_1(Es, Mirror, [Va,Vb|Acc]);
+visible_vs_1([], _, Acc) -> ordsets:from_list(Acc).
+
+visible_vs(Vs, #we{mirror=Face,es=Etab}=We) ->
+    case any_hidden(We) of
+	false -> Vs;
+	true ->
+	    Vis0 = visible_vs_1(gb_trees:values(Etab), Face, []),
+	    case Vs of
+		[{_,_}|_] ->
+		    VsSet = sofs:relation(Vs),
+		    VisSet = sofs:from_external(Vis0, [atom]),
+		    sofs:to_external(sofs:restriction(VsSet, VisSet));
+		[_|_] ->
+		    ordsets:intersection(Vis0, Vs);
+		[] ->
+		    [];
+		true ->
+		    Vis = gb_sets:from_ordset(Vis0),
+		    gb_sets:intersection(Vis, Vs)
+	    end
+    end.
+
+visible_edges(#we{es=Etab,mirror=Face}=We) ->
+    case any_hidden(We) of
+	false -> gb_trees:keys(Etab);
+	true -> visible_es_1(gb_trees:to_list(Etab), Face, [])
+    end.
+
+visible_es_1([{E,#edge{lf=Lf,rf=Rf}}|Es], Face, Acc) ->
+    if
+	Lf < 0 ->
+	    %% Left face hidden.
+	    if
+		Rf < 0; Rf =:= Face ->
+		    %% Both faces invisible (in some way).
+		    visible_es_1(Es, Face, Acc);
+		true ->
+		    %% Right face is visible.
+		    visible_es_1(Es, Face, [E|Acc])
+	    end;
+	Lf =:= Face, Rf < 0 ->
+	    %% Left face mirror, right face hidden.
+	    visible_es_1(Es, Face, Acc);
+	true ->
+	    %% At least one face visible.
+	    visible_es_1(Es, Face, [E|Acc])
+    end;
+visible_es_1([], _, Acc) -> ordsets:from_list(Acc).
+
+visible_edges(Es, We) ->
+    case any_hidden(We) of
+	false -> Es;
+	true ->
+	    Vis0 = visible_edges(We),
+	    if
+		is_list(Es) ->
+		    ordsets:intersection(Vis0, Es);
+		true ->
+		    Vis = gb_sets:from_ordset(Vis0),
+		    gb_sets:intersection(Vis, Es)
+	    end
+    end.
+
 show_faces(We) ->
     case any_hidden(We) of
 	false -> We;
@@ -145,32 +301,45 @@ show_faces(We) ->
     end.
 
 show_faces_1(#we{es=Etab0}=We0) ->
-    Etab1 = gb_trees:to_list(Etab0),
-    Etab2 = foldl(fun({E,#edge{lf=Lf0,rf=Rf0}=R0}, A) ->
-			  Lf = show_face(Lf0),
-			  Rf = show_face(Rf0),
-			  case R0#edge{lf=Lf,rf=Rf} of
-			      R0 -> [{E,R0}|A];
-			      R -> [{E,R}|A]
-			  end
-		  end, [], Etab1),
-    Etab = gb_trees:from_orddict(reverse(Etab2)),
+    Map = fun(_, #edge{lf=Lf0,rf=Rf0}=R) when Lf0 < 0; Rf0 < 0 ->
+		  Lf = show_face(Lf0),
+		  Rf = show_face(Rf0),
+		  R#edge{lf=Lf,rf=Rf};
+	     (_, R) -> R
+	  end,
+    Etab = wings_util:gb_trees_map(Map, Etab0),
     We = We0#we{es=Etab,fs=undefined},
-    rebuild(We).
+    wings_facemat:show_faces(rebuild(We)).
 
 show_face(F) when F < 0 -> -F-1;
 show_face(F) -> F.
+
+validate_mirror(#we{mirror=none}=We) -> We;
+validate_mirror(#we{fs=Ftab,mirror=Face}=We) ->
+    case gb_trees:is_defined(Face, Ftab) of
+	false -> We#we{mirror=none};
+	true -> We
+    end.
+
+mirror_flatten(_, #we{mirror=none}=We) -> We;
+mirror_flatten(#we{mirror=OldFace}=OldWe, #we{mirror=Face,vp=Vtab0}=We) ->
+    PlaneNormal = wings_face:normal(OldFace, OldWe),
+    FaceVs = wings_face:to_vertices(gb_sets:singleton(OldFace), OldWe),
+    Origin = wings_vertex:center(FaceVs, OldWe),
+    M0 = e3d_mat:translate(Origin),
+    M = e3d_mat:mul(M0, e3d_mat:project_to_plane(PlaneNormal)),
+    Flatten = e3d_mat:mul(M, e3d_mat:translate(e3d_vec:neg(Origin))),
+    Vtab = foldl(fun(V, Vt) ->
+			 Pos0 = gb_trees:get(V, Vt),
+			 Pos = e3d_mat:mul_point(Flatten, Pos0),
+			 gb_trees:update(V, Pos, Vt)
+		 end, Vtab0, wings_face:vertices_ccw(Face, We)),
+    We#we{vp=Vtab}.
     
 %%%
 %%% Build Winged-Edges.
 %%%
 
-build(Mode, #e3d_mesh{fs=Fs0,vs=Vs,tx=Tx,he=He}) when is_atom(Mode) ->
-    Fs = translate_faces(Fs0, list_to_tuple(Tx), []),
-    build(Mode, Fs, Vs, He);
-build(Fs, Vs) ->
-    build(material, Fs, Vs, []).
-
 translate_faces([#e3d_face{vs=Vs,tx=Tx0,mat=Mat0}|Fs], Txs, Acc) ->
     Mat = translate_mat(Mat0),
     FaceData = case Tx0 of
@@ -214,13 +383,13 @@ build_rest(Type, Es, Fs, Vs, HardEdges) ->
 assign_materials([L|_], We) when is_list(L) -> We;
 assign_materials(Fs, We) ->
     MatFace = mat_face(Fs),
-    wings_material:assign_materials(MatFace, We).
+    wings_facemat:assign(MatFace, We).
 
 mat_face(Fs) ->
     mat_face(Fs, 0, []).
 
 mat_face([T|Ts], Face, Acc) ->
-    mat_face(Ts, Face+1, [{element(1, T),Face}|Acc]);
+    mat_face(Ts, Face+1, [{Face,element(1, T)}|Acc]);
 mat_face([], _, Acc) -> Acc.
 
 number_vertices([P|Ps], V, Acc) ->
@@ -375,24 +544,6 @@ make_digraph([{{Vb,Va},[{left,_Data}]}|Es], G) ->
     make_digraph(Es, G);
 make_digraph([], _G) -> ok.
 
-%%% Utilities for allocating IDs.
-
-new_wrap_range(Items, Inc, #we{next_id=Id}=We) ->
-    NumIds = Items*Inc,
-    {{0,Id,Inc,NumIds},We#we{next_id=Id+NumIds}}.
-
-id(N, {Current,BaseId,_Inc,NumIds}) ->
-    BaseId + ((Current+N) rem NumIds).
-
-bump_id({Id,BaseId,Inc,NumIds}) ->
-    {Id+Inc,BaseId,Inc,NumIds}.
-
-new_id(#we{next_id=Id}=We) ->
-    {Id,We#we{next_id=Id+1}}.
-
-new_ids(N, #we{next_id=Id}=We) ->
-    {Id,We#we{next_id=Id+N}}.
-
 %%% Invert all normals.
 
 invert_normals(#we{es=Etab0}=We0) ->
@@ -444,7 +595,8 @@ merge([]) -> [];
 merge([We]) -> We;
 merge([#we{id=Id,name=Name}|_]=Wes0) ->
     Wes1 = merge_renumber(Wes0),
-    {Vpt0,Et0,Ht0,MatTab} = merge_1(Wes1),
+    MatTab = wings_facemat:merge(Wes1),
+    {Vpt0,Et0,Ht0} = merge_1(Wes1),
     Vpt = gb_trees:from_orddict(Vpt0),
     Et = gb_trees:from_orddict(Et0),
     Ht = gb_sets:from_ordset(Ht0),
@@ -452,20 +604,18 @@ merge([#we{id=Id,name=Name}|_]=Wes0) ->
 		vp=Vpt,es=Et,he=Ht,mat=MatTab}).
 
 merge_1([We]) -> We;
-merge_1(Wes) -> merge_1(Wes, [], [], [], []).
+merge_1(Wes) -> merge_1(Wes, [], [], []).
 
-merge_1([#we{vp=Vp0,es=Es,he=He,mat=Mat}=We|Wes], Vpt0, Et0, Ht0, Mt0) ->
+merge_1([#we{vp=Vp0,es=Es,he=He}|Wes], Vpt0, Et0, Ht0) ->
     Vpt = [gb_trees:to_list(Vp0)|Vpt0],
     Et = [gb_trees:to_list(Es)|Et0],
     Ht = [gb_sets:to_list(He)|Ht0],
-    Mt = [{Mat,We}|Mt0],
-    merge_1(Wes, Vpt, Et, Ht, Mt);
-merge_1([], Vpt0, Et0, Ht0, Mt0) ->
+    merge_1(Wes, Vpt, Et, Ht);
+merge_1([], Vpt0, Et0, Ht0) ->
     Vpt = lists:merge(Vpt0),
     Et = lists:merge(Et0),
     Ht = lists:merge(Ht0),
-    Mt = wings_material:merge(Mt0),
-    {Vpt,Et,Ht,Mt}.
+    {Vpt,Et,Ht}.
 			       
 merge_renumber(Wes0) ->
     [{_,We}|Wes] = merge_bounds(Wes0, []),
@@ -518,7 +668,7 @@ do_renumber(#we{mode=Mode,vp=Vtab0,es=Etab0,fs=Ftab0,
     Vtab = renumber_vertices(Vtab1, Vmap),
 
     Fmap = make_map(gb_trees:to_list(Ftab0), Id),
-    MatTab = wings_material:renumber(MatTab0, Fmap),
+    MatTab = wings_facemat:renumber(MatTab0, Fmap),
 
     Etab1 = gb_trees:to_list(Etab0),
     Emap = make_map(Etab1, Id),
@@ -673,20 +823,31 @@ copy_dependents(We0) ->
     Vtab1 = sofs:relation(gb_trees:to_list(Vtab0), [{vertex,edge}]),
     Vtab2 = sofs:restriction(Vtab1, Vs),
     Vtab = gb_trees:from_orddict(sofs:to_external(Vtab2)),
-    wings_material:cleanup(We#we{he=Htab,vp=Vtab}).
+    wings_facemat:gc(We#we{he=Htab,vp=Vtab}).
 
-%% build_incident_tab([{Elem,Edge}]) -> GbTree([{Elem,Edge1}])
+%% build_incident_tab([{Elem,Edge}]) -> [{Elem,Edge}]
 %%      Elem = Face or Vertex
 %%  Build the table of incident edges for either faces or vertices.
+%%  Returns an ordered list where each Elem is unique.
 
 build_incident_tab(ElemToEdgeRel) ->
-    build_incident_tab(keysort(1, ElemToEdgeRel), []).
+    T = ets:new(?MODULE, [ordered_set]),
+    ets:insert(T, ElemToEdgeRel),
+    R = ets:tab2list(T),
+    ets:delete(T),
+    R.
+
+%% Reference implementation below. On my Mac is twice as slow as the
+%% implementation above (lists:keysort/2 takes most of the time).
+
+%% build_incident_tab(ElemToEdgeRel) ->
+%%     build_incident_tab_1(keysort(1, ElemToEdgeRel), []).
     
-build_incident_tab([{V,_}|VsEs], [{V,_}|_]=Acc) ->
-    build_incident_tab(VsEs, Acc);
-build_incident_tab([{V,Edge}|VsEs], Acc) ->
-    build_incident_tab(VsEs, [{V,Edge}|Acc]);
-build_incident_tab([], Acc) -> reverse(Acc).
+%% build_incident_tab_1([{V,_}|VsEs], [{V,_}|_]=Acc) ->
+%%     build_incident_tab_1(VsEs, Acc);
+%% build_incident_tab_1([Pair|VsEs], Acc) ->
+%%     build_incident_tab_1(VsEs, [Pair|Acc]);
+%% build_incident_tab_1([], Acc) -> reverse(Acc).
 
 %%%
 %%% Convert textures to vertex colors.
@@ -740,8 +901,26 @@ transform_vs_1(Transform, #we{vp=Vtab0}=We) ->
 %%% Calculate normals.
 %%%
 
-normals(FaceNormals, #we{he=He}=We) ->
-    wings_pb:start(?STR(normals,1,"calculating soft normals")),
+normals(Ns, #we{mirror=none}=We) ->
+    case any_hidden(We) of
+	false -> normals_2(Ns, We);
+	true -> normals_1(Ns, We)
+    end;
+normals(Ns, We) -> normals_1(Ns, We).
+
+normals_1(FaceNormals, #we{fs=Ftab,he=Htab0}=We) ->
+    Edges = case {visible(We),gb_trees:size(Ftab)} of
+		{Vis,Sz} when 2*length(Vis) < Sz ->
+		    wings_face:outer_edges(Vis, We);
+		{Vis,_} ->
+		    InVis = ordsets:subtract(gb_trees:keys(Ftab), Vis),
+		    wings_face:outer_edges(InVis, We)
+	    end,
+    Htab = gb_sets:union(Htab0, gb_sets:from_ordset(Edges)),
+    normals_2(FaceNormals, We#we{he=Htab}).
+
+normals_2(FaceNormals, #we{he=He}=We) ->
+    wings_pb:start(?__(1,"calculating soft normals")),
     Res = case FaceNormals of
 	      [_,_] ->
 		  two_faced(FaceNormals, We);
@@ -754,13 +933,14 @@ normals(FaceNormals, #we{he=He}=We) ->
     wings_pb:done(Res).
 
 all_soft(FaceNormals, #we{vp=Vtab}=We) ->
-    wings_pb:update(0.10, ?STR(all_soft,1,"preparing")),
-    VtxNormals = soft_vertex_normals(gb_trees:to_list(Vtab), FaceNormals, We),
+    wings_pb:update(0.10, ?__(1,"preparing")),
+    VisVs = visible_vs(gb_trees:to_list(Vtab), We),
+    VtxNormals = soft_vertex_normals(VisVs, FaceNormals, We),
     FoldFun = fun(V, VInfo, A) ->
 		      Normal = gb_trees:get(V, VtxNormals),
 		      [[VInfo|Normal]|A]
 	      end,
-    wings_pb:update(0.6, ?STR(all_soft,2,"collecting")),
+    wings_pb:update(0.6, ?__(2,"collecting")),
     all_soft_1(FoldFun, FaceNormals, We, []).
 
 all_soft_1(FoldFun, [{Face,_}|FNs], We, Acc) ->
@@ -768,27 +948,13 @@ all_soft_1(FoldFun, [{Face,_}|FNs], We, Acc) ->
     all_soft_1(FoldFun, FNs, We, [{Face,Vs}|Acc]);
 all_soft_1(_, [], _, Acc) -> reverse(Acc).
 
-soft_vertex_normals(Vtab, FaceNormals0, We) when is_list(FaceNormals0) ->
-    FaceNormals = gb_trees:from_orddict(FaceNormals0),
-    soft_vertex_normals(Vtab, FaceNormals, We);
-soft_vertex_normals(Vtab, FaceNormals, We) ->
-    FoldFun = fun(_, Face, _, A) ->
-		      [gb_trees:get(Face, FaceNormals)|A]
-	      end,
-    Soft = foldl(fun({V,_}, Acc) ->
-			 Ns = wings_vertex:fold(FoldFun, [], V, We),
-			 N = e3d_vec:norm(e3d_vec:add(Ns)),
-			 [{V,N}|Acc]
-		 end, [], Vtab),
-    gb_trees:from_orddict(reverse(Soft)).
-
 mixed_edges(FaceNormals0, We) ->
-    wings_pb:update(0.20, ?STR(mixed_edges,1,"preparing")),
+    wings_pb:update(0.20, ?__(1,"preparing")),
     G = digraph:new(),
     FaceNormals = gb_trees:from_orddict(FaceNormals0),
-    wings_pb:update(0.50,  ?STR(mixed_edges,2,"vertex normals")),
+    wings_pb:update(0.50,  ?__(2,"vertex normals")),
     VtxNormals = vertex_normals(We, G, FaceNormals),
-    wings_pb:update(0.99,  ?STR(mixed_edges,3,"vertex normals per face")),
+    wings_pb:update(0.99,  ?__(3,"vertex normals per face")),
     Ns = foldl(fun({Face,_}, Acc) ->
 		       Vs = n_face(Face, G, FaceNormals, VtxNormals, We),
 		       [{Face,Vs}|Acc]
@@ -824,7 +990,12 @@ two_faced_1(Face, Normal, We) ->
 				  [[VInfo|Normal]|Acc]
 			  end, [], Face, We).
 
-vertex_normals(#we{vp=Vtab,es=Etab,he=Htab}=We, G, FaceNormals) ->
+vertex_normals(#we{vp=Vtab}=We, G, FaceNormals) ->
+    Vs0 = visible_vs(gb_trees:to_list(Vtab), We),
+    Vs = sofs:from_external(Vs0, [{vertex,data}]),
+    vertex_normals_1(Vs, We, G, FaceNormals).
+
+vertex_normals_1(Vs, #we{es=Etab,he=Htab}=We, G, FaceNormals) ->
     He0 = gb_sets:to_list(Htab),
     He = sofs:from_external(He0, [edge]),
     Es0 = gb_trees:to_list(Etab),
@@ -834,7 +1005,6 @@ vertex_normals(#we{vp=Vtab,es=Etab,he=Htab}=We, G, FaceNormals) ->
 			 [Va,Vb|A]
 		 end, [], sofs:to_external(Es)),
     Hvs = sofs:set(Hvs0, [vertex]),
-    Vs = sofs:from_external(gb_trees:to_list(Vtab), [{vertex,data}]),
     Svs = sofs:drestriction(Vs, Hvs),
     SoftVs = sofs:to_external(Svs),
     HardVs = sofs:to_external(Hvs),
@@ -856,109 +1026,93 @@ update_digraph(G, V, #we{he=Htab}=We) ->
 	      end
       end, [], V, We).
 
-%%%
-%%% Returns sets of newly created items.
-%%%
+soft_vertex_normals(Vtab, FaceNormals0, We) when is_list(FaceNormals0) ->
+    FaceNormals = gb_trees:from_orddict(FaceNormals0),
+    soft_vertex_normals(Vtab, FaceNormals, We);
+soft_vertex_normals(Vtab, FaceNormals, We) ->
+    FoldFun = fun(_, Face, _, A) ->
+		      [gb_trees:get(Face, FaceNormals)|A]
+	      end,
+    Soft = foldl(fun({V,_}, Acc) ->
+			 Ns = wings_vertex:fold(FoldFun, [], V, We),
+			 N = e3d_vec:norm(e3d_vec:add(Ns)),
+			 [{V,N}|Acc]
+		 end, [], Vtab),
+    gb_trees:from_orddict(reverse(Soft)).
 
-%% new_items(vertex|edge|face, OldWe, NewWe) -> NewItemsGbSet.
-%%  Return all items in NewWe that are not in OldWe (as a GbSet).
 
-new_items(vertex, #we{next_id=Wid}, #we{next_id=NewWid,vp=Tab}) ->
-    new_items_1(Tab, Wid, NewWid);
-new_items(edge, #we{next_id=Wid}, #we{next_id=NewWid,es=Tab}) ->
-    new_items_1(Tab, Wid, NewWid);
-new_items(face, #we{next_id=Wid}, #we{next_id=NewWid,fs=Tab}) ->
-    new_items_1(Tab, Wid, NewWid).
+new_items_as_ordset_1(Tab, Wid, NewWid) when NewWid-Wid < 32 ->
+    new_items_as_ordset_2(Wid, NewWid, Tab, []);
+new_items_as_ordset_1(Tab, Wid, _NewWid) ->
+    [Item || Item <- gb_trees:keys(Tab), Item >= Wid].
 
-new_items_1(Tab, Wid, NewWid) when NewWid-Wid < 32 ->
-    new_items_2(Wid, NewWid, Tab, []);
-new_items_1(Tab, Wid, _NewWid) ->
-    Items = [Item || Item <- gb_trees:keys(Tab), Item >= Wid],
-    gb_sets:from_ordset(Items).
-
-new_items_2(Wid, NewWid, Tab, Acc) when Wid < NewWid ->
+new_items_as_ordset_2(Wid, NewWid, Tab, Acc) when Wid < NewWid ->
     case gb_trees:is_defined(Wid, Tab) of
-	true -> new_items_2(Wid+1, NewWid, Tab, [Wid|Acc]);
-	false -> new_items_2(Wid+1, NewWid, Tab, Acc)
+	true -> new_items_as_ordset_2(Wid+1, NewWid, Tab, [Wid|Acc]);
+	false -> new_items_as_ordset_2(Wid+1, NewWid, Tab, Acc)
     end;
-new_items_2(_Wid, _NewWid, _Tab, Acc) ->
-    gb_sets:from_ordset(reverse(Acc)).
+new_items_as_ordset_2(_Wid, _NewWid, _Tab, Acc) -> reverse(Acc).
 
 %%%
-%%% Test if a winged-edge record is consistent.
+%%% Test the consistency of a #we{}.
 %%%
 
 is_consistent(#we{}=We) ->
-    case catch validate_we(We) of
-	{'EXIT',Reason} ->
-	    io:format("~P\n", [Reason,30]),
-	    false;
-	true -> true
+    try
+	validate_vertex_tab(We),
+	validate_faces(We)
+    catch error:_ -> false
     end.
 
-is_face_consistent(Face, #we{fs=Ftab}=We) ->
+is_face_consistent(Face, #we{fs=Ftab,es=Etab}) ->
     Edge = gb_trees:get(Face, Ftab),
-    case catch validate_face(Face, Edge, We) of
-	true -> true;
-	_ -> false
+    try validate_face(Face, Edge, Etab)
+    catch error:_ -> false
     end.
-
-validate_we(We) ->
-    validate_vertex_tab(We),
-    validate_faces(We),
-    true.
     
-validate_faces(#we{fs=Ftab}=We) ->
-    foreach(fun({Face,Edge}) ->
-		    validate_face(Face, Edge, We)
-	    end, gb_trees:to_list(Ftab)).
+validate_faces(#we{fs=Ftab,es=Etab}) ->
+    validate_faces_1(gb_trees:to_list(Ftab), Etab).
 
-validate_face(Face, Edge, We) ->
-    Cw = walk_face_cw(Face, Edge, Edge, We, []),
-    Ccw = walk_face_ccw(Face, Edge, Edge, We, []),
-    case reverse(Ccw) of
-	Cw -> validate_face_vertices(Cw);
-	_Other -> exit({face_cw_ccw_inconsistency,Face})
-    end.
+validate_faces_1([{Face,Edge}|Fs], Etab) ->
+    validate_face(Face, Edge, Etab),
+    validate_faces_1(Fs, Etab);
+validate_faces_1([], _) -> true.
 
-validate_face_vertices(Vs) ->
-    validate_face_vertices_1(sort(Vs)).
+validate_face(Face, Edge, Etab) ->
+    Ccw = walk_face_ccw(Edge, Etab, Face, Edge, []),
+    Edge = walk_face_cw(Edge, Etab, Face, Ccw),
+    [V|Vs] = sort(Ccw),
+    validate_face_vertices(Vs, V).
 
-validate_face_vertices_1([V,V|_]) ->
-    exit(repeated_vertex);
-validate_face_vertices_1([_|T]) ->
-    validate_face_vertices_1(T);
-validate_face_vertices_1([]) -> true.
+validate_face_vertices([V|_], V) ->
+    erlang:error(repeated_vertex);
+validate_face_vertices([_], _) ->
+    true;
+validate_face_vertices([V|Vs], _) ->
+    validate_face_vertices(Vs, V).
 
-walk_face_cw(_Face, LastEdge, LastEdge, _We, [_|_]=Acc) -> Acc;
-walk_face_cw(Face, Edge, LastEdge, We, Acc) ->
-    #we{es=Etab} = We,
-    case gb_trees:get(Edge, Etab) of
-	#edge{vs=V,lf=Face,ltsu=Next} ->
-	    walk_face_cw(Face, Next, LastEdge, We, [V|Acc]);
-	#edge{ve=V,rf=Face,rtsu=Next} ->
-	    walk_face_cw(Face, Next, LastEdge, We, [V|Acc])
-    end.
-
-walk_face_ccw(_Face, LastEdge, LastEdge, _We, [_|_]=Acc) -> Acc;
-walk_face_ccw(Face, Edge, LastEdge, We, Acc) ->
-    #we{es=Etab} = We,
+walk_face_ccw(LastEdge, _, _, LastEdge, [_|_]=Acc) -> Acc;
+walk_face_ccw(Edge, Etab, Face, LastEdge, Acc) ->
     case gb_trees:get(Edge, Etab) of
 	#edge{ve=V,lf=Face,ltpr=Next} ->
-	    walk_face_ccw(Face, Next, LastEdge, We, [V|Acc]);
+	    walk_face_ccw(Next, Etab, Face, LastEdge, [V|Acc]);
 	#edge{vs=V,rf=Face,rtpr=Next} ->
-	    walk_face_ccw(Face, Next, LastEdge, We, [V|Acc])
+	    walk_face_ccw(Next, Etab, Face, LastEdge, [V|Acc])
+    end.
+
+walk_face_cw(Edge, _, _, []) -> Edge;
+walk_face_cw(Edge, Etab, Face, [V|Vs]) ->
+    case gb_trees:get(Edge, Etab) of
+	#edge{vs=V,lf=Face,ltsu=Next} ->
+	    walk_face_cw(Next, Etab, Face, Vs);
+	#edge{ve=V,rf=Face,rtsu=Next} ->
+	    walk_face_cw(Next, Etab, Face, Vs)
     end.
 
 validate_vertex_tab(#we{es=Etab,vc=Vct}) ->
     foreach(fun({V,Edge}) ->
 		    case gb_trees:get(Edge, Etab) of
-			#edge{vs=V}=Rec ->
-			    validate_edge_rec(Rec);
-			#edge{ve=V}=Rec ->
-			    validate_edge_rec(Rec)
+			#edge{vs=V} -> ok;
+			#edge{ve=V} -> ok
 		    end
 	    end, gb_trees:to_list(Vct)).
-
-validate_edge_rec(#edge{ltpr=LP,ltsu=LS,rtpr=RP,rtsu=RS})
-  when is_integer(LP+LS+RP+RS) -> ok.
