@@ -8,11 +8,11 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm.erl,v 1.147 2004/11/14 13:47:27 bjorng Exp $
+%%     $Id: wings_wm.erl,v 1.151 2004/12/18 19:36:22 bjorng Exp $
 %%
 
 -module(wings_wm).
--export([toplevel/6,toplevel_title/2,set_knob/3]).
+-export([toplevel/6,toplevel_title/1,toplevel_title/2,set_knob/3]).
 -export([init/0,enter_event_loop/0,dirty/0,dirty_mode/1,pdirty/0,
 	 reinit_opengl/0,
 	 new/4,delete/1,raise/1,
@@ -715,17 +715,32 @@ translate_event({drop,{X,Y},DropData}, Ox, Oy) ->
     {drop,{X-Ox,Y-Oy},DropData};
 translate_event(Ev, _, _) -> Ev.
     
-handle_event(#se{h=Handler}, Event, Stk) ->
-    handle_response(catch Handler(Event), Event, Stk).
+handle_event(State, Event, Stk) ->
+    try
+	case State of
+	    #se{h=Handler} -> Handler(Event);
+	    Fun when is_function(Fun) -> Fun()
+	end of
+	Res ->
+	    handle_response(Res, Event, Stk)
+    catch
+	throw:{command_error,Error} ->
+	    wings_u:message(Error),
+	    Stk;
+	  exit:normal ->
+	    exit(normal);
+	  exit:Exit ->
+	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
+	    handle_response(CrashHandler({crash,Exit}), Event,
+			    default_stack(this()));
+	  error:Reason ->
+	    [#se{h=CrashHandler}] = pop_all_but_one(Stk),
+	    handle_response(CrashHandler({crash,Reason}), Event,
+			    default_stack(this()))
+    end.
 
 handle_response(Res, Event, Stk0) ->
     case Res of
-	{'EXIT',normal} ->
-	    exit(normal);
-	{'EXIT',Reason} ->
-	    [#se{h=CrashHandler}] = pop_all_but_one(Stk0),
-	    handle_response(CrashHandler({crash,Reason}), Event,
-			    default_stack(this()));
 	keep -> Stk0;
 	next -> next_handler(Event, Stk0);
 	pop -> pop(Stk0);
@@ -741,9 +756,9 @@ handle_response(Res, Event, Stk0) ->
 	    handle_response(Then, Event, Stk);
 	{replace,Top} when is_function(Top) ->
 	    replace_handler(Top, Stk0);
-	{replace,Top,Continue} when is_function(Top) ->
+	{replace,Top,Continue} when is_function(Top), is_function(Continue) ->
 	    Stk = replace_handler(Top, Stk0),
-	    handle_response(catch Continue(), Event, Stk);
+	    handle_event(Continue, dummy_event, Stk);
 	{pop_handler,_}=PopH -> replace_handler(PopH, Stk0);
 	Top when is_function(Top) -> replace_handler(Top, Stk0)
     end.
@@ -1052,7 +1067,7 @@ message_event(redraw) ->
 message_event({action,_}=Action) ->
     send(geom, Action);
 message_event(got_focus) ->
-    message(?STR(message_event,1,"This is the information line")),
+    message(?__(1,"This is the information line")),
     dirty();
 message_event(_) -> keep.
 
@@ -1111,6 +1126,11 @@ message_setup() ->
 toplevel(Name, Title, Pos, Size, Flags, Op) ->
     wings_wm_toplevel:toplevel(Name, Title, Pos, Size, Flags, Op).
 
+toplevel_title(Title) ->
+    Win = this(),
+    send({controller,Win}, {title,Title}),
+    ok.
+
 toplevel_title(Win, Title) ->
     send({controller,Win}, {title,Title}),
     ok.
