@@ -3,22 +3,26 @@
 %%
 %%     Wings Plugin API.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2005 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpa.erl,v 1.56 2004/10/31 09:12:11 bjorng Exp $
+%%     $Id: wpa.erl,v 1.70 2005/03/02 13:00:05 dgud Exp $
 %%
+%% Note: To keep the call graph clean, wpa MUST NOT be called
+%%       from the wings core modules.
+
 -module(wpa).
--export([ask/3,ask/4,dialog/3,dialog/4,error/1,
+-export([ask/3,ask/4,dialog/3,dialog/4,error/1,error/2,
 	 yes_no/2,yes_no/3,yes_no_cancel/3,
 	 bind_unicode/2,bind_virtual/3,
 	 import/2,import/3,import_filename/2,
 	 export/3,export_selected/3,
 	 export_filename/2,export_filename/3,
 	 save_images/3,
-	 dialog_template/2,import_matrix/1,export_matrix/1,
+	 dialog_template/2,dialog_template/3,
+	 import_matrix/1,export_matrix/1,
 	 send_command/1,
 	 pref_get/2,pref_get/3,pref_set/2,pref_set/3,
 	 pref_set_default/3,pref_delete/2,
@@ -30,7 +34,7 @@
 	 pick/3,
 	 vertices/1,vertex_pos/2,vertex_flatten/3,vertex_center/2,
 	 faces/1,face_vertices/2,face_outer_vertices/2,face_outer_edges/2,
-	 face_dissolve/2,
+	 face_dissolve/2,face_dissolve_complement/2,
 	 edge_loop_vertices/2,
 	 obj_name/1,obj_id/1,
 	 camera_info/1,lights/1,import_lights/2,
@@ -48,7 +52,7 @@
 -import(lists, [reverse/1,foldl/3,foreach/2]).
 
 format_error({crash,Term}) ->
-    lists:flatten(io_lib:format(?STR(format_error,1,"Internal error: ~P\n"), [Term,20])).
+    lists:flatten(io_lib:format(?__(1,"Internal error: ~P\n"), [Term,20])).
 
 %%%
 %%% ask/3,4 is simpler to use, but only supports a single list of fields.
@@ -69,16 +73,19 @@ dialog(Bool, Title, Qs, Fun) ->
 
 %% Show String in a dialog box.
 error(String) ->
-    wings_util:error(String).
+    wings_u:error(String).
+
+error(Format, Args) ->
+    wings_u:error(Format, Args).
 
 yes_no(Question, Yes) ->
-    wings_util:yes_no(Question, Yes).
+    wings_u:yes_no(Question, Yes).
 
 yes_no(Question, Yes, No) ->
-    wings_util:yes_no(Question, Yes, No).
+    wings_u:yes_no(Question, Yes, No).
 
 yes_no_cancel(Question, Yes, No) ->
-    wings_util:yes_no_cancel(Question, Yes, No).
+    wings_u:yes_no_cancel(Question, Yes, No).
 
 bind_unicode(Key, Command) ->
     wings_hotkey:bind_unicode(Key, Command, plugin).
@@ -100,46 +107,41 @@ import(Props, Importer, St0) ->
 		   case ?SLOW(do_import(Importer, Name, St0)) of
 		       #st{}=St -> St;
 		       {error,Reason} ->
-			   error(?STR(import,1,"Import failed: ") ++ Reason)
+			   error(?__(1,"Import failed: ") ++ Reason)
 		   end
 	   end,
     import_filename(Props, Cont).
 
 do_import(Importer, Name, St0) ->
-    wings_pb:start(?STR(import_filename,1,"reading file")),
+    wings_pb:start(?__(1,"reading file")),
     wings_pb:update(1.0),
     case wings_pb:done(Importer(Name)) of
 	{ok,#e3d_file{}=E3DFile} ->
 	    wings_import:import(E3DFile, St0);
 	{error,Reason} ->
-	    wings_util:error(Reason)
+	    wings_u:error(Reason)
     end.
 
 %% import_filename([Prop], Continuation).
 %%   The Continuation fun will be called like this: Continuation(Filename).
-import_filename(Ps0, Cont) ->
-    This = wings_wm:this(),
-    Dir = wings_pref:get_value(current_directory),
-    Ps = Ps0 ++ [{title,?STR(import_filename,2,"Import")},{directory,Dir}],
-    Fun = fun(Name) ->
-		  case catch Cont(Name) of
-		      {command_error,Error} ->
-			  wings_util:message(Error);
-		      #st{}=St ->
-			  wings_wm:send(This, {new_state,St});
-		      Tuple when is_tuple(Tuple) ->
-			  wings_wm:send(This, {action,Tuple});
-		      ignore -> keep;
-		      keep -> keep
-		  end
-	  end,
-    wings_plugin:call_ui({file,open_dialog,Ps,Fun}).
+import_filename(Ps, Cont) ->
+    wings_file:import_filename(Ps, Cont).
+
+%% export([Property], ExporterFun, St)
+%%  
+%%  Recognized values for Property:
+%%       {subdivisions,Levels}
+%%       {tesselation,none|triangulation|quadrangulation}
+%%       {include_colors,Bool}
+%%       {include_uvs,Bool}
+%%       {include_hard_edges,Bool}
 
 export(none, Exporter, St) ->
-    wings_export:export(Exporter, none, 0, St);
+    wings_export:export(Exporter, none, [], St);
 export(Ps, Exporter, St) ->
-    SubDivs = proplists:get_value(subdivisions, Ps, 0),
-    Cont = fun(Name) -> wings_export:export(Exporter, Name, SubDivs, St) end,
+    Cont = fun(Name) -> 
+		   wings_export:export(Exporter, Name, Ps, St) 
+	   end,
     export_filename(Ps, St, Cont).
 
 export_selected(Props, Exporter, #st{selmode=Mode}=St)
@@ -150,49 +152,25 @@ export_selected(Props, Exporter, #st{selmode=Mode}=St)
 	     end, [], St),
     Shs = gb_trees:from_orddict(reverse(Shs0)),
     export(Props, Exporter, St#st{shapes=Shs});
-export_selected(_, _, _) -> error(?STR(export_selected,1,"Select objects or faces.")).
+export_selected(_, _, _) -> error(?__(1,"Select objects or faces.")).
 
 export_sel_set_holes(body, _, We) -> We;
 export_sel_set_holes(face, Faces0, #we{fs=Ftab}=We) ->
     Faces1 = gb_sets:to_list(Faces0),
     AllFaces = gb_trees:keys(Ftab),
     Faces = ordsets:subtract(AllFaces, Faces1),
-    wings_material:assign('_hole_', Faces, We).
+    wings_facemat:assign('_hole_', Faces, We).
 
 %% export_filename([Prop], Continuation).
 %%   The Continuation fun will be called like this: Continuation(Filename).
-export_filename(Prop0, Cont) ->
-    This = wings_wm:this(),
-    Dir = wings_pref:get_value(current_directory),
-    Prop = Prop0 ++ [{directory,Dir}],
-    Fun = fun(Name) ->
-		  case catch Cont(Name) of
-		      {command_error,Error} ->
-			  wings_util:message(Error);
-		      #st{}=St ->
-			  wings_wm:send(This, {new_state,St});
-		      Tuple when is_tuple(Tuple) ->
-			  wings_wm:send(This, {action,Tuple});
-		      ignore -> keep;
-		      keep -> keep;
-		      ok -> keep
-		  end
-	  end,
-	wings_plugin:call_ui({file,save_dialog,Prop++[{title,?STR(export_filename,1,"Export")}],Fun}).
+export_filename(Prop, Cont) ->
+    wings_file:export_filename(Prop, Cont).
 
 %% export_filename([Prop], St, Continuation).
 %%   The St will only be used to setup the default filename.
 %%   The Continuation fun will be called like this: Continuation(Filename).
-export_filename(Prop, #st{file=undefined}, Cont) ->
-    export_filename(Prop, Cont);
-export_filename(Prop0, #st{file=File}, Cont) ->
-    Prop = case proplists:get_value(ext, Prop0) of
-	       undefined -> Prop0;
-	       Ext ->
-		   Def = filename:rootname(filename:basename(File), ".wings") ++ Ext,
-		   [{default_filename,Def}|Prop0]
-	   end,
-    export_filename(Prop, Cont).
+export_filename(Prop, St, Cont) ->
+    wings_file:export_filename(Prop, St, Cont).
 
 %% save_images(E3DFile0, Directory, DefaultFiletype) -> E3DFile
 %%  Save all images in all materials, inserting the filename
@@ -208,7 +186,7 @@ save_images(E3DFile, Directory, Filetype) ->
 %%  Type = import|export
 dialog_template(Mod, import) ->
     {vframe,
-     [{?STR(dialog_template,1,"Swap Y and Z Axes"),pref_get(Mod, swap_y_z, false),
+     [{?__(1,"Swap Y and Z Axes"),pref_get(Mod, swap_y_z, false),
        [{key,swap_y_z}]},
       {label_column,
        [{import_scale_s(),
@@ -223,7 +201,7 @@ dialog_template(Mod, export) ->
 		    {Key,Val} <- image_formats()],
     DefFileType = pref_get(Mod, default_filetype, ".bmp"),
     {vframe,
-     [{?STR(dialog_template,1,"Swap Y and Z Axes"),pref_get(Mod, swap_y_z, false),
+     [{?__(1,"Swap Y and Z Axes"),pref_get(Mod, swap_y_z, false),
        [{key,swap_y_z}]},
       {label_column,
        [{"("++import_scale_s()++")",
@@ -232,16 +210,56 @@ dialog_template(Mod, export) ->
 	{export_scale_s(),
 	 {text,pref_get(Mod, export_scale, 1.0),
 	  [{key,export_scale}]}},
-	{?STR(dialog_template,6,"Sub-division Steps"),
+	{?__(6,"Sub-division Steps"),
 	 {text,pref_get(Mod, subdivisions, 0),
 	  [{key,subdivisions},{range,0,4}]}}]},
       panel,
+      {?__(norms, "Export normals/smoothing groups"),  
+       pref_get(Mod, include_normals, true), [{key,include_normals}]},
+      {?__(uv,    "Export UV coordinates"),
+       pref_get(Mod, include_uvs, true), [{key,include_uvs}]},
+      {?__(color, "Export vertex colors"),
+       pref_get(Mod, include_colors, true), [{key,include_colors}]},
+      panel,
       {vframe,
        [{menu,FileTypes,DefFileType,[{key,default_filetype}]}],
-       [{title,?STR(dialog_template,7,"Default texture file type")}]} ]}.
+       [{title,?__(7,"Default texture file type")}]} ]};
+dialog_template(Mod, tesselation) ->
+    {vframe,[{vradio,[{?__(tess0,"None"),none},
+		      {?__(tess3,"Triangulation"),triangulate},
+		      {?__(tess4,"Quadrangulation"),quadrangulate}],
+	      pref_get(Mod,tesselation,none), [{key,tesselation}]}],
+     [{title,?__(tess,"Tesselation")}]}.
 
-import_scale_s() -> ?STR(import_scale_s, 1, "Import scale").
-export_scale_s() -> ?STR(export_scale_s, 1, "Export scale").
+%% dialog_template(Module, Type, ExcludeKeys) -> Template
+%%  Return a template for a standard dialog. The flags argument makes
+%%  it possible to exclude certain keys.
+%%  Module = caller's module (used as preference key)
+%%  Type = import|export
+%%  Flags = List of keys to exclude from the dialog template
+
+dialog_template(Mod, Type, Flags) ->
+    Dlg = dialog_template(Mod, Type),
+    prune_keys(Dlg, ordsets:from_list(Flags)).
+
+prune_keys({_,_,[{key,Key}|_]}=El, Exclude) ->
+    case ordsets:is_element(Key, Exclude) of
+	false -> El;
+	true -> deleted
+    end;
+prune_keys({vframe,Items}, Exclude) ->
+    {vframe,prune_list(Items, Exclude)};
+prune_keys(Other, _) -> Other.
+
+prune_list([I0|Is], Exclude) ->
+    case prune_keys(I0, Exclude) of
+	deleted -> prune_list(Is, Exclude);
+	I -> [I|prune_list(Is, Exclude)]
+    end;
+prune_list([], _) -> [].
+
+import_scale_s() -> ?__( 1, "Import scale").
+export_scale_s() -> ?__( 1, "Export scale").
 
 import_matrix(Attr) ->
     Scale = e3d_mat:scale(proplists:get_value(import_scale, Attr, 1.0)),
@@ -462,12 +480,13 @@ face_outer_vertices(Faces, We) ->
     wings_vertex:outer_partition(Faces, We).
 
 face_outer_edges(Faces, We) ->
-    wings_face_cmd:outer_edge_partition(Faces, We).
+    wings_dissolve:outer_edge_partition(Faces, We).
 
-face_dissolve(Faces, We) when is_list(Faces) ->
-    wings_face_cmd:dissolve(gb_sets:from_list(Faces), We);
 face_dissolve(Faces, We) ->
-    wings_face_cmd:dissolve(Faces, We).
+    wings_dissolve:faces(Faces, We).
+
+face_dissolve_complement(Faces, We) ->
+    wings_dissolve:complement(Faces, We).
 
 %%% Objects.
 
@@ -499,17 +518,13 @@ import_lights(Lights, St) ->
 %%%
 
 image_formats() ->
-    wings_plugin:call_ui({image,formats,[]}).
+    wings_image:image_formats().
 
 image_read(Ps) ->
-    wings_plugin:call_ui({image,read,Ps}).
+    wings_image:image_read(Ps).
 
 image_write(Ps) ->
-    case catch wings_plugin:call_ui({image,write,Ps}) of
-	{'EXIT',Reason} ->
-	    {error,{none,?MODULE,{crash,Reason}}};
-	Result -> Result
-    end.
+    wings_image:image_write(Ps).
 
 %%%
 %%% Virtual mirror.
